@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -43,7 +43,7 @@ cli.connect_session(s_uuid, password=password)
 # the next lines will resume the first session in the avail_sessions dictionary
 avail_sessions = cli.list_sessions(s_uuid)
 if avail_sessions:
-    session_info = avail_sessions.values()[0]
+    session_info = list(avail_sessions.values())[0]
     sys.stdout.write("Trying to resume session %s\n" % (session_info.name,))
     cli.resume_session(s_uuid, session_name=session_info.name)
 
@@ -56,6 +56,6 @@ if avail_sessions:
     cli.terminate_session(s_uuid)
 
 else:
-    print "No sessions to be resumed."
+    print("No sessions to be resumed.")
 
 
