@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -38,9 +38,9 @@ command  = "XFCE"
 def my_progress_bar(ps):
 
     for status in ps:
-        print '---------------'
-        print 'SESSION STATUS: ' + '#' * status + "(" + str(status) + "%%)"
-        print '---------------'
+        print('---------------')
+        print('SESSION STATUS: ' + '#' * status + "(" + str(status) + "%%)")
+        print('---------------')
 
 password = getpass.getpass()
 
