@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -22,6 +22,9 @@ Monkey Patch and feature map for Python Paramiko
 
 """
 
+__package__ = 'x2go'
+__name__    = 'x2go._paramiko'
+
 import paramiko
 import re
 try:
@@ -29,7 +32,7 @@ try:
 except ImportError:
     SSH_PORT=22
 import platform
-from utils import compare_versions
+from x2go.utils import compare_versions
 
 PARAMIKO_VERSION = paramiko.__version__.split()[0]
 PARAMIKO_FEATURE = {
@@ -50,13 +53,12 @@ def _SSHClient_save_host_keys(self, filename):
 
     Save the host keys back to a file.
     Only the host keys loaded with
-    L{load_host_keys} (plus any added directly) will be saved -- not any
-    host keys loaded with L{load_system_host_keys}.
+    :func:`load_host_keys()` (plus any added directly) will be saved -- not any
+    host keys loaded with :func:`load_system_host_keys()`.
 
-    @param filename: the filename to save to
-    @type filename: str
-
-    @raise IOError: if the file could not be written
+    :param filename: the filename to save to
+    :type filename: str
+    :raises IOError: if the file could not be written
 
     """
     # update local host keys from file (in case other SSH clients
@@ -67,9 +69,9 @@ def _SSHClient_save_host_keys(self, filename):
     f = open(filename, 'w')
     #f.write('# SSH host keys collected by paramiko\n')
     _host_keys = self.get_host_keys()
-    for hostname, keys in _host_keys.iteritems():
+    for hostname, keys in list(_host_keys.items()):
 
-        for keytype, key in keys.iteritems():
+        for keytype, key in list(keys.items()):
             f.write('%s %s %s\n' % (hostname, keytype, key.get_base64()))
 
     f.close()
@@ -82,16 +84,15 @@ def _HostKeys_load(self, filename):
     Read a file of known SSH host keys, in the format used by openssh.
     This type of file unfortunately doesn't exist on Windows, but on
     posix, it will usually be stored in
-    C{os.path.expanduser("~/.ssh/known_hosts")}.
+    ``os.path.expanduser("~/.ssh/known_hosts")``.
 
     If this method is called multiple times, the host keys are merged,
-    not cleared. So multiple calls to C{load} will just call L{add},
+    not cleared. So multiple calls to ``load`` will just call :func:`add()`,
     replacing any existing entries and adding new ones.
 
-    @param filename: name of the file to read host keys from
-    @type filename: str
-
-    @raise IOError: if there was an error reading the file
+    :param filename: name of the file to read host keys from
+    :type filename: str
+    :raises IOError: if there was an error reading the file
 
     """
     f = open(filename, 'r')
@@ -113,14 +114,16 @@ def _HostKeys_load(self, filename):
 def _HostKeys_add(self, hostname, keytype, key, hash_hostname=True):
     """\
     Add a host key entry to the table. Any existing entry for a
-    C{(hostname, keytype)} pair will be replaced.
+    ``(hostname, keytype)`` pair will be replaced.
 
-    @param hostname: the hostname (or IP) to add
-    @type hostname: str
-    @param keytype: key type (C{"ssh-rsa"}, C{"ssh-dss"} or C{"ecdsa-sha2-nistp256"})
-    @type keytype: str
-    @param key: the key to add
-    @type key: L{PKey}
+    :param hostname: the hostname (or IP) to add
+    :type hostname: str
+    :param keytype: key type (``"ssh-rsa"``, ``"ssh-dss"`` or ``"ecdsa-sha2-nistp256"``)
+    :type keytype: str
+    :param key: the key to add
+    :type key: :class:`PKey`
+    :param hash_hostname: hash the system's hostname (Default value = True)
+    :type hash_hostname: ``bool``
 
     """
     # IPv4 and IPv6 addresses using the SSH default port need to be stripped off the port number
