@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -17,11 +17,5 @@
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from x2go.defaults import BACKEND_SERVERSESSIONINFO_DEFAULT
-from x2go.defaults import BACKEND_SERVERSESSIONLIST_DEFAULT
-
-from _stdout import X2GoServerSessionInfoSTDOUT
-from _stdout import X2GoServerSessionListSTDOUT
-
-X2GoServerSessionInfo = eval(BACKEND_SERVERSESSIONINFO_DEFAULT)
-X2GoServerSessionList = eval(BACKEND_SERVERSESSIONLIST_DEFAULT)
+__package__ = 'x2go.backends.info'
+__name__    = 'x2go.backends.info'
