@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -17,11 +17,5 @@
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from x2go.defaults import BACKEND_CLIENTPRINTING_DEFAULT
-
-from _file import X2GoClientPrintingFILE
-from _winreg import X2GoClientPrintingWINREG
-from _gconf import X2GoClientPrintingGCONF
-
-X2GoClientPrinting = eval(BACKEND_CLIENTPRINTING_DEFAULT)
-
+__package__ = 'x2go.backends.printing'
+__name__    = 'x2go.backends.printing'
