@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -18,14 +18,17 @@
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-L{X2GoSessionProfiles} class - managing x2goclient session profiles.
+:class:`x2go.backends.profiles.file.X2GoSessionProfiles` class - managing X2Go Client session profiles read from a file (``~/.x2goclient/sessions``).
 
-L{X2GoSessionProfiles} is a public API class. Use this class in your Python X2Go based
+:class:`x2go.backends.profiles.base.X2GoSessionProfiles` is a public API class. Use this class in your Python X2Go based
 applications.
 
 """
 __NAME__ = 'x2gosessionprofiles-pylib'
 
+__package__ = 'x2go.backends.profiles'
+__name__ = 'x2go.backends.profiles.file'
+
 import random
 
 # Python X2Go modules
@@ -38,18 +41,18 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles, inifiles.X2GoIniFile):
 
     def __init__(self, config_files=_X2GO_SESSIONPROFILES_CONFIGFILES, session_profile_defaults=None, logger=None, loglevel=log.loglevel_DEFAULT, **kwargs):
         """\
-        Retrieve X2Go session profiles from a file, typically C{~/.x2goclient/sessions}.
+        Retrieve X2Go session profiles from a file, typically ``~/.x2goclient/sessions``.
 
-        @param config_files: a list of config file locations, the first file name in this list the user has write access to will be the user configuration file
-        @type config_files: C{list}
-        @param session_profile_defaults: a default session profile
-        @type session_profile_defaults: C{dict}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{x2go.backends.profiles.file.X2GoSessionProfiles} constructor
-        @type logger: L{X2GoLogger} instance
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
-            constructed with the given loglevel
-        @type loglevel: C{int}
+        :param config_files: a list of config file locations, the first file name in this list the user has write access to will be the user configuration file
+        :type config_files: ``list``
+        :param session_profile_defaults: a default session profile
+        :type session_profile_defaults: ``dict``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+                    :class:`x2go.backends.profiles.file.X2GoSessionProfiles` constructor
+        :type logger: :class:`x2go.log.X2GoLogger` instance
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
+                    constructed with the given loglevel
+        :type loglevel: ``int``
 
         """
         # providing defaults for an X2GoSessionProfiles instance will---in the worst case---override your
@@ -62,13 +65,12 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles, inifiles.X2GoIniFile):
         Override the inifile class's get_type method due to the special layout of the session profile
         class.
 
-        @param section: INI file section
-        @type section: C{str}
-        @param key: key in INI file section
-        @type key: C{str}
-
-        @return: the data type of C{key} in C{section}
-        @rtype: C{type}
+        :param section: INI file section
+        :type section: ``str``
+        :param key: key in INI file section
+        :type key: ``str``
+        :returns: the data type of ``key`` in ``section``
+        :rtype: ``type``
 
         """
         # we have to handle the get_type method separately...
@@ -79,14 +81,16 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles, inifiles.X2GoIniFile):
         Populate the set of session profiles by loading the session
         profile configuration from a file in INI format.
 
-        @return: a set of session profiles
-        @rtype: C{dict}
+
+        :returns: a set of session profiles
+
+        :rtype: ``dict``
 
         """
         session_profiles = [ p for p in self.iniConfig.sections() if p not in self._non_profile_sections and p != 'none' ]
         _session_profiles_dict = {}
         for session_profile in session_profiles:
-            for key, default_value in self.defaultSessionProfile.iteritems():
+            for key, default_value in list(self.defaultSessionProfile.items()):
                 if not self.iniConfig.has_option(session_profile, key):
                     self._storeValue(session_profile, key, default_value)
             # update cached meta type session profile information
