@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -18,14 +18,18 @@
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-L{X2GoSessionProfiles} class - managing x2goclient session profiles.
+:class:`x2go.backends.profiles.httpbroker.X2GoSessionProfiles` class - managing X2Go Client session profiles obtain from an HTTP based X2Go Session Broker.
 
-L{X2GoSessionProfiles} is a public API class. Use this class in your Python X2Go based
+:class:`x2go.backends.profiles.httpbroker.X2GoSessionProfiles` is a public API class. Use this class in your Python X2Go based
 applications.
 
 """
+from __future__ import print_function
 __NAME__ = 'x2gosessionprofiles-pylib'
 
+__package__ = 'x2go.backends.profiles'
+__name__ = 'x2go.backends.profiles.httpbroker'
+
 import re
 import requests
 import urllib3.exceptions
@@ -56,19 +60,19 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
         """\
         Retrieve X2Go session profiles from a HTTP(S) session broker.
 
-        @param session_profile_defaults: a default session profile
-        @type session_profile_defaults: C{dict}
-        @param broker_url: URL for accessing the X2Go Session Broker
-        @type broker_url: C{str}
-        @param broker_password: use this password for authentication against the X2Go Session Broker (avoid
-            password string in the C{broker_URL} parameter is highly recommended)
-        @type broker_password: C{str}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{x2go.backends.profiles.httpbroker.X2GoSessionProfiles} constructor
-        @type logger: L{X2GoLogger} instance
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param session_profile_defaults: a default session profile
+        :type session_profile_defaults: ``dict``
+        :param broker_url: URL for accessing the X2Go Session Broker
+        :type broker_url: ``str``
+        :param broker_password: use this password for authentication against the X2Go Session Broker (avoid
+            password string in the ``broker_URL`` parameter is highly recommended)
+        :type broker_password: ``str``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.backends.profiles.httpbroker.X2GoSessionProfiles` constructor
+        :type logger: :class:`x2go.log.X2GoLogger` instance
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if broker_url.upper() != "HTTP":
@@ -89,6 +93,8 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
             p['path'] = "/{path}".format(**p)
             if p['port'] is not None:
                 p['port'] = ":{port}".format(**p)
+            else:
+                p['port'] = ''
 
             self.broker_url = "{protocol}://{hostname}{port}{path}".format(**p)
 
@@ -114,45 +120,50 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
 
     def get_broker_noauth(self):
         """\
-        Accessor for the class's C{broker_noauth} property.
+        Accessor for the class's ``broker_noauth`` property.
 
-        @return: C{True} if the broker probably does not expect authentication.
-        @rtype: C{bool}
+
+        :returns: ``True`` if the broker probably does not expect authentication.
+
+        :rtype: ``bool``
 
         """
         return self.broker_noauth
 
     def get_broker_username(self):
         """\
-        Accessor for the class's C{broker_username} property.
+        Accessor for the class's ``broker_username`` property.
 
-        @return: the username used for authentication against the session broker URL
-        @rtype: C{str}
+
+        :returns: the username used for authentication against the session broker URL
+
+        :rtype: ``str``
 
         """
         return self.broker_username
 
     def get_broker_url(self):
         """\
-        Accessor for the class's C{broker_url} property.
+        Accessor for the class's ``broker_url`` property.
 
-        @return: the session broker URL that was used at broker session instantiation
-        @rtype: C{str}
+
+        :returns: the session broker URL that was used at broker session instantiation
+
+        :rtype: ``str``
 
         """
         return self.broker_url
 
     def set_broker_url(self, broker_url):
         """\
-        Mutator for the class's C{broker_url} property.
+        Mutator for the class's ``broker_url`` property.
 
-        @param broker_url: A new broker URL to use with this instance. Format is
-            C{<protocol>://<hostname>:<port>/<path>} (where protocol has to be C{http}
-            or C{https}.
-        @type broker_url: C{str}
-
-        @return: the session broker URL that was used at broker session instantiation
-        @rtype: C{str}
+        :param broker_url: A new broker URL to use with this instance. Format is
+            ``<protocol>://<hostname>:<port>/<path>`` (where protocol has to be ``http``
+            or ``https``.
+        :type broker_url: ``str``
+        :returns: the session broker URL that was used at broker session instantiation
+        :rtype: ``str``
 
         """
         self.broker_url = broker_url
@@ -161,8 +172,10 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
         """\
         Accessor of the class's {_broker_type} property.
 
-        @return: either C{http} or C{https}.
-        @rtype: C{str}
+
+        :returns: either ``http`` or ``https``.
+
+        :rtype: ``str``
 
         """
         return self._broker_type
@@ -172,15 +185,13 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
         Attempt a username / password authentication against the instance's
         broker URL.
 
-        @param broker_username: username to use for authentication
-        @type broker_username: C{str}
-        @param broker_password: password to use for authentication
-        @type broker_password: C{str}
-
-        @return: C{True} if authentication has been successful
-        @rtype: C{bool}
-
-        @raise X2GoBrokerConnectionException: Raised on any kind of connection /
+        :param broker_username: username to use for authentication
+        :type broker_username: ``str``
+        :param broker_password: password to use for authentication
+        :type broker_password: ``str``
+        :returns: ``True`` if authentication has been successful
+        :rtype: ``bool``
+        :raises X2GoBrokerConnectionException: Raised on any kind of connection /
             authentication failure.
 
         """
@@ -206,7 +217,7 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
                 payload = json.loads(r.text)
                 if not self.broker_authid and not self.broker_password:
                     self.broker_noauth = True
-                elif payload.has_key('next-authid'):
+                elif 'next-authid' in payload:
                     self.broker_authid = payload['next-authid']
                 self.broker_username = broker_username or ''
                 self.broker_password = broker_password or ''
@@ -225,6 +236,7 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
         this instance has to re-authenticate against / re-connect to the
         session broker before any new interaction with the broker is possible.
 
+
         """
         _profile_ids = copy.deepcopy(self.profile_ids)
 
@@ -251,9 +263,11 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
         will be attempted. If that fails, user credentials are not provided /
         valid.
 
-        @return: C{True} if the broker session has already been authenticated
+
+        :returns: ``True`` if the broker session has already been authenticated
             and user credentials are known / valid
-        @rtype: C{bool}
+
+        :rtype: ``bool``
 
         """
         if self._broker_auth_successful is None:
@@ -268,8 +282,10 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
         """\
         Obtain a session profile list from the X2Go Session Broker.
 
-        @return: session profiles as a Python dictionary.
-        @rtype: C{dict}
+
+        :returns: session profiles as a Python dictionary.
+
+        :rtype: ``dict``
 
         """
         if self.broker_url is not None:
@@ -293,9 +309,9 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
                 raise x2go.x2go_exceptions.X2GoBrokerConnectionException('Failed to connect to URL %s' % self.broker_url)
             if r.status_code == 200 and r.headers['content-type'].startswith("text/json"):
                 payload = json.loads(r.text)
-                if payload.has_key('next-authid'):
+                if 'next-authid' in payload:
                     self.broker_authid = payload['next-authid']
-                if payload.has_key('mutable_profile_ids'):
+                if 'mutable_profile_ids' in payload:
                     self._mutable_profile_ids = payload['mutable_profile_ids']
                 self._broker_auth_successful = True
                 return payload['profiles'] if payload['task'] == 'listprofiles' else {}
@@ -306,20 +322,19 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
     def broker_selectsession(self, profile_id):
         """\
         Select a session from the list of available session profiles (presented by
-        L{broker_listprofiles}). This method requests a session information dictionary
+        :func:`broker_listprofiles()`). This method requests a session information dictionary
         (server, port, SSH keys, already running / suspended sessions, etc.) from the
-        session broker for the provided C{profile_id}.
+        session broker for the provided ``profile_id``.
 
-        @param profile_id: profile ID of the selected session profile
-        @type profile_id: C{str}
-
-        @return: session information (server, port, SSH keys, etc.) for a selected
-            session profile (i.e. C{profile_id})
-        @rtype: C{dict}
+        :param profile_id: profile ID of the selected session profile
+        :type profile_id: ``str``
+        :returns: session information (server, port, SSH keys, etc.) for a selected
+            session profile (i.e. ``profile_id``)
+        :rtype: ``dict``
 
         """
         if self.broker_url is not None:
-            if not self._broker_profile_cache.has_key(profile_id) or not self._broker_profile_cache[profile_id]:
+            if profile_id not in self._broker_profile_cache or not self._broker_profile_cache[profile_id]:
                 request_data = {
                     'task': 'selectsession',
                     'profile-id': profile_id,
@@ -342,7 +357,7 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
                     raise x2go.x2go_exceptions.X2GoBrokerConnectionException('Failed to connect to URL %s' % self.broker_url)
                 if r.status_code == 200 and r.headers['content-type'].startswith("text/json"):
                     payload = json.loads(r.text)
-                    if payload.has_key('next-authid'):
+                    if 'next-authid' in payload:
                         self.broker_authid = payload['next-authid']
                     self._broker_profile_cache[profile_id] = payload['selected_session'] if payload['task'] == 'selectsession' else {}
                     self._broker_auth_successful = True
@@ -354,16 +369,18 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
         return {}
 
     def _init_profile_cache(self, profile_id):
-        if self._broker_profile_cache.has_key(unicode(profile_id)):
-            del self._broker_profile_cache[unicode(profile_id)]
+        if str(profile_id) in self._broker_profile_cache:
+            del self._broker_profile_cache[str(profile_id)]
 
     def _populate_session_profiles(self):
         """\
         Populate the set of session profiles by loading the session
         profile configuration from a file in INI format.
 
-        @return: a set of session profiles
-        @rtype: C{dict}
+
+        :returns: a set of session profiles
+
+        :rtype: ``dict``
 
         """
         if self.is_broker_authenticated() and \
@@ -374,12 +391,12 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
             _session_profiles = copy.deepcopy(session_profiles)
 
             for session_profile in _session_profiles:
-                session_profile = unicode(session_profile)
-                for key, default_value in self.defaultSessionProfile.iteritems():
-                    key = unicode(key)
-                    if type(default_value) is types.StringType:
-                        default_value = unicode(default_value)
-                    if not session_profiles[session_profile].has_key(key):
+                session_profile = str(session_profile)
+                for key, default_value in list(self.defaultSessionProfile.items()):
+                    key = str(key)
+                    if type(default_value) is bytes:
+                        default_value = str(default_value)
+                    if key not in session_profiles[session_profile]:
                         session_profiles[session_profile][key] = default_value
 
         else:
@@ -388,35 +405,35 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
         return session_profiles
 
     def _is_mutable(self, profile_id):
-        if type(self._mutable_profile_ids) is types.ListType and profile_id in self._mutable_profile_ids:
+        if type(self._mutable_profile_ids) is list and profile_id in self._mutable_profile_ids:
             return True
         return False
 
     def _supports_mutable_profiles(self):
-        if type(self._mutable_profile_ids) is types.ListType:
+        if type(self._mutable_profile_ids) is list:
             return True
         return False
 
     def _write(self):
-        print "not suported, yet"
+        print("not suported, yet")
 
     def _delete_profile(self, profile_id):
-        del self.session_profiles[unicode(profile_id)]
+        del self.session_profiles[str(profile_id)]
 
     def _update_value(self, profile_id, option, value):
-        if type(value) is types.StringType:
-            value = unicode(value)
-        self.session_profiles[unicode(profile_id)][unicode(option)] = value
+        if type(value) is bytes:
+            value = str(value)
+        self.session_profiles[str(profile_id)][str(option)] = value
 
     def _get_profile_parameter(self, profile_id, option, key_type):
-        return key_type(self.session_profiles[unicode(profile_id)][unicode(option)])
+        return key_type(self.session_profiles[str(profile_id)][str(option)])
 
     def _get_profile_options(self, profile_id):
-        return self.session_profiles[unicode(profile_id)].keys()
+        return list(self.session_profiles[str(profile_id)].keys())
 
     def _get_profile_ids(self):
-        self.session_profiles.keys()
-        return self.session_profiles.keys()
+        list(self.session_profiles.keys())
+        return list(self.session_profiles.keys())
 
     def _get_server_hostname(self, profile_id):
         selected_session = self.broker_selectsession(profile_id)
@@ -428,7 +445,7 @@ class X2GoSessionProfiles(base.X2GoSessionProfiles):
 
     def _get_pkey_object(self, profile_id):
         selected_session = self.broker_selectsession(profile_id)
-        if selected_session.has_key('authentication_pubkey') and selected_session['authentication_pubkey'] == 'ACCEPTED':
+        if 'authentication_pubkey' in selected_session and selected_session['authentication_pubkey'] == 'ACCEPTED':
             time.sleep(2)
             return self.broker_my_privkey
         return None
