@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2014 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -17,8 +17,5 @@
 # Free Software Foundation, Inc.,
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from x2go.defaults import BACKEND_PROXY_DEFAULT
-
-from _nx3 import X2GoProxyNX3
-
-X2GoProxy = eval(BACKEND_PROXY_DEFAULT)
+__package__ = 'x2go.backends.proxy'
+__name__    = 'x2go.backends.proxy'
