@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,6 +23,9 @@ X2GoProxy class - proxying your connection through NX3 and others.
 """
 __NAME__ = 'x2goproxy-pylib'
 
+__package__ = 'x2go.backends.proxy'
+__name__    = 'x2go.backends.proxy.base'
+
 # modules
 import gevent
 import os
@@ -52,7 +55,7 @@ class X2GoProxy(threading.Thread):
     X2GoProxy is an abstract class for X2Go proxy connections.
 
     This class needs to be inherited from a concrete proxy class. Only
-    currently available proxy class is: L{x2go.backends.proxy.nx3.X2GoProxy}.
+    currently available proxy class is: :class:`x2go.backends.proxy.nx3.X2GoProxy`.
 
     """
     PROXY_CMD = ''
@@ -75,26 +78,26 @@ class X2GoProxy(threading.Thread):
                  session_instance=None,
                  logger=None, loglevel=log.loglevel_DEFAULT, ):
         """\
-        @param session_info: session information provided as an C{X2GoServerSessionInfo*} backend
+        :param session_info: session information provided as an ``X2GoServerSessionInfo*`` backend
             instance
-        @type session_info: C{X2GoServerSessionInfo*} instance
-        @param ssh_transport: SSH transport object from C{paramiko.SSHClient}
-        @type ssh_transport: C{paramiko.Transport} instance
-        @param session_log: name of the proxy's session logfile
-        @type session_log: C{str}
-        @param sessions_rootdir: base dir where X2Go session files are stored (by default: ~/.x2go)
-        @type sessions_rootdir: C{str}
-        @param proxy_options: a set of very L{base.X2GoProxy} backend specific options; any option that is not known
-            to the L{base.X2GoProxy} backend will simply be ignored
-        @type proxy_options: C{dict}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{base.X2GoProxy} constructor
-        @param session_instance: the L{X2GoSession} instance this L{base.X2GoProxy} instance belongs to
-        @type session_instance: L{X2GoSession} instance
-        @type logger: L{X2GoLogger} instance
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :type session_info: ``X2GoServerSessionInfo*`` instance
+        :param ssh_transport: SSH transport object from ``paramiko.SSHClient``
+        :type ssh_transport: ``paramiko.Transport`` instance
+        :param session_log: name of the proxy's session logfile
+        :type session_log: ``str``
+        :param sessions_rootdir: base dir where X2Go session files are stored (by default: ~/.x2go)
+        :type sessions_rootdir: ``str``
+        :param proxy_options: a set of very :class:`X2GoProxy <x2go.backends.proxy.base.X2GoProxy>` backend specific options; any option that is not known
+            to the :class:`x2go.backends.proxy.base.X2GoProxy` backend will simply be ignored
+        :type proxy_options: ``dict``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.backends.proxy.base.X2GoProxy` constructor
+        :param session_instance: the :class:`x2go.session.X2GoSession` instance this :class:`x2go.backends.proxy.base.X2GoProxy` instance belongs to
+        :type session_instance: :class:`x2go.session.X2GoSession` instance
+        :type logger: :class:`x2go.log.X2GoLogger` instance
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: int
+        :type loglevel: int
 
         """
         if logger is None:
@@ -130,12 +133,13 @@ class X2GoProxy(threading.Thread):
         Close any left open port forwarding tunnel, also close session log file,
         if left open.
 
+
         """
         if self.proxy:
             self.logger('Shutting down X2Go proxy subprocess', loglevel=log.loglevel_DEBUG)
             try:
                 self.proxy.kill()
-            except OSError, e:
+            except OSError as e:
                 self.logger('X2Go proxy shutdown gave a message that we may ignore: %s' % str(e), loglevel=log.loglevel_WARN)
             self.proxy = None
         if self.fw_tunnel is not None:
@@ -172,7 +176,7 @@ class X2GoProxy(threading.Thread):
 
         try:
             os.makedirs(self.session_info.local_container)
-        except OSError, e:
+        except OSError as e:
             if e.errno == 17:
                 # file exists
                 pass
@@ -230,7 +234,7 @@ class X2GoProxy(threading.Thread):
         try:
             p.terminate()
             self.logger('terminating proxy: %s' % p, loglevel=log.loglevel_DEBUG)
-        except OSError, e:
+        except OSError as e:
             if e.errno == 3:
                 # No such process
                 pass
@@ -241,7 +245,7 @@ class X2GoProxy(threading.Thread):
 
     def process_proxy_options(self):
         """\
-        Override this method to incorporate elements from C{proxy_options}
+        Override this method to incorporate elements from ``proxy_options``
         into actual proxy subprocess execution.
 
         This method (if overridden) should (by design) never fail nor raise an exception.
@@ -251,7 +255,7 @@ class X2GoProxy(threading.Thread):
 
             1. remove processed proxy_options from self.proxy_options
             2. once you have finished processing the proxy_options call
-            the parent class method L{x2go.backends.proxy.base.X2GoProxy.process_proxy_options()}
+               the parent class method :class:`x2go.backends.proxy.base.X2GoProxy.process_proxy_options()`
 
         """
         # do the logging of remaining options
@@ -268,8 +272,10 @@ class X2GoProxy(threading.Thread):
         """\
         Start the thread runner and wait for the proxy to come up.
 
-        @return: a subprocess instance that knows about the externally started proxy command.
-        @rtype: C{obj}
+
+        :returns: a subprocess instance that knows about the externally started proxy command.
+
+        :rtype: ``obj``
 
         """
         threading.Thread.start(self)
@@ -298,8 +304,10 @@ class X2GoProxy(threading.Thread):
         """\
         Check if a proxy instance is up and running.
 
-        @return: Proxy state, C{True} for proxy being up-and-running, C{False} otherwise
-        @rtype C{bool}
+
+        :returns: Proxy state, ``True`` for proxy being up-and-running, ``False`` otherwise
+
+        :rtype: ``bool``
 
         """
         return bool(self.proxy and self.proxy.poll() is None) and self.fw_tunnel.is_active
