@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,23 +23,26 @@ X2GoListSessionCache class - caching X2Go session information.
 """
 __NAME__ = 'x2gocache-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.cache'
+
 # modules
 import copy
 import gevent
 
 # Python X2Go modules
-import log
-import x2go_exceptions
+from . import log
+from . import x2go_exceptions
 
 class X2GoListSessionsCache(object):
     """\
     For non-blocking operations in client applications using Python X2Go, it is
-    recommended to enable the L{X2GoListSessionsCache}. This can be done by calling
-    the constructor of the L{X2GoClient} class.
+    recommended to enable the :class:`x2go.cache.X2GoListSessionsCache`. This can be done by calling
+    the constructor of the :class:`x2go.client.X2GoClient` class.
 
     The session list and desktop cache gets updated in regular intervals by a threaded
-    L{X2GoSessionGuardian} instance. For the session list and desktop list update, the
-    X2Go server commands C{x2golistsessions} and C{x2godesktopsessions} are called and
+    :class:`x2go.guardian.X2GoSessionGuardian` instance. For the session list and desktop list update, the
+    X2Go server commands ``x2golistsessions`` and ``x2godesktopsessions`` are called and
     the command's stdout is cached in the session list cache.
 
     Whenever your client application needs access to either the server's session list
@@ -47,18 +50,19 @@ class X2GoListSessionsCache(object):
     the server's session/desktop list is available without delay, even on slow internet
     connections.
 
+
     """
     x2go_listsessions_cache = {}
 
     def __init__(self, client_instance, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: the L{X2GoClient} instance that uses this L{X2GoListSessionsCache}
-        @type client_instance: C{obj}
-        @param logger: you can pass an L{X2GoLogger} object to the L{X2GoListSessionsCache} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the :class:`x2go.client.X2GoClient` instance that uses this :class:`x2go.cache.X2GoListSessionsCache`
+        :type client_instance: ``obj``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the :class:`x2go.cache.X2GoListSessionsCache` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         self.x2go_listsessions_cache = {}
@@ -77,8 +81,8 @@ class X2GoListSessionsCache(object):
         """\
         Remove session list from cache for a given profile.
 
-        @param profile_name: name of profile to operate on
-        @type profile_name: C{str}
+        :param profile_name: name of profile to operate on
+        :type profile_name: ``str``
 
         """
         while self.protected:
@@ -92,19 +96,20 @@ class X2GoListSessionsCache(object):
         session profiles are still connected). If not so, remove invalid cache entries from
         the session list cache.
 
+
         """
-        for profile_name in self.x2go_listsessions_cache.keys():
+        for profile_name in list(self.x2go_listsessions_cache.keys()):
             if profile_name not in self.client_instance.client_connected_profiles(return_profile_names=True):
                 del self.x2go_listsessions_cache[profile_name]
 
     def update_all(self, update_sessions=True, update_desktops=False):
         """\
-        Update L{X2GoListSessionsCache} for all connected session profiles.
+        Update :class:`x2go.cache.X2GoListSessionsCache` for all connected session profiles.
 
-        @param update_sessions: cache recent session lists from all connected servers
-        @type update_sessions: C{bool}
-        @param update_desktops: cache recent desktop lists from all connected servers
-        @type update_desktops: C{bool}
+        :param update_sessions: cache recent session lists from all connected servers (Default value = True)
+        :type update_sessions: ``bool``
+        :param update_desktops: cache recent desktop lists from all connected servers (Default value = False)
+        :type update_desktops: ``bool``
 
         """
         for profile_name in self.client_instance.client_connected_profiles(return_profile_names=True):
@@ -114,22 +119,22 @@ class X2GoListSessionsCache(object):
 
     def update(self, profile_name, update_sessions=True, update_desktops=False, update_mounts=False):
         """\
-        Update L{X2GoListSessionsCache} (i.e. session/desktops) for session profile C{profile_name}.
+        Update :class:`x2go.cache.X2GoListSessionsCache` (i.e. session/desktops) for session profile ``profile_name``.
 
-        @param profile_name: name of profile to update
-        @type profile_name: C{str}
-        @param update_sessions: cache recent session list from server
-        @type update_sessions: C{bool}
-        @param update_desktops: cache recent desktop list from server
-        @type update_desktops: C{bool}
-        @param update_mounts: cache list of client-side mounts on server
-        @type update_mounts: C{bool}
+        :param profile_name: name of profile to update
+        :type profile_name: ``str``
+        :param update_sessions: cache recent session list from server (Default value = True)
+        :type update_sessions: ``bool``
+        :param update_desktops: cache recent desktop list from server (Default value = False)
+        :type update_desktops: ``bool``
+        :param update_mounts: cache list of client-side mounts on server (Default value = False)
+        :type update_mounts: ``bool``
 
         """
         self.protected = True
         self.last_listsessions_cache = copy.deepcopy(self.x2go_listsessions_cache)
         control_session = self.client_instance.client_control_session_of_profile_name(profile_name)
-        if not self.x2go_listsessions_cache.has_key(profile_name):
+        if profile_name not in self.x2go_listsessions_cache:
             self.x2go_listsessions_cache[profile_name] = {'sessions': None, 'desktops': None, 'mounts': {}, }
         if update_sessions:
             self._update_sessions(profile_name, control_session)
@@ -141,12 +146,13 @@ class X2GoListSessionsCache(object):
 
     def _update_mounts(self, profile_name, control_session):
         """\
-        Update mounts list of L{X2GoListSessionsCache} for session profile C{profile_name}.
+        Update mounts list of :class:`x2go.cache.X2GoListSessionsCache` for session profile ``profile_name``.
 
-        @param profile_name: name of profile to update
-        @type profile_name: C{str}
+        :param profile_name: name of profile to update
+        :type profile_name: ``str``
+        :param control_session: X2Go control session instance
+        :raises X2GoControlSessionException: if the control session's ``list_mounts`` method fails
 
-        @raise X2GoControlSessionException: if the control session's C{list_mounts} method fails
         """
         try:
             self.x2go_listsessions_cache[profile_name]['mounts'] = {}
@@ -156,8 +162,8 @@ class X2GoListSessionsCache(object):
                     if session is not None and session.is_running():
                         if control_session is not None and not control_session.has_session_died():
                             self.x2go_listsessions_cache[profile_name]['mounts'].update(control_session.list_mounts(session_name))
-        except (x2go_exceptions.X2GoControlSessionException, AttributeError), e:
-            if profile_name in self.x2go_listsessions_cache.keys():
+        except (x2go_exceptions.X2GoControlSessionException, AttributeError) as e:
+            if profile_name in list(self.x2go_listsessions_cache.keys()):
                 del self.x2go_listsessions_cache[profile_name]
             self.protected = False
             raise x2go_exceptions.X2GoControlSessionException(str(e))
@@ -168,20 +174,20 @@ class X2GoListSessionsCache(object):
 
     def _update_desktops(self, profile_name, control_session):
         """\
-        Update session lists of L{X2GoListSessionsCache} for session profile C{profile_name}.
+        Update session lists of :class:`x2go.cache.X2GoListSessionsCache` for session profile ``profile_name``.
 
-        @param profile_name: name of profile to update
-        @type profile_name: C{str}
-        @param control_session: X2Go control session instance
-        @type control_session: C{obj}
+        :param profile_name: name of profile to update
+        :type profile_name: ``str``
+        :param control_session: X2Go control session instance
+        :type control_session: ``obj``
+        :raises X2GoControlSessionException: if the control session's ``list_desktop`` method fails
 
-        @raise X2GoControlSessionException: if the control session's C{list_desktop} method fails
         """
         try:
             if control_session is not None and not control_session.has_session_died():
                 self.x2go_listsessions_cache[profile_name]['desktops'] = control_session.list_desktops()
-        except (x2go_exceptions.X2GoControlSessionException, AttributeError), e:
-            if profile_name in self.x2go_listsessions_cache.keys():
+        except (x2go_exceptions.X2GoControlSessionException, AttributeError) as e:
+            if profile_name in list(self.x2go_listsessions_cache.keys()):
                 del self.x2go_listsessions_cache[profile_name]
             self.protected = False
             raise x2go_exceptions.X2GoControlSessionException(str(e))
@@ -192,18 +198,20 @@ class X2GoListSessionsCache(object):
 
     def _update_sessions(self, profile_name, control_session):
         """\
-        Update desktop list of L{X2GoListSessionsCache} for session profile C{profile_name}.
+        Update desktop list of :class:`x2go.cache.X2GoListSessionsCache` for session profile ``profile_name``.
 
-        @param profile_name: name of profile to update
-        @type profile_name: C{str}
+        :param profile_name: name of profile to update
+        :type profile_name: ``str``
+        :param control_session: X2Go control session instance
+        :type control_session: ``obj``
+        :raises X2GoControlSessionException: if the control session's ``list_sessions`` method fails
 
-        @raise X2GoControlSessionException: if the control session's C{list_sessions} method fails
         """
         try:
             if control_session is not None and not control_session.has_session_died():
                 self.x2go_listsessions_cache[profile_name]['sessions'] = control_session.list_sessions()
-        except (x2go_exceptions.X2GoControlSessionException, AttributeError), e:
-            if profile_name in self.x2go_listsessions_cache.keys():
+        except (x2go_exceptions.X2GoControlSessionException, AttributeError) as e:
+            if profile_name in list(self.x2go_listsessions_cache.keys()):
                 del self.x2go_listsessions_cache[profile_name]
             self.protected = False
             raise x2go_exceptions.X2GoControlSessionException(str(e))
@@ -212,14 +220,13 @@ class X2GoListSessionsCache(object):
 
     def list_sessions(self, session_uuid):
         """\
-        Retrieve a session list from the current cache content of L{X2GoListSessionsCache}
-        for a given L{X2GoSession} instance (specified by its unique session UUID).
+        Retrieve a session list from the current cache content of :class:`x2go.cache.X2GoListSessionsCache`
+        for a given :class:`x2go.session.X2GoSession` instance (specified by its unique session UUID).
 
-        @param session_uuid: unique identifier of session to query cache for
-        @type session_uuid: C{str}
-
-        @return: a data object containing available session information
-        @rtype: C{X2GoServerSessionList*} instance (or C{None})
+        :param session_uuid: unique identifier of session to query cache for
+        :type session_uuid: ``str``
+        :returns: a data object containing available session information
+        :rtype: ``X2GoServerSessionList*`` instance (or ``None``)
 
         """
         profile_name = self.client_instance.get_session_profile_name(session_uuid)
@@ -231,14 +238,13 @@ class X2GoListSessionsCache(object):
     def list_desktops(self, session_uuid):
         """\
         Retrieve a list of available desktop sessions from the current cache content of
-        L{X2GoListSessionsCache} for a given L{X2GoSession} instance (specified by its
+        :class:`x2go.cache.X2GoListSessionsCache` for a given :class:`x2go.session.X2GoSession` instance (specified by its
         unique session UUID).
 
-        @param session_uuid: unique identifier of session to query cache for
-        @type session_uuid: C{str}
-
-        @return: a list of strings representing X2Go desktop sessions available for sharing
-        @rtype: C{list} (or C{None})
+        :param session_uuid: unique identifier of session to query cache for
+        :type session_uuid: ``str``
+        :returns: a list of strings representing X2Go desktop sessions available for sharing
+        :rtype: ``list`` (or ``None``)
 
         """
         profile_name = self.client_instance.get_session_profile_name(session_uuid)
@@ -250,14 +256,13 @@ class X2GoListSessionsCache(object):
     def list_mounts(self, session_uuid):
         """\
         Retrieve a list of mounted client shares from the current cache content of
-        L{X2GoListSessionsCache} for a given L{X2GoSession} instance (specified by its
+        :class:`x2go.cache.X2GoListSessionsCache` for a given :class:`x2go.session.X2GoSession` instance (specified by its
         unique session UUID).
 
-        @param session_uuid: unique identifier of session to query cache for
-        @type session_uuid: C{str}
-
-        @return: a list of strings representing mounted client shares
-        @rtype: C{list} (or C{None})
+        :param session_uuid: unique identifier of session to query cache for
+        :type session_uuid: ``str``
+        :returns: a list of strings representing mounted client shares
+        :rtype: ``list`` (or ``None``)
 
         """
         profile_name = self.client_instance.get_session_profile_name(session_uuid)
@@ -270,13 +275,14 @@ class X2GoListSessionsCache(object):
         """\
         Check if session information is cached.
 
-        @param profile_name: name of profile to update
-        @type profile_name: C{str}
-        @param session_uuid: unique identifier of session to query cache for
-        @type session_uuid: C{str}
-
-        @return: C{True} if session information is cached
-        @rtype: C{bool}
+        :param profile_name: name of profile to update (Default value = None)
+        :type profile_name: ``str``
+        :param session_uuid: unique identifier of session to query cache for (Default value = None)
+        :type session_uuid: ``str``
+        :param cache_type: cache type (e.g. 'mounts', 'desktops', 'sessions') (Default value = None)
+        :type cache_type: ``str``
+        :returns: ``True`` if session information is cached
+        :rtype: ``bool``
 
         """
         if profile_name is None and session_uuid and self.client_instance:
@@ -284,8 +290,8 @@ class X2GoListSessionsCache(object):
                 profile_name = self.client_instance.get_session_profile_name(session_uuid)
             except x2go_exceptions.X2GoSessionRegistryException:
                 raise x2go_exceptions.X2GoSessionCacheException("requested session UUID is not valid anymore")
-        _is_profile_cached = self.x2go_listsessions_cache.has_key(profile_name)
-        _is_cache_type_cached = _is_profile_cached and self.x2go_listsessions_cache[profile_name].has_key(cache_type)
+        _is_profile_cached = profile_name in self.x2go_listsessions_cache
+        _is_cache_type_cached = _is_profile_cached and cache_type in self.x2go_listsessions_cache[profile_name]
         if cache_type is None:
             return _is_profile_cached
         else:
