@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -18,19 +18,22 @@
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-Providing mechanisms to C{X2GoControlSession*} backends for checking host validity.
+Providing mechanisms to ``X2GoControlSession*`` backends for checking host validity.
 
 """
 __NAME__ = 'x2gocheckhosts-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.checkhosts'
+
 # modules
 import paramiko
 import binascii
+import sys
 
 # Python X2Go modules
-import sshproxy
-import log
-import x2go_exceptions
+from . import log
+from . import x2go_exceptions
 import random
 import string
 
@@ -38,13 +41,14 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
     """\
     Skeleton class for Python X2Go's missing host key policies.
 
+
     """
     def __init__(self, caller=None, session_instance=None, fake_hostname=None):
         """\
-        @param caller: calling instance
-        @type caller: C{class}
-        @param session_instance: an X2Go session instance
-        @type session_instance: L{X2GoSession} instance
+        :param caller: calling instance
+        :type caller: ``class``
+        :param session_instance: an X2Go session instance
+        :type session_instance: :class:`x2go.session.X2GoSession` instance
 
         """
         self.caller = caller
@@ -55,8 +59,10 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         """\
         Retrieve the Paramiko SSH/Client.
 
-        @return: the associated X2Go control session instance.
-        @rtype: C{X2GoControlSession*} instance
+
+        :returns: the associated X2Go control session instance.
+
+        :rtype: ``X2GoControlSession*`` instance
 
         """
         return self.client
@@ -65,8 +71,10 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         """\
         Retrieve the server hostname:port expression of the server to be validated.
 
-        @return: hostname:port
-        @rtype: C{str}
+
+        :returns: hostname:port
+
+        :rtype: ``str``
 
         """
         return self.fake_hostname or self.hostname
@@ -75,8 +83,10 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         """\
         Retrieve the server hostname string of the server to be validated.
 
-        @return: hostname
-        @rtype: C{str}
+
+        :returns: hostname
+
+        :rtype: ``str``
 
         """
         if ":" in self.get_hostname():
@@ -88,8 +98,10 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         """\
         Retrieve the server port of the server to be validated.
 
-        @return: port
-        @rtype: C{str}
+
+        :returns: port
+
+        :rtype: ``str``
 
         """
         if ":" in self.get_hostname():
@@ -101,8 +113,10 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         """\
         Retrieve the host key of the server to be validated.
 
-        @return: host key
-        @rtype: Paramiko/SSH key instance
+
+        :returns: host key
+
+        :rtype: Paramiko/SSH key instance
 
         """
         return self.key
@@ -111,8 +125,10 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         """\
         Retrieve the host key name of the server to be validated.
 
-        @return: host key name (RSA, DSA, ECDSA...)
-        @rtype: C{str}
+
+        :returns: host key name (RSA, DSA, ECDSA...)
+
+        :rtype: ``str``
 
         """
         return self.key.get_name().upper()
@@ -121,19 +137,26 @@ class X2GoMissingHostKeyPolicy(paramiko.MissingHostKeyPolicy):
         """\
         Retrieve the host key fingerprint of the server to be validated.
 
-        @return: host key fingerprint
-        @rtype: C{str}
+
+        :returns: host key fingerprint
+
+        :rtype: ``str``
 
         """
-        return binascii.hexlify(self.key.get_fingerprint())
+        if sys.version_info[0] >= 3:
+            return binascii.hexlify(self.key.get_fingerprint()).decode()
+        else:
+            return binascii.hexlify(self.key.get_fingerprint())
 
     def get_key_fingerprint_with_colons(self):
         """\
         Retrieve the (colonized) host key fingerprint of the server
         to be validated.
 
-        @return: host key fingerprint (with colons)
-        @rtype: C{str}
+
+        :returns: host key fingerprint (with colons)
+
+        :rtype: ``str``
 
         """
         _fingerprint = self.get_key_fingerprint()
@@ -167,41 +190,41 @@ class X2GoInteractiveAddPolicy(X2GoMissingHostKeyPolicy):
     """\
     Policy for making host key information available to Python X2Go after a
     Paramiko/SSH connect has been attempted. This class needs information
-    about the associated L{X2GoSession} instance.
+    about the associated :class:`x2go.session.X2GoSession` instance.
 
-    Once called, the L{missing_host_key} method of this class will try to call
-    L{X2GoSession.HOOK_check_host_dialog()}. This hook method---if not re-defined
-    in your application---will then try to call the L{X2GoClient.HOOK_check_host_dialog()},
-    which then will return C{True} by default if not customized in your application.
+    Once called, the :func:`missing_host_key()` method of this class will try to call
+    :func:`X2GoSession.HOOK_check_host_dialog() <x2go.session.X2GoSession.HOOK_check_host_dialog()>`. This hook method---if not re-defined
+    in your application---will then try to call the :func:`X2GoClient.HOOK_check_host_dialog() <x2go.client.X2GoClient.HOOK_check_host_dialog()>`,
+    which then will return ``True`` by default if not customized in your application.
 
     To accept host key checks, make sure to either customize the
-    L{X2GoClient.HOOK_check_host_dialog()} method or the L{X2GoSession.HOOK_check_host_dialog()}
+    :func:`X2GoClient.HOOK_check_host_dialog() <x2go.client.X2GoClient.HOOK_check_host_dialog()>` method or the :func:`X2GoSession.HOOK_check_host_dialog() <x2go.session.X2GoSession.HOOK_check_host_dialog()>`
     method and hook some interactive user dialog to either of them.
 
+
     """
     def missing_host_key(self, client, hostname, key):
         """\
         Handle a missing host key situation. This method calls
 
-        Once called, the L{missing_host_key} method will try to call
-        L{X2GoSession.HOOK_check_host_dialog()}. This hook method---if not re-defined
-        in your application---will then try to call the L{X2GoClient.HOOK_check_host_dialog()},
-        which then will return C{True} by default if not customized in your application.
+        Once called, the :func:`missing_host_key()` method will try to call
+        :func:`X2GoSession.HOOK_check_host_dialog() <x2go.session.X2GoSession.HOOK_check_host_dialog()>`. This hook method---if not re-defined
+        in your application---will then try to call the :func:`X2GoClient.HOOK_check_host_dialog() <x2go.client.X2GoClient.HOOK_check_host_dialog()>`,
+        which then will return ``True`` by default if not customized in your application.
 
         To accept host key checks, make sure to either customize the
-        L{X2GoClient.HOOK_check_host_dialog()} method or the L{X2GoSession.HOOK_check_host_dialog()}
+        :func:`X2GoClient.HOOK_check_host_dialog() <x2go.client.X2GoClient.HOOK_check_host_dialog()>` method or the :func:`X2GoSession.HOOK_check_host_dialog() <x2go.session.X2GoSession.HOOK_check_host_dialog()>`
         method and hook some interactive user dialog to either of them.
 
-        @param client: SSH client (C{X2GoControlSession*}) instance
-        @type client: C{X2GoControlSession*} instance
-        @param hostname: remote hostname
-        @type hostname: C{str}
-        @param key: host key to validate
-        @type key: Paramiko/SSH key instance
-
-        @raise X2GoHostKeyException: if the X2Go server host key is not in the C{known_hosts} file
-        @raise X2GoSSHProxyHostKeyException: if the SSH proxy host key is not in the C{known_hosts} file
-        @raise SSHException: if this instance does not know its {self.session_instance}
+        :param client: SSH client (``X2GoControlSession*``) instance
+        :type client: ``X2GoControlSession*`` instance
+        :param hostname: remote hostname
+        :type hostname: ``str``
+        :param key: host key to validate
+        :type key: Paramiko/SSH key instance
+        :raises X2GoHostKeyException: if the X2Go server host key is not in the ``known_hosts`` file
+        :raises X2GoSSHProxyHostKeyException: if the SSH proxy host key is not in the ``known_hosts`` file
+        :raises SSHException: if this instance does not know its {self.session_instance}
 
         """
         self.client = client
@@ -257,17 +280,15 @@ def check_ssh_host_key(x2go_sshclient_instance, hostname, port=22):
     validating the results (i.e. by validating raised exceptions during the
     connect process).
 
-    @param x2go_sshclient_instance: a Paramiko/SSH client instance to be used for testing host key validity.
-    @type x2go_sshclient_instance: C{X2GoControlSession*} instance
-    @param hostname: hostname of server to validate
-    @type hostname: C{str}
-    @param port: port of server to validate
-    @type port: C{int}
-
-    @return: returns a tuple with the following components (<host_ok>, <hostname>, <port>, <fingerprint>, <fingerprint_type>)
-    @rtype: C{tuple}
-
-    @raise SSHException: if an SSH exception occurred, that we did not provocate in L{X2GoInteractiveAddPolicy.missing_host_key()}
+    :param x2go_sshclient_instance: a Paramiko/SSH client instance to be used for testing host key validity.
+    :type x2go_sshclient_instance: ``X2GoControlSession*`` instance
+    :param hostname: hostname of server to validate
+    :type hostname: ``str``
+    :param port: port of server to validate (Default value = 22)
+    :type port: ``int``
+    :returns: returns a tuple with the following components (<host_ok>, <hostname>, <port>, <fingerprint>, <fingerprint_type>)
+    :rtype: ``tuple``
+    :raises SSHException: if an SSH exception occurred, that we did not provocate in :func:`X2GoInteractiveAddPolicy.missing_host_key() <x2go.checkhosts.X2GoInteractiveAddPolicy.missing_host_key()`
 
     """
     _hostname = hostname
@@ -284,7 +305,7 @@ def check_ssh_host_key(x2go_sshclient_instance, hostname, port=22):
     except x2go_exceptions.AuthenticationException:
         host_ok = True
         x2go_sshclient_instance.logger('SSH host key verification for host [%s]:%s succeeded. Host is already known to the client\'s Paramiko/SSH sub-system.' % (_hostname, _port), loglevel=log.loglevel_NOTICE)
-    except x2go_exceptions.SSHException, e:
+    except x2go_exceptions.SSHException as e:
         msg = str(e)
         if msg.startswith('Policy has collected host key information on '):
             _hostname = _check_policy.get_hostname().split(':')[0].lstrip('[').rstrip(']')
