@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -22,23 +22,25 @@ A recommended X2Go session clean up helper function.
 
 """
 
+__package__ = 'x2go'
+__name__    = 'x2go.cleanup'
+
 import gevent
 import paramiko
 import threading
 
 # Python X2Go modules
-import guardian
-import rforward
-from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
+from . import rforward
+from .defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 
 if _X2GOCLIENT_OS == 'Windows':
-    import xserver
-    import pulseaudio
+    from . import xserver
+    from . import pulseaudio
 
 def x2go_cleanup(e=None, threads=None):
     """\
     For every Python X2Go application you write, please make sure to
-    capture the C{KeyboardInterrupt} and the C{SystemExit} exceptions and
+    capture the ``KeyboardInterrupt`` and the ``SystemExit`` exceptions and
     call this function if either of the exceptions occurs.
 
     Example::
@@ -54,12 +56,12 @@ def x2go_cleanup(e=None, threads=None):
         except (KeyboardInterrupt, SystemExit):
             x2go.x2go_cleanup()
 
-    @param e: if L{x2go_cleanup} got called as you caught an exception in your code this can be the
-        C{Exception} that we will process at the end of the clean-up (or if clean-up failed or was not
-        appropriate)
-    @type e: C{exception}
-    @param threads: a list of threads to clean up
-    @type threads: C{list}
+    :param e: if :func:`x2go_cleanup()` got called as you caught an exception in your code this can be the
+        ``Exception`` that we will process at the end of the clean-up (or if clean-up failed or was not
+        appropriate) (Default value = None)
+    :type e: ``exception``
+    :param threads: a list of threads to clean up (Default value = None)
+    :type threads: ``list``
 
     """
     try:
@@ -96,7 +98,9 @@ def x2go_cleanup(e=None, threads=None):
                     del t
 
         for t in threads:
-            if type(t) == guardian.X2GoSessionGuardian:
+            # now let's catch X2GoSessionGuardian, which is a bit tricky
+            # as we need to avoid circular imports
+            if hasattr(t, 'stop_thread') and hasattr(t, 'guardian'):
                 t.stop_thread()
                 del t
 
