@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,6 +23,9 @@ Default variables and values for Python X2Go.
 """
 __NAME__ = 'x2godefaults-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.defaults'
+
 import os
 import paramiko
 import platform
@@ -91,7 +94,7 @@ elif X2GOCLIENT_OS == "Mac":
 
 else:
     import exceptions
-    class OSNotSupportedException(exceptions.StandardError): pass
+    class OSNotSupportedException(exceptions.Exception): pass
     raise OSNotSupportedException('Platform %s is not supported' % platform.system())
 
 ##
@@ -125,19 +128,16 @@ BACKENDS = {
         'GCONF': 'x2go.backends.profiles.gconf',
         'HTTPBROKER': 'x2go.backends.profiles.httpbroker',
         'SSHBROKER': 'x2go.backends.profiles.sshbroker',
-        'WINREG': 'x2go.backends.profiles.winreg',
     },
     'X2GoClientSettings': {
         'default': 'FILE',
         'FILE': 'x2go.backends.settings.file',
         'GCONF': 'x2go.backends.settings.gconf',
-        'WINREG': 'x2go.backends.settings.winreg',
     },
     'X2GoClientPrinting': {
         'default': 'FILE',
         'FILE': 'x2go.backends.printing.file',
         'GCONF': 'x2go.backends.printing.gconf',
-        'WINREG': 'x2go.backends.printing.winreg',
     }
 }
 
@@ -306,7 +306,7 @@ X2GO_SESSIONPROFILE_DEFAULTS = {
     'print': False,
     'xdmcpserver': 'localhost',
 }
-"""L{X2GoSessionProfiles} default values to fill a new session profile with."""
+""":class:`x2go.backends.profiles.base.X2GoSessionProfiles` default values to fill a new session profile with."""
 ##
 ## X2Go Proxy defaults
 ##
@@ -370,6 +370,7 @@ pack_methods_nx3.sort()
 
 X2GO_DESKTOPSESSIONS={
     'CINNAMON': 'cinnamon',
+    'IceWM': 'icewm',
     'KDE': 'startkde',
     'GNOME': 'gnome-session',
     'MATE': 'mate-session',
