@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -22,6 +22,9 @@ proxying of graphical X2Go elements.
 """
 __NAME__ = "x2gofwtunnel-pylib"
 
+__package__ = 'x2go'
+__name__    = 'x2go.forward'
+
 # modules
 import copy
 
@@ -31,16 +34,17 @@ from gevent import select, socket
 from gevent.server import StreamServer
 
 # Python X2Go modules
-import log
-from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
+from . import log
+from .defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 
 class X2GoFwServer(StreamServer):
     """\
-    L{X2GoFwServer} implements a gevent's StreamServer based Paramiko/SSH port
+    :class:`x2go.forward.X2GoFwServer` implements a gevent's StreamServer based Paramiko/SSH port
     forwarding server.
 
-    An L{X2GoFwServer} class object is used to tunnel graphical trafic
-    through an external proxy command launched by a C{X2GoProxy*} backend.
+    An :class:`x2go.forward.X2GoFwServer` class object is used to tunnel graphical trafic
+    through an external proxy command launched by a ``X2GoProxy*`` backend.
+
 
     """
     def __init__ (self, listener,
@@ -48,25 +52,25 @@ class X2GoFwServer(StreamServer):
                   ssh_transport, session_instance=None, session_name=None,
                   subsystem=None, logger=None, loglevel=log.loglevel_DEFAULT,):
         """\
-        @param listener: listen on TCP/IP socket C{(<IP>, <Port>)}
-        @type listener: C{tuple}
-        @param remote_host: hostname or IP of remote host (in case of X2Go mostly 127.0.0.1)
-        @type remote_host: C{str}
-        @param remote_port: port of remote host
-        @type remote_port: C{int}
-        @param ssh_transport: a valid Paramiko/SSH transport object
-        @type ssh_transport: C{obj}
-        @param session_instance: the complete L{X2GoSession} instance of the X2Go session this port forwarding server belongs to.
-            Note: for new L{X2GoSession} instances the object has the session name not yet set(!!!)
-        @type session_instance: C{obj}
-        @param session_name: the session name of the X2Go session this port forwarding server belongs to
-        @type session_name: C{str}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoFwServer} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param listener: listen on TCP/IP socket ``(<IP>, <Port>)``
+        :type listener: ``tuple``
+        :param remote_host: hostname or IP of remote host (in case of X2Go mostly 127.0.0.1)
+        :type remote_host: ``str``
+        :param remote_port: port of remote host
+        :type remote_port: ``int``
+        :param ssh_transport: a valid Paramiko/SSH transport object
+        :type ssh_transport: ``obj``
+        :param session_instance: the complete :class:`x2go.session.X2GoSession` instance of the X2Go session this port forwarding server belongs to.
+            Note: for new :class:`x2go.session.X2GoSession` instances the object has the session name not yet set(!!!)
+        :type session_instance: ``obj``
+        :param session_name: the session name of the X2Go session this port forwarding server belongs to
+        :type session_name: ``str``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.forward.X2GoFwServer` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if logger is None:
@@ -99,10 +103,10 @@ class X2GoFwServer(StreamServer):
         """\
         Handle for SSH/Paramiko forwarding tunnel.
 
-        @param fw_socket: local end of the forwarding tunnel
-        @type fw_socket: C{obj}
-        @param address: unused/ignored
-        @type address: C{tuple}
+        :param fw_socket: local end of the forwarding tunnel
+        :type fw_socket: ``obj``
+        :param address: unused/ignored
+        :type address: ``tuple``
 
         """
         self.fw_socket = fw_socket
@@ -123,7 +127,7 @@ class X2GoFwServer(StreamServer):
                                                             self.fw_socket.getpeername())
                 chan_peername = self.chan.getpeername()
                 _success = True
-            except Exception, e:
+            except Exception as e:
                 if self.keepalive:
                     self.logger('incoming request to %s:%d failed on attempt %d of %d: %s' % (self.chain_host,
                                                                                               self.chain_port,
@@ -177,6 +181,7 @@ class X2GoFwServer(StreamServer):
         """\
         Close an open channel again.
 
+
         """
         #if self.chan is not None and _X2GOCLIENT_OS != "Windows":
         if self.chan is not None:
@@ -191,6 +196,7 @@ class X2GoFwServer(StreamServer):
         """\
         Close the forwarding tunnel's socket again.
 
+
         """
         _success = False
         _count = 0
@@ -215,6 +221,7 @@ class X2GoFwServer(StreamServer):
         """\
         Stop the forwarding tunnel.
 
+
         """
         self.is_active = False
         self.close_socket()
@@ -233,27 +240,26 @@ def start_forward_tunnel(local_host='127.0.0.1', local_port=22022,
 
     The tunnel is used to transport X2Go graphics data through a proxy application like nxproxy.
 
-    @param local_host: local starting point of the forwarding tunnel
-    @type local_host: C{int}
-    @param local_port: listen port of the local starting point
-    @type local_port: C{int}
-    @param remote_host: from the endpoint of the tunnel, connect to host C{<remote_host>}...
-    @type remote_host: C{str}
-    @param remote_port: ... on port C{<remote_port>}
-    @type remote_port: C{int}
-    @param ssh_transport: the Paramiko/SSH transport (i.e. the X2Go session's Paramiko/SSH transport object)
-    @type ssh_transport: C{obj}
-    @param session_instance: the L{X2GoSession} instance that initiates this tunnel
-    @type session_instance: C{obj}
-    @param session_name: the session name of the X2Go session this port forwarding server belongs to
-    @type session_name: C{str}
-    @param subsystem: a custom string with a component name that tries to evoke a new tunnel setup
-    @type subsystem: C{str}
-    @param logger: an X2GoLogger object
-    @type logger: C{obj}
-
-    @return: returns an L{X2GoFwServer} instance
-    @rtype: C{obj}
+    :param local_host: local starting point of the forwarding tunnel (Default value = '127.0.0.1')
+    :type local_host: ``int``
+    :param local_port: listen port of the local starting point (Default value = 22022)
+    :type local_port: ``int``
+    :param remote_host: from the endpoint of the tunnel, connect to host ``<remote_host>``... (Default value = '127.0.0.1')
+    :type remote_host: ``str``
+    :param remote_port: on port ``<remote_port>`` (Default value = 22)
+    :type remote_port: ``int``
+    :param ssh_transport: the Paramiko/SSH transport (i.e. the X2Go session's Paramiko/SSH transport object) (Default value = None)
+    :type ssh_transport: ``obj``
+    :param session_instance: the :class:`x2go.session.X2GoSession` instance that initiates this tunnel (Default value = None)
+    :type session_instance: ``obj``
+    :param session_name: the session name of the X2Go session this port forwarding server belongs to (Default value = None)
+    :type session_name: ``str``
+    :param subsystem: a custom string with a component name that tries to evoke a new tunnel setup (Default value = None)
+    :type subsystem: ``str``
+    :param logger: an X2GoLogger object (Default value = None)
+    :type logger: ``obj``
+    :returns: returns an :class:`x2go.forward.X2GoFwServer` instance
+    :rtype: ``obj``
 
     """
     fw_server = X2GoFwServer(listener=(local_host, local_port),
@@ -275,8 +281,8 @@ def stop_forward_tunnel(fw_server):
     """\
     Tear down a given Paramiko/SSH port forwarding tunnel.
 
-    @param fw_server: an L{X2GoFwServer} instance as returned by the L{start_forward_tunnel()} function
-    @type fw_server: C{obj}
+    :param fw_server: an :class:`x2go.forward.X2GoFwServer` instance as returned by the :func:`start_forward_tunnel()` function
+    :type fw_server: ``obj``
 
     """
     if fw_server is not None:
