@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -36,6 +36,9 @@
 ### for cases in that it is used in context of the X2Go project.
 ###
 
+__package__ = 'x2go'
+__name__    = 'x2go.gevent_subprocess'
+
 
 """Implementation of the standard :mod:`subprocess` module that spawns greenlets"""
 import errno
@@ -48,7 +51,6 @@ from gevent import socket, select, hub
 
 # identical to original
 CalledProcessError = _subprocess.CalledProcessError
-MAXFD = _subprocess.MAXFD
 PIPE = _subprocess.PIPE
 STDOUT = _subprocess.STDOUT
 call = _subprocess.call
@@ -81,7 +83,7 @@ class Popen(object):
                 try:
                     # f.write() doesn't return anything, so use os.write.
                     bytes_written += os.write(f.fileno(), input[bytes_written:])
-                except IOError, ex:
+                except IOError as ex:
                     if ex[0] != errno.EAGAIN:
                         raise
                     sys.exc_clear()
@@ -98,7 +100,7 @@ class Popen(object):
                 if not chunk:
                     break
                 chunks.append(chunk)
-            except IOError, ex:
+            except IOError as ex:
                 if ex[0] != errno.EAGAIN:
                     raise
                 sys.exc_clear()
@@ -149,7 +151,7 @@ class Popen(object):
         while read_set or write_set:
             try:
                 rlist, wlist, xlist = select.select(read_set, write_set, [])
-            except select.error, e:
+            except select.error as e:
                 if e.args[0] == errno.EINTR:
                     continue
                 raise
@@ -205,7 +207,7 @@ class Popen(object):
                 if status >= 0:
                     return status
                 hub.sleep(check_interval)
-        except OSError, e:
+        except OSError as e:
             if e.errno == errno.ECHILD:
                 # no child process, this happens if the child process
                 # already died and has been cleaned up
