@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -25,23 +25,27 @@ etc.).
 """
 __NAME__ = 'x2goguardian-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.guardian'
+
 # modules
 import gevent
 import threading
 import copy
 
 # Python X2Go modules
-from cleanup import x2go_cleanup
-import log
+from .cleanup import x2go_cleanup
+from . import log
 
 class X2GoSessionGuardian(threading.Thread):
     """\
-    L{X2GoSessionGuardian} thread controls X2Go session threads and their sub-threads (like
+    :class:`x2go.guardian.X2GoSessionGuardian` thread controls X2Go session threads and their sub-threads (like
     reverse forwarding tunnels, Paramiko transport threads, etc.). Its main function is
     to tidy up once a session gets interrupted (SIGTERM, SIGINT).
 
-    There is one L{X2GoSessionGuardian} for each L{X2GoClient} instance (thus: for normal
-    setups there should be _one_ L{X2GoClient} and _one_ L{X2GoSessionGuardian} in use).
+    There is one :class:`x2go.guardian.X2GoSessionGuardian` for each :class:`x2go.client.X2GoClient` instance (thus: for normal
+    setups there should be _one_ :class:`x2go.client.X2GoClient` and _one_ :class:`x2go.guardian.X2GoSessionGuardian` in use).
+
 
     """
     def __init__(self, client_instance,
@@ -54,26 +58,26 @@ class X2GoSessionGuardian(threading.Thread):
                  refresh_interval=5,
                  logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param auto_update_listsessions_cache: let L{X2GoSessionGuardian} refresh the session list cache for all L{X2GoSession} objects
-        @type auto_update_listsessions_cache: C{bool}
-        @param auto_update_listdesktops_cache: let L{X2GoSessionGuardian} refresh desktop lists in the session list cache for all L{X2GoSession} objects
-        @type auto_update_listdesktops_cache: C{bool}
-        @param auto_update_listmounts_cache: let L{X2GoSessionGuardian} refresh mount lists in the session list cache for all L{X2GoSession} objects
-        @type auto_update_listmounts_cache: C{bool}
-        @param auto_update_sessionregistry: if set to C{True} the session status will be updated in regular intervals
-        @type auto_update_sessionregistry: C{bool}
-        @param auto_register_sessions: register new sessions automatically once they appear in the X2Go session (e.g.
+        :param auto_update_listsessions_cache: let :class:`x2go.guardian.X2GoSessionGuardian` refresh the session list cache for all :class:`x2go.session.X2GoSession` objects
+        :type auto_update_listsessions_cache: ``bool``
+        :param auto_update_listdesktops_cache: let :class:`x2go.guardian.X2GoSessionGuardian` refresh desktop lists in the session list cache for all :class:`x2go.session.X2GoSession` objects
+        :type auto_update_listdesktops_cache: ``bool``
+        :param auto_update_listmounts_cache: let :class:`x2go.guardian.X2GoSessionGuardian` refresh mount lists in the session list cache for all :class:`x2go.session.X2GoSession` objects
+        :type auto_update_listmounts_cache: ``bool``
+        :param auto_update_sessionregistry: if set to ``True`` the session status will be updated in regular intervals
+        :type auto_update_sessionregistry: ``bool``
+        :param auto_register_sessions: register new sessions automatically once they appear in the X2Go session (e.g.
             instantiated by another client that is connected to the same X2Go server under same user ID)
-        @type auto_register_sessions: C{bool}
-        @param no_auto_reg_pubapp_sessions: do not auto-register published applications sessions
-        @type no_auto_reg_pubapp_sessions: C{bool}
-        @param refresh_interval: refresh cache and session registry every <refresh_interval> seconds
-        @type refresh_interval: C{int}
-        @param logger: you can pass an L{X2GoLogger} object to the L{X2GoSessionGuardian} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :type auto_register_sessions: ``bool``
+        :param no_auto_reg_pubapp_sessions: do not auto-register published applications sessions
+        :type no_auto_reg_pubapp_sessions: ``bool``
+        :param refresh_interval: refresh cache and session registry every <refresh_interval> seconds
+        :type refresh_interval: ``int``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the :class:`x2go.guardian.X2GoSessionGuardian` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if logger is None:
@@ -97,7 +101,8 @@ class X2GoSessionGuardian(threading.Thread):
 
     def guardian(self):
         """\
-        The handler of this L{X2GoSessionGuardian} thread.
+        The handler of this :class:`x2go.guardian.X2GoSessionGuardian` thread.
+
 
         """
         seconds = 0
@@ -125,16 +130,15 @@ class X2GoSessionGuardian(threading.Thread):
 
         self.logger('X2Go session guardian thread waking up after %s seconds' % seconds, loglevel=log.loglevel_DEBUG)
 
-        for session_uuid in self.client_instance.session_registry.keys():
+        for session_uuid in list(self.client_instance.session_registry.keys()):
             session_summary = self.client_instance.get_session_summary(session_uuid)
             self.logger('calling session cleanup on profile %s for terminal session: %s' % (session_summary['profile_name'], session_summary['session_name']), loglevel=log.loglevel_DEBUG)
             x2go_cleanup(threads=session_summary['active_threads'])
 
     def stop_thread(self):
         """\
-        Stop this L{X2GoSessionGuardian} thread.
+        Stop this :class:`x2go.guardian.X2GoSessionGuardian` thread.
+
 
         """
         self._keepalive = False
-
-
