@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -27,22 +27,29 @@
 X2GoProcessIniFile - helper class for parsing .ini files
 
 """
+from __future__ import print_function
 __NAME__ = 'x2goinifiles-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.inifiles'
+
 # modules
 import os
-import ConfigParser
+try:
+    import configparser
+except:
+    import ConfigParser as configparser
 import types
-import cStringIO
+import io
 import copy
 
 # Python X2Go modules
-from defaults import LOCAL_HOME as _current_home
-import log
-import utils
+from .defaults import LOCAL_HOME as _current_home
+from . import log
+from . import utils
 
 class X2GoIniFile(object):
-    """
+    """\
     Base class for processing the different ini files used by X2Go
     clients. Primarily used to standardize the content of the different
     X2Go client ini file (settings, printing, sessions, xconfig).
@@ -51,24 +58,23 @@ class X2GoIniFile(object):
     default values (as hard coded in Python X2Go), so the resulting objects
     always contain the same fields.
 
+    :param config_files: a list of configuration file names (e.g. a global filename and a user's home
+        directory filename)
+    :type config_files: ``list``
+    :param defaults: a cascaded Python dicitionary structure with ini file defaults (to override
+        Python X2Go's hard coded defaults in :mod:`x2go.defaults`
+    :type defaults: ``dict``
+    :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+        :class:`x2go.inifiles.X2GoIniFile` constructor
+    :type logger: :class:`x2go.log.X2GoLogger` instance
+    :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
+        constructed with the given loglevel
+    :type loglevel: ``int``
+
     """
 
     def __init__(self, config_files, defaults=None, logger=None, loglevel=log.loglevel_DEFAULT):
-        """\
-        @param config_files: a list of configuration file names (e.g. a global filename and a user's home
-            directory filename)
-        @type config_files: C{list}
-        @param defaults: a cascaded Python dicitionary structure with ini file defaults (to override
-            Python X2Go's hard coded defaults in L{defaults}
-        @type defaults: C{dict}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoIniFile} constructor
-        @type logger: L{X2GoLogger} instance
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
-            constructed with the given loglevel
-        @type loglevel: C{int}
 
-        """
         self.user_config_file = None
         self._write_user_config = True
 
@@ -93,7 +99,7 @@ class X2GoIniFile(object):
         # here as we do not want to run into name conflicts between
         # X2Go ini file options and method / property names in
         # SafeConfigParser... This is a pre-cautious approach...
-        self.iniConfig = ConfigParser.SafeConfigParser()
+        self.iniConfig = configparser.SafeConfigParser()
         self.iniConfig.optionxform = str
 
         _create_file = False
@@ -115,6 +121,7 @@ class X2GoIniFile(object):
         """\
         R(e-r)ead configuration file(s).
 
+
         """
         self.logger('proposed config files are %s' % self.config_files, loglevel=log.loglevel_INFO, )
         _found_config_files = self.iniConfig.read(self.config_files)
@@ -132,7 +139,7 @@ class X2GoIniFile(object):
     def __repr__(self):
         result = 'X2GoIniFile('
         for p in dir(self):
-            if '__' in p or not p in self.__dict__ or type(p) is types.InstanceType: continue
+            if '__' in p or not p in self.__dict__: continue
             result += p + '=' + str(self.__dict__[p]) + ','
         result = result.strip(',')
         return result + ')'
@@ -144,35 +151,34 @@ class X2GoIniFile(object):
         This methods affects a SafeConfigParser object held in
         RAM. No configuration file is affected by this
         method. To write the configuration to disk use
-        the L{write()} method.
+        the :func:`write()` method.
 
-        @param section: the ini file section
-        @type section: C{str}
-        @param key: the ini file key in the given section
-        @type key: C{str}
-        @param value: the value for the given section and key
-        @type value: C{str}, C{list}, C{booAl}, ...
+        :param section: the ini file section
+        :type section: ``str``
+        :param key: the ini file key in the given section
+        :type key: ``str``
+        :param value: the value for the given section and key
+        :type value: ``str``, ``list``, ``booAl``, ...
 
         """
-        if type(value) == type(u''):
-            value = value.encode(utils.get_encoding())
-        if type(value) is types.BooleanType:
+        if type(value) is bool:
             self.iniConfig.set(section, key, str(int(value)))
-        elif type(value) in (types.ListType, types.TupleType):
+        elif type(value) in (list, tuple):
             self.iniConfig.set(section, key, ", ".join(value))
         else:
             self.iniConfig.set(section, key, str(value))
 
     def _fill_defaults(self):
         """\
-        Fills a C{SafeConfigParser} object with the default ini file
+        Fills a ``SafeConfigParser`` object with the default ini file
         values as pre-defined in Python X2Go or. This SafeConfigParser
         object is held in RAM. No configuration file is affected by this
         method.
 
+
         """
-        for section, sectionvalue in self.defaultValues.items():
-            for key, value in sectionvalue.items():
+        for section, sectionvalue in list(self.defaultValues.items()):
+            for key, value in list(sectionvalue.items()):
                 if self.iniConfig.has_option(section, key): continue
                 if not self.iniConfig.has_section(section):
                     self.iniConfig.add_section(section)
@@ -183,12 +189,12 @@ class X2GoIniFile(object):
         Change a value for a given section and key. This method
         does not have any effect on configuration files.
 
-        @param section: the ini file section
-        @type section: C{str}
-        @param key: the ini file key in the given section
-        @type key: C{str}
-        @param value: the value for the given section and key
-        @type value: C{str}, C{list}, C{bool}, ...
+        :param section: the ini file section
+        :type section: ``str``
+        :param key: the ini file key in the given section
+        :type key: ``str``
+        :param value: the value for the given section and key
+        :type value: ``str``, ``list``, ``bool``, ...
 
         """
         if not self.iniConfig.has_section(section):
@@ -201,11 +207,13 @@ class X2GoIniFile(object):
         """\
         Write the ini file modifications (SafeConfigParser object) from RAM to disk.
 
-        For writing the first of the C{config_files} specified on instance construction
+        For writing the first of the ``config_files`` specified on instance construction
         that is writable will be used.
 
-        @return: C{True} if the user config file has been successfully written, C{False} otherwise.
-        @rtype: C{bool}
+
+        :returns: ``True`` if the user config file has been successfully written, ``False`` otherwise.
+
+        :rtype: ``bool``
 
         """
         if self.user_config_file and self._write_user_config:
@@ -215,8 +223,8 @@ class X2GoIniFile(object):
                 fd.close()
                 self._write_user_config = False
                 return True
-            except Exception, e:
-                print e
+            except Exception as e:
+                print(e)
         return False
     __write = write
 
@@ -225,13 +233,12 @@ class X2GoIniFile(object):
         Retrieve a value type for a given section and key. The returned
         value type is based on the default values dictionary.
 
-        @param section: the ini file section
-        @type section: C{str}
-        @param key: the ini file key in the given section
-        @type key: C{str}
-
-        @return: a Python variable type
-        @rtype: class
+        :param section: the ini file section
+        :type section: ``str``
+        :param key: the ini file key in the given section
+        :type key: ``str``
+        :returns: a Python variable type
+        :rtype: class
 
         """
         return type(self.defaultValues[section][key])
@@ -240,23 +247,23 @@ class X2GoIniFile(object):
         """\
         Retrieve a value for a given section and key.
 
-        @param section: the ini file section
-        @type section: C{str}
-        @param key: the ini file key in the given section
-        @type key: C{str}
-
-        @return: the value for the given section and key
-        @rtype: class
+        :param section: the ini file section
+        :type section: ``str``
+        :param key: the ini file key in the given section
+        :type key: ``str``
+        :param key_type: Python data type of the given key (Default value = None)
+        :returns: the value for the given section and key
+        :rtype: class
 
         """
         if key_type is None:
             key_type = self.get_type(section, key)
         if self.iniConfig.has_option(section, key):
-            if key_type is types.BooleanType:
+            if key_type is bool:
                 return self.iniConfig.getboolean(section, key)
-            elif key_type is types.IntType:
+            elif key_type is int:
                 return self.iniConfig.getint(section, key)
-            elif key_type is types.ListType:
+            elif key_type is list:
                 _val = self.iniConfig.get(section, key)
                 _val = _val.strip()
                 if _val.startswith('[') and _val.endswith(']'):
@@ -268,7 +275,7 @@ class X2GoIniFile(object):
                 return _val
             else:
                 _val = self.iniConfig.get(section, key)
-                return _val.decode(utils.get_encoding())
+                return _val
     get = get_value
     __call__ = get_value
 
@@ -277,8 +284,12 @@ class X2GoIniFile(object):
         """\
         Returns a printable configuration file as a multi-line string.
 
+
         """
-        stdout = cStringIO.StringIO()
+        if sys.version_info[0] >= 3:
+            stdout = io.BytesIO()
+        else:
+            stdout = io.StringIO()
         self.iniConfig.write(stdout)
         _ret_val = stdout.getvalue()
         stdout.close()
