@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,6 +23,9 @@ X2GoLogger class - flexible handling of log and debug output.
 """
 __NAME__ = 'x2gologger-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.log'
+
 # modules
 import os
 import sys
@@ -38,16 +41,17 @@ loglevel_DEBUG_SFTPXFER = 1024
 
 loglevel_DEFAULT = loglevel_ERROR | loglevel_WARN | loglevel_NOTICE
 """\
-Default loglevel of X2GoLogger objects is: NOTICE & WARN & ERROR
+Default loglevel of X2GoLogger objects is: NOTICE | WARN | ERROR
 """
 
 # Python X2Go modules
-import utils
+from . import utils
 
 class X2GoLogger(object):
     """\
     A simple logger class, that is used by all Python X2Go classes.
 
+
     """
     name = ''
     tag = ''
@@ -65,12 +69,12 @@ class X2GoLogger(object):
 
     def __init__(self, name=sys.argv[0], loglevel=loglevel_DEFAULT, tag=None):
         """\
-        @param name: name of the programme that uses Python X2Go
-        @type name: C{str}
-        @param loglevel: log level for Python X2Go
-        @type loglevel: C{int}
-        @param tag: additional tag for all log entries
-        @type tag: C{str}
+        :param name: name of the programme that uses Python X2Go
+        :type name: ``str``
+        :param loglevel: log level for Python X2Go
+        :type loglevel: ``int``
+        :param tag: additional tag for all log entries
+        :type tag: ``str``
 
         """
         self.name = os.path.basename(name)
@@ -82,12 +86,12 @@ class X2GoLogger(object):
         """\
         Log a message.
 
-        @param msg: log message text
-        @type msg: C{str}
-        @param loglevel: log level of this message
-        @type loglevel: C{int}
-        @param tag: additional tag for this log entry
-        @type tag: C{str}
+        :param msg: log message text
+        :type msg: ``str``
+        :param loglevel: log level of this message (Default value = loglevel_NONE)
+        :type loglevel: ``int``
+        :param tag: additional tag for this log entry (Default value = None)
+        :type tag: ``str``
 
         """
         if tag is None:
@@ -98,17 +102,19 @@ class X2GoLogger(object):
             msg = msg.encode(utils.get_encoding())
 
             if self.tag is not None:
-                self.destination.write('%s[%s] (%s) %s: %s\n' % (self.name, self.progpid, tag, self._loglevel_NAMES[loglevel].upper(), msg))
+                self.destination.write(u'%s[%s] (%s) %s: %s\n' % (self.name, self.progpid, tag, self._loglevel_NAMES[loglevel].upper(), msg.decode()))
             else:
-                self.destination.write('%s[%s] %s: %s\n' % (self.name, self.progpid, self._loglevel_NAMES[loglevel].upper(), msg))
+                self.destination.write(u'%s[%s] %s: %s\n' % (self.name, self.progpid, self._loglevel_NAMES[loglevel].upper(), msg.decode()))
     __call__ = message
 
     def get_loglevel(self):
         """\
         Get the current loglevel.
 
-        @return: current log level
-        @rtype: C{int}
+
+        :returns: current log level
+
+        :rtype: ``int``
 
         """
         return self.loglevel
@@ -117,13 +123,13 @@ class X2GoLogger(object):
         """\
         Set log level by name.
 
-        @param loglevel_name: name of loglevel to be set
-        @type loglevel_name: C{str}
+        :param loglevel_name: name of loglevel to be set (Default value = 'none')
+        :type loglevel_name: ``str``
 
         """
         if type(loglevel_name) is types.IntegerType:
             self.loglevel = loglevel_name
-        elif type(loglevel_name) is types.StringType and loglevel_name in self._loglevel_NAMES.values():
+        elif type(loglevel_name) is bytes and loglevel_name in list(self._loglevel_NAMES.values()):
             _method = getattr(self, 'self.set_loglevel_%s' % loglevel_name)
             _method()
         else:
@@ -133,40 +139,46 @@ class X2GoLogger(object):
         """\
         Silence logging completely.
 
+
         """
         self.loglevel = 0
 
     def set_loglevel_error(self):
         """\
-        Set log level to I{ERROR}.
+        Set log level to *ERROR*.
+
 
         """
         self.loglevel = loglevel_ERROR
 
     def set_loglevel_warn(self):
         """\
-        Set log level to I{WARN}.
+        Set log level to *WARN*.
+
 
         """
         self.loglevel = loglevel_ERROR | loglevel_WARN
 
     def set_loglevel_notice(self):
         """\
-        Set log level to I{NOTICE} (default).
+        Set log level to *NOTICE* (default).
+
 
         """
         self.loglevel = loglevel_ERROR | loglevel_WARN | loglevel_NOTICE
 
     def set_loglevel_info(self):
         """\
-        Set log level to I{INFO}.
+        Set log level to *INFO*.
+
 
         """
         self.loglevel = loglevel_ERROR | loglevel_WARN | loglevel_NOTICE | loglevel_INFO
 
     def set_loglevel_debug(self):
         """\
-        Set log level to I{DEBUG}.
+        Set log level to *DEBUG*.
+
 
         """
         self.loglevel = loglevel_ERROR | loglevel_WARN | loglevel_NOTICE | loglevel_INFO | loglevel_DEBUG
@@ -175,6 +187,7 @@ class X2GoLogger(object):
         """\
         Additionally, switch on sFTP data transfer debugging
 
+
         """
         self.loglevel = self.loglevel | loglevel_DEBUG_SFTPXFER
 
@@ -182,6 +195,7 @@ class X2GoLogger(object):
         """\
         Switch off sFTP data transfer debugging.
 
+
         """
         self.loglevel = self.loglevel ^ loglevel_DEBUG_SFTPXFER
 
