@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -19,28 +19,31 @@
 
 """\
 For MIME box jobs there are currently three handling actions available:
-L{X2GoMIMEboxActionOPEN}, L{X2GoMIMEboxActionOPENWITH} and L{X2GoMIMEboxActionSAVEAS}.
+:class:`x2go.mimeboxactions.X2GoMIMEboxActionOPEN`, :class:`x2go.mimeboxactions.X2GoMIMEboxActionOPENWITH` and :class:`x2go.mimeboxactions.X2GoMIMEboxActionSAVEAS`.
 
 """
 __NAME__ = 'x2gomimeboxactions-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.mimeboxactions'
+
 # modules
 import os
 import copy
 import time
 
-from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
+from .defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 if _X2GOCLIENT_OS in ("Windows"):
     import subprocess
     import win32api
 else:
-    import gevent_subprocess as subprocess
-    import x2go_exceptions
+    from . import gevent_subprocess as subprocess
+    from . import x2go_exceptions
     WindowsErrror = x2go_exceptions.WindowsError
 
 # Python X2Go modules
-import log
-import x2go_exceptions
+from . import log
+from . import x2go_exceptions
 
 _MIMEBOX_ENV = os.environ.copy()
 
@@ -55,14 +58,14 @@ class X2GoMIMEboxAction(object):
         This is a meta class and has no functionality as such. It is used as parent
         class by »real« X2Go MIME box actions.
 
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoMIMEboxAction} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.mimeboxactions.X2GoMIMEboxAction` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if logger is None:
@@ -82,6 +85,11 @@ class X2GoMIMEboxAction(object):
         """\
         Return the X2Go MIME box action's name.
 
+
+        :returns: MIME box action name
+
+        :rtype: ``str``
+
         """
         return self.__name__
 
@@ -90,17 +98,22 @@ class X2GoMIMEboxAction(object):
         """\
         Return the X2Go MIME box action's description text.
 
+
+        :returns: MIME box action's description
+
+        :rtype: ``str``
+
         """
         return self.__description__
 
     def _do_process(self, mimebox_file, mimebox_dir, ):
         """\
-        Perform the defined MIME box action (doing nothing in L{X2GoMIMEboxAction} parent class).
+        Perform the defined MIME box action (doing nothing in :class:`x2go.mimeboxactions.X2GoMIMEboxAction` parent class).
 
-        @param mimebox_file: file name as placed in to the X2Go MIME box directory
-        @type mimebox_file: C{str}
-        @param mimebox_dir: location of the X2Go session's MIME box directory
-        @type mimebox_dir: C{str}
+        :param mimebox_file: file name as placed in to the X2Go MIME box directory
+        :type mimebox_file: ``str``
+        :param mimebox_dir: location of the X2Go session's MIME box directory
+        :type mimebox_dir: ``str``
 
         """
         pass
@@ -110,10 +123,10 @@ class X2GoMIMEboxAction(object):
         Wrapper method for the actual processing of MIME
         box actions.
 
-        @param mimebox_file: file name as placed in to the X2Go MIME box directory
-        @type mimebox_file: C{str}
-        @param mimebox_dir: location of the X2Go session's MIME box directory
-        @type mimebox_dir: C{str}
+        :param mimebox_file: file name as placed in to the X2Go MIME box directory
+        :type mimebox_file: ``str``
+        :param mimebox_dir: location of the X2Go session's MIME box directory
+        :type mimebox_dir: ``str``
 
         """
         mimebox_file = os.path.normpath(mimebox_file)
@@ -126,20 +139,21 @@ class X2GoMIMEboxActionOPEN(X2GoMIMEboxAction):
     """\
     MIME box action that opens incoming files in the system's default application.
 
+
     """
     __name__= 'OPEN'
     __decription__= 'Open incoming file with local system\'s default application.'
 
     def __init__(self, client_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoMIMEboxActionOPEN} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.mimeboxactions.X2GoMIMEboxActionOPEN` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         self.client_instance = client_instance
@@ -149,10 +163,10 @@ class X2GoMIMEboxActionOPEN(X2GoMIMEboxAction):
         """\
         Open an incoming MIME box file in the system's default application.
 
-        @param mimebox_file: file name as placed in to the MIME box directory
-        @type mimebox_file: C{str}
-        @param mimebox_dir: location of the X2Go session's MIME box directory
-        @type mimebox_dir: C{str}
+        :param mimebox_file: file name as placed in to the MIME box directory
+        :type mimebox_file: ``str``
+        :param mimebox_dir: location of the X2Go session's MIME box directory
+        :type mimebox_dir: ``str``
 
         """
         mimebox_file = os.path.normpath(mimebox_file)
@@ -162,7 +176,7 @@ class X2GoMIMEboxActionOPEN(X2GoMIMEboxAction):
             self.logger('opening incoming MIME box file with Python\'s os.startfile() command: %s' % mimebox_file, loglevel=log.loglevel_DEBUG)
             try:
                 os.startfile(os.path.join(mimebox_dir, mimebox_file))
-            except WindowsError, win_err:
+            except WindowsError as win_err:
                 if self.client_instance:
                     self.client_instance.HOOK_mimeboxaction_error(mimebox_file,
                                                                   profile_name=self.profile_name,
@@ -184,20 +198,21 @@ class X2GoMIMEboxActionOPENWITH(X2GoMIMEboxAction):
     MIME box action that calls the system's ,,Open with...'' dialog on incoming files. Currently only
     properly implementable on Windows platforms.
 
+
     """
     __name__= 'OPENWITH'
     __decription__= 'Evoke ,,Open with...\'\' dialog on incoming MIME box files.'
 
     def __init__(self, client_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoMIMEboxActionOPENWITH} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.mimeboxactions.X2GoMIMEboxActionOPENWITH` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         self.client_instance = client_instance
@@ -207,10 +222,10 @@ class X2GoMIMEboxActionOPENWITH(X2GoMIMEboxAction):
         """\
         Open an incoming MIME box file in the system's default application.
 
-        @param mimebox_file: file name as placed in to the MIME box directory
-        @type mimebox_file: C{str}
-        @param mimebox_dir: location of the X2Go session's MIME box directory
-        @type mimebox_dir: C{str}
+        :param mimebox_file: file name as placed in to the MIME box directory
+        :type mimebox_file: ``str``
+        :param mimebox_dir: location of the X2Go session's MIME box directory
+        :type mimebox_dir: ``str``
 
         """
         mimebox_file = os.path.normpath(mimebox_file)
@@ -238,28 +253,29 @@ class X2GoMIMEboxActionOPENWITH(X2GoMIMEboxAction):
 class X2GoMIMEboxActionSAVEAS(X2GoMIMEboxAction):
     """\
     MIME box action that allows saving incoming MIME box files to a local folder. What this
-    MIME box actually does is calling a hook method in the L{X2GoClient} instance that
+    MIME box actually does is calling a hook method in the :class:`x2go.client.X2GoClient` instance that
     can be hi-jacked by one of your application's methods which then can handle the ,,Save as...''
     request.
 
+
     """
     __name__ = 'SAVEAS'
     __decription__= 'Save incoming file as...'
 
     def __init__(self, client_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: an L{X2GoClient} instance, within your customized L{X2GoClient} make sure
-            you have a C{HOOK_open_mimebox_saveas_dialog(filename=<str>)} method defined that will actually
+        :param client_instance: an :class:`x2go.client.X2GoClient` instance, within your customized :class:`x2go.client.X2GoClient` make sure
+            you have a ``HOOK_open_mimebox_saveas_dialog(filename=<str>)`` method defined that will actually
             handle the incoming mimebox file.
-        @type client_instance: C{obj}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoMIMEboxActionSAVEAS} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :type client_instance: ``obj``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.mimeboxactions.X2GoMIMEboxActionSAVEAS` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
-        @raise X2GoMIMEboxActionException: if the client_instance has not been passed to the SAVEAS MIME box action
+        :raises X2GoMIMEboxActionException: if the client_instance has not been passed to the SAVEAS MIME box action
 
         """
         if client_instance is None:
@@ -268,14 +284,14 @@ class X2GoMIMEboxActionSAVEAS(X2GoMIMEboxAction):
 
     def _do_process(self, mimebox_file, mimebox_dir):
         """\
-        Call an L{X2GoClient} hook method (C{HOOK_open_mimebox_saveas_dialog}) that
+        Call an :class:`x2go.client.X2GoClient` hook method (``HOOK_open_mimebox_saveas_dialog``) that
         can handle the MIME box's SAVEAS action.
 
-        @param mimebox_file: file name as placed in to the MIME box directory
-        @type mimebox_file: C{str}
-        @param mimebox_dir: location of the X2Go session's MIME box directory
-        @type mimebox_dir: C{str}
-        @param mimebox_file: PDF file name as placed in to the X2Go spool directory
+        :param mimebox_file: file name as placed in to the MIME box directory
+        :type mimebox_file: ``str``
+        :param mimebox_dir: location of the X2Go session's MIME box directory
+        :type mimebox_dir: ``str``
+        :param mimebox_file: PDF file name as placed in to the X2Go spool directory
 
         """
         mimebox_file = os.path.normpath(mimebox_file)
