@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -21,12 +21,15 @@
 Print jobs can either be sent to any of the local print queues (CUPS, Win32API),
 be opened in an external PDF viewer, be saved to a local folder or be handed
 over to a custom (print) command. This is defined by four print action classes
-(L{X2GoPrintActionDIALOG}, L{X2GoPrintActionPDFVIEW}, L{X2GoPrintActionPDFSAVE}, L{X2GoPrintActionPRINT} and
-L{X2GoPrintActionPRINTCMD}).
+(:class:`x2go.printactions.X2GoPrintActionDIALOG`, :class:`x2go.printactions.X2GoPrintActionPDFVIEW`, :class:`x2go.printactions.X2GoPrintActionPDFSAVE`, :class:`x2go.printactions.X2GoPrintActionPRINT` and
+:class:`x2go.printactions.X2GoPrintActionPRINTCMD`).
 
 """
 __NAME__ = 'x2goprintactions-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.printactions'
+
 # modules
 import os
 import shutil
@@ -34,22 +37,22 @@ import copy
 import time
 import gevent
 
-from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
+from .defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 if _X2GOCLIENT_OS in ("Windows"):
     import subprocess
     import win32api
     import win32print
 else:
-    import gevent_subprocess as subprocess
-    import x2go_exceptions
+    from . import gevent_subprocess as subprocess
+    from . import x2go_exceptions
     WindowsError = x2go_exceptions.WindowsError
 
 # Python X2Go modules
-import log
-import defaults
+from . import log
+from . import defaults
 # we hide the default values from epydoc (that's why we transform them to _UNDERSCORE variables)
-import utils
-import x2go_exceptions
+from . import utils
+from . import x2go_exceptions
 
 _PRINT_ENV = os.environ.copy()
 
@@ -64,14 +67,14 @@ class X2GoPrintAction(object):
         This is a meta class and has no functionality as such. It is used as parent
         class by »real« X2Go print actions.
 
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoPrintAction} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.printactions.X2GoPrintAction` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if logger is None:
@@ -91,6 +94,11 @@ class X2GoPrintAction(object):
         """\
         Return the X2Go print action's name.
 
+
+        :returns: print action name
+
+        :rtype: ``str``
+
         """
         return self.__name__
 
@@ -99,34 +107,39 @@ class X2GoPrintAction(object):
         """\
         Return the X2Go print action's description text.
 
+
+        :returns: print action's description
+
+        :rtype: ``str``
+
         """
         return self.__description__
 
     def _do_print(self, pdf_file, job_title, spool_dir, ):
-        """
-        Perform the defined print action (doing nothing in L{X2GoPrintAction} parent class).
+        """\
+        Perform the defined print action (doing nothing in :class:`x2go.printactions.X2GoPrintAction` parent class).
 
-        @param pdf_file: PDF file name as placed in to the X2Go spool directory
-        @type pdf_file: C{str}
-        @param job_title: human readable print job title
-        @type job_title: C{str}
-        @param spool_dir: location of the X2Go client's spool directory
-        @type spool_dir: C{str}
+        :param pdf_file: PDF file name as placed in to the X2Go spool directory
+        :type pdf_file: ``str``
+        :param job_title: human readable print job title
+        :type job_title: ``str``
+        :param spool_dir: location of the X2Go client's spool directory
+        :type spool_dir: ``str``
 
         """
         pass
 
     def do_print(self, pdf_file, job_title, spool_dir, ):
         """\
-        Wrap around the actual print action (C{self._do_print}) with
+        Wrap around the actual print action (``self._do_print``) with
         gevent.spawn().
 
-        @param pdf_file: PDF file name as placed in to the X2Go spool directory
-        @type pdf_file: C{str}
-        @param job_title: human readable print job title
-        @type job_title: C{str}
-        @param spool_dir: location of the X2Go client's spool directory
-        @type spool_dir: C{str}
+        :param pdf_file: PDF file name as placed in to the X2Go spool directory
+        :type pdf_file: ``str``
+        :param job_title: human readable print job title
+        :type job_title: ``str``
+        :param spool_dir: location of the X2Go client's spool directory
+        :type spool_dir: ``str``
 
         """
         pdf_file = os.path.normpath(pdf_file)
@@ -138,14 +151,14 @@ class X2GoPrintAction(object):
         """\
         Extract a human readable filename for the X2Go print job file.
 
-        @param pdf_file: PDF file name as placed in to the X2Go spool directory
-        @type pdf_file: C{str}
-        @param job_title: human readable print job title
-        @type job_title: C{str}
-        @param target_path: target path for human readable file
-        @type target_path: C{str}
-        @return: full readable file name path
-        @rtype: C{str}
+        :param pdf_file: PDF file name as placed in to the X2Go spool directory
+        :type pdf_file: ``str``
+        :param job_title: human readable print job title
+        :type job_title: ``str``
+        :param target_path: target path for human readable file
+        :type target_path: ``str``
+        :returns: full readable file name path
+        :rtype: ``str``
 
         """
         _hr_path = os.path.normpath(os.path.expanduser(os.path.join(os.path.normpath(target_path), '%s.pdf' % utils.slugify(job_title))))
@@ -162,6 +175,7 @@ class X2GoPrintActionPDFVIEW(X2GoPrintAction):
     """\
     Print action that views incoming print job in an external PDF viewer application.
 
+
     """
     __name__= 'PDFVIEW'
     __decription__= 'View as PDF document'
@@ -170,16 +184,16 @@ class X2GoPrintActionPDFVIEW(X2GoPrintAction):
 
     def __init__(self, client_instance=None, pdfview_cmd=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param pdfview_cmd: command that starts the external PDF viewer application
-        @type pdfview_cmd: C{str}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoPrintActionPDFVIEW} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param pdfview_cmd: command that starts the external PDF viewer application
+        :type pdfview_cmd: ``str``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.printactions.X2GoPrintActionPDFVIEW` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if pdfview_cmd is None:
@@ -191,14 +205,13 @@ class X2GoPrintActionPDFVIEW(X2GoPrintAction):
         """\
         Open an incoming X2Go print job (PDF file) in an external PDF viewer application.
 
-        @param pdf_file: PDF file name as placed in to the X2Go spool directory
-        @type pdf_file: C{str}
-        @param job_title: human readable print job title
-        @type job_title: C{str}
-        @param spool_dir: location of the X2Go client's spool directory
-        @type spool_dir: C{str}
-
-        @raise OSError: pass through all C{OSError}s except no. 2
+        :param pdf_file: PDF file name as placed in to the X2Go spool directory
+        :type pdf_file: ``str``
+        :param job_title: human readable print job title
+        :type job_title: ``str``
+        :param spool_dir: location of the X2Go client's spool directory
+        :type spool_dir: ``str``
+        :raises OSError: pass through all ``OSError``s except no. 2
 
         """
         pdf_file = os.path.normpath(pdf_file)
@@ -208,7 +221,7 @@ class X2GoPrintActionPDFVIEW(X2GoPrintAction):
             self.logger('viewing incoming job in PDF viewer with Python\'s os.startfile(command): %s' % pdf_file, loglevel=log.loglevel_DEBUG)
             try:
                 gevent.spawn(os.startfile, pdf_file)
-            except WindowsError, win_err:
+            except WindowsError as win_err:
                 if self.client_instance:
                     self.client_instance.HOOK_printaction_error(pdf_file,
                                                                 profile_name=self.profile_name,
@@ -225,7 +238,7 @@ class X2GoPrintActionPDFVIEW(X2GoPrintAction):
             self.logger('viewing incoming PDF with command: %s' % ' '.join(cmd_line), loglevel=log.loglevel_DEBUG)
             try:
                 subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
-            except OSError, e:
+            except OSError as e:
                 if e.errno == 2:
                     cmd_line = [ defaults.DEFAULT_PDFVIEW_CMD, _hr_filename ]
                     subprocess.Popen(cmd_line, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=_PRINT_ENV)
@@ -240,6 +253,7 @@ class X2GoPrintActionPDFSAVE(X2GoPrintAction):
     """\
     Print action that saves incoming print jobs to a local folder.
 
+
     """
     __name__ = 'PDFSAVE'
     __decription__= 'Save as PDF'
@@ -248,16 +262,16 @@ class X2GoPrintActionPDFSAVE(X2GoPrintAction):
 
     def __init__(self, client_instance=None, save_to_folder=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param save_to_folder: saving location for incoming print jobs (PDF files)
-        @type save_to_folder: C{str}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoPrintActionPDFSAVE} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param save_to_folder: saving location for incoming print jobs (PDF files)
+        :type save_to_folder: ``str``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.printactions.X2GoPrintActionPDFSAVE` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if save_to_folder is None:
@@ -272,18 +286,18 @@ class X2GoPrintActionPDFSAVE(X2GoPrintAction):
 
         self.logger('Save location for incoming PDFs is: %s' % self.save_to_folder, loglevel=log.loglevel_DEBUG)
         if not os.path.exists(self.save_to_folder):
-            os.makedirs(self.save_to_folder, mode=0755)
+            os.makedirs(self.save_to_folder, mode=0o755)
 
     def _do_print(self, pdf_file, job_title, spool_dir):
         """\
         Save an incoming X2Go print job (PDF file) to a local folder.
 
-        @param pdf_file: PDF file name as placed in to the X2Go spool directory
-        @type pdf_file: C{str}
-        @param job_title: human readable print job title
-        @type job_title: C{str}
-        @param spool_dir: location of the X2Go client's spool directory
-        @type spool_dir: C{str}
+        :param pdf_file: PDF file name as placed in to the X2Go spool directory
+        :type pdf_file: ``str``
+        :param job_title: human readable print job title
+        :type job_title: ``str``
+        :param spool_dir: location of the X2Go client's spool directory
+        :type spool_dir: ``str``
 
         """
         pdf_file = os.path.normpath(pdf_file)
@@ -297,22 +311,23 @@ class X2GoPrintActionPRINT(X2GoPrintAction):
     """\
     Print action that actually prints an incoming print job file.
 
+
     """
     __name__ = 'PRINT'
     __decription__= 'UNIX/Win32GDI printing'
 
     def __init__(self, client_instance=None, printer=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param printer: name of the preferred printer, if C{None} the system's/user's default printer will be used
-        @type printer: C{str}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoPrintActionPRINT} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param printer: name of the preferred printer, if ``None`` the system's/user's default printer will be used
+        :type printer: ``str``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.printactions.X2GoPrintActionPRINT` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         self.printer = printer
@@ -322,12 +337,12 @@ class X2GoPrintActionPRINT(X2GoPrintAction):
         """\
         Really print an incoming X2Go print job (PDF file) to a local printer device.
 
-        @param pdf_file: PDF file name as placed in to the X2Go spool directory
-        @type pdf_file: C{str}
-        @param job_title: human readable print job title
-        @type job_title: C{str}
-        @param spool_dir: location of the X2Go client's spool directory
-        @type spool_dir: C{str}
+        :param pdf_file: PDF file name as placed in to the X2Go spool directory
+        :type pdf_file: ``str``
+        :param job_title: human readable print job title
+        :type job_title: ``str``
+        :param spool_dir: location of the X2Go client's spool directory
+        :type spool_dir: ``str``
 
         """
         pdf_file = os.path.normpath(pdf_file)
@@ -376,7 +391,7 @@ class X2GoPrintActionPRINT(X2GoPrintAction):
                     )
                     # give the win32api some time to find our printer...
                     time.sleep(10)
-                except win32api.error, e:
+                except win32api.error as e:
                     if self.client_instance:
                         self.client_instance.HOOK_printaction_error(filename=_hr_filename, printer=_printer, err_msg=e.message, profile_name=self.profile_name, session_name=self.session_name)
                     else:
@@ -424,7 +439,8 @@ class X2GoPrintActionPRINTCMD(X2GoPrintAction):
     Print action that calls an external command for further processing of incoming print jobs.
 
     The print job's PDF filename will be prepended as last argument to the print command
-    used in L{X2GoPrintActionPRINTCMD} instances.
+    used in :class:`x2go.printactions.X2GoPrintActionPRINTCMD` instances.
+
 
     """
     __name__      = 'PRINTCMD'
@@ -432,16 +448,16 @@ class X2GoPrintActionPRINTCMD(X2GoPrintAction):
 
     def __init__(self, client_instance=None, print_cmd=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param print_cmd: external command to be called on incoming print jobs
-        @type print_cmd: C{str}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoPrintActionPRINTCMD} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param print_cmd: external command to be called on incoming print jobs
+        :type print_cmd: ``str``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.printactions.X2GoPrintActionPRINTCMD` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if print_cmd is None:
@@ -452,14 +468,14 @@ class X2GoPrintActionPRINTCMD(X2GoPrintAction):
     def _do_print(self, pdf_file, job_title, spool_dir):
         """\
         Execute an external command that has been defined on construction
-        of this L{X2GoPrintActionPRINTCMD} instance.
+        of this :class:`x2go.printactions.X2GoPrintActionPRINTCMD` instance.
 
-        @param pdf_file: PDF file name as placed in to the X2Go spool directory
-        @type pdf_file: C{str}
-        @param job_title: human readable print job title
-        @type job_title: C{str}
-        @param spool_dir: location of the X2Go client's spool directory
-        @type spool_dir: C{str}
+        :param pdf_file: PDF file name as placed in to the X2Go spool directory
+        :type pdf_file: ``str``
+        :param job_title: human readable print job title
+        :type job_title: ``str``
+        :param spool_dir: location of the X2Go client's spool directory
+        :type spool_dir: ``str``
 
         """
         pdf_file = os.path.normpath(pdf_file)
@@ -484,7 +500,8 @@ class X2GoPrintActionDIALOG(X2GoPrintAction):
     """\
     Print action that mediates opening a print dialog window. This class is rather empty,
     the actual print dialog box must be implemented in our GUI application (with the application's
-    L{X2GoClient} instance.
+    :class:`x2go.client.X2GoClient` instance.
+
 
     """
     __name__      = 'DIALOG'
@@ -492,18 +509,18 @@ class X2GoPrintActionDIALOG(X2GoPrintAction):
 
     def __init__(self, client_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        @param client_instance: an L{X2GoClient} instance, within your customized L{X2GoClient} make sure
-            you have a C{HOOK_open_print_dialog(filename=<str>)} method defined that will actually
+        :param client_instance: an :class:`x2go.client.X2GoClient` instance, within your customized :class:`x2go.client.X2GoClient` make sure
+            you have a ``HOOK_open_print_dialog(filename=<str>)`` method defined that will actually
             open the print dialog.
-        @type client_instance: C{obj}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoPrintActionDIALOG} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :type client_instance: ``obj``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.printactions.X2GoPrintActionDIALOG` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
-        @raise X2GoPrintActionException: if the client_instance has not been passed to the DIALOG print action
+        :raises X2GoPrintActionException: if the client_instance has not been passed to the DIALOG print action
 
         """
         if client_instance is None:
@@ -513,14 +530,14 @@ class X2GoPrintActionDIALOG(X2GoPrintAction):
     def _do_print(self, pdf_file, job_title, spool_dir):
         """\
         Execute an external command that has been defined on construction
-        of this L{X2GoPrintActionPRINTCMD} instance.
+        of this :class:`x2go.printactions.X2GoPrintActionPRINTCMD` instance.
 
-        @param pdf_file: PDF file name as placed in to the X2Go spool directory
-        @type pdf_file: C{str}
-        @param job_title: human readable print job title
-        @type job_title: C{str}
-        @param spool_dir: location of the X2Go client's spool directory
-        @type spool_dir: C{str}
+        :param pdf_file: PDF file name as placed in to the X2Go spool directory
+        :type pdf_file: ``str``
+        :param job_title: human readable print job title
+        :type job_title: ``str``
+        :param spool_dir: location of the X2Go client's spool directory
+        :type spool_dir: ``str``
 
         """
         pdf_file = os.path.normpath(pdf_file)
