@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -18,15 +18,18 @@
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-L{X2GoPrintQueue} sets up a thread that listens for incoming print jobs.
+:class:`x2go.printing.X2GoPrintQueue` sets up a thread that listens for incoming print jobs.
 
 For each incoming print job in an X2Go session's spool directory an
-individual thread is started (L{X2GoPrintJob}) that handles the processing
+individual thread is started (:class:`x2go.printqueue.X2GoPrintJob`) that handles the processing
 of the incoming print job.
 
 """
 __NAME__ = 'x2goprintqueue-pylib'
 
+__package__ = 'x2go'
+__name_     = 'x2go.printqueue'
+
 # modules
 import os
 import copy
@@ -34,18 +37,19 @@ import threading
 import gevent
 
 # Python X2Go modules
-import defaults
-import utils
-import log
+from . import defaults
+from . import utils
+from . import log
 
-from defaults import X2GO_PRINTING_FILENAME as _X2GO_PRINTING_FILENAME
-from defaults import BACKENDS as _BACKENDS
+from .defaults import X2GO_PRINTING_FILENAME as _X2GO_PRINTING_FILENAME
+from .defaults import BACKENDS as _BACKENDS
 
 class X2GoPrintQueue(threading.Thread):
     """\
-    If X2Go printing is supported in a particular L{X2GoSession} instance
+    If X2Go printing is supported in a particular :class:`x2go.session.X2GoSession` instance
     this class provides a sub-thread for handling incoming X2Go print jobs.
 
+
     """
     print_action = None
 
@@ -64,25 +68,25 @@ class X2GoPrintQueue(threading.Thread):
                  logger=None,
                  loglevel=log.loglevel_DEFAULT):
         """\
-        @param profile_name: name of the session profile this print queue belongs to
-        @type profile_name: C{str}
-        @param spool_dir: local spool directory for incoming print job files
-        @type spool_dir: C{str}
-        @param print_action: name or instance of either of the possible X2Go print action classes
-        @type print_action: C{str} or instance
-        @param print_action_args: depending of the chosen C{print_action} this dictionary may contain different
-            values; the C{print_action_args} will be passed on to the X2Go print action instance constructor, so
-            refer to either of these: L{X2GoPrintActionPDFVIEW}, L{X2GoPrintActionPRINT} et al.
-        @param client_instance: the underlying L{X2GoClient} instance
-        @type client_instance: C{obj}
-        @param printing_backend: the client printing configuration backend class
-        @type printing_backend: C{obj}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoPrintQueue} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param profile_name: name of the session profile this print queue belongs to
+        :type profile_name: ``str``
+        :param spool_dir: local spool directory for incoming print job files
+        :type spool_dir: ``str``
+        :param print_action: name or instance of either of the possible X2Go print action classes
+        :type print_action: ``str`` or instance
+        :param print_action_args: depending of the chosen ``print_action`` this dictionary may contain different
+            values; the ``print_action_args`` will be passed on to the X2Go print action instance constructor, so
+            refer to either of these: :class:`x2go.printactions.X2GoPrintActionPDFVIEW`, :class:`x2go.printactions.X2GoPrintActionPRINT` et al.
+        :param client_instance: the underlying :class:`x2go.client.X2GoClient` instance
+        :type client_instance: ``obj``
+        :param printing_backend: the client printing configuration backend class
+        :type printing_backend: ``obj``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.printing.X2GoPrintQueue` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if logger is None:
@@ -116,6 +120,7 @@ class X2GoPrintQueue(threading.Thread):
         Prevent acceptance of new incoming print jobs. The processing of print jobs that
         are currently still active will be completed, though.
 
+
         """
         if self._accept_jobs == True:
             self._accept_jobs = False
@@ -126,6 +131,7 @@ class X2GoPrintQueue(threading.Thread):
         Resume operation of the X2Go print spooler and continue accepting new incoming
         print jobs.
 
+
         """
         if self._accept_jobs == False:
             self._accept_jobs = True
@@ -133,7 +139,8 @@ class X2GoPrintQueue(threading.Thread):
 
     def stop_thread(self):
         """\
-        Stops this L{X2GoPrintQueue} thread completely.
+        Stops this :class:`x2go.printing.X2GoPrintQueue` thread completely.
+
 
         """
         self.pause()
@@ -157,34 +164,35 @@ class X2GoPrintQueue(threading.Thread):
                     job_title = 'X2Go Print Job'
                 _job_file_handle.close()
                 jobs.append((_job_file, pdf_filename, job_title))
-            return [ j for j in jobs if j[1] not in self.active_jobs.keys() ]
+            return [ j for j in jobs if j[1] not in list(self.active_jobs.keys()) ]
         else:
             return []
 
     def set_print_action(self, print_action, **kwargs):
         """\
-        Modify the print action of this L{X2GoPrintQueue} thread during runtime. The
+        Modify the print action of this :class:`x2go.printing.X2GoPrintQueue` thread during runtime. The
         change of print action will be valid for the next incoming print job.
 
         As kwargs you can pass arguments for the print action class to be set. Refer
-        to the class descriptions of L{X2GoPrintActionDIALOG}, L{X2GoPrintActionPDFVIEW},
-        L{X2GoPrintActionPRINT}, etc.
+        to the class descriptions of :class:`x2go.printactions.X2GoPrintActionDIALOG`, :class:`x2go.printactions.X2GoPrintActionPDFVIEW`,
+        :class:`x2go.printactions.X2GoPrintActionPRINT`, etc.
 
-        @param print_action: new print action to be valid for incoming print jobs
-        @type print_action: C{str} or C{class}
-        @param kwargs: extra options for the specified print action
-        @type kwargs: C{dict}
+        :param print_action: new print action to be valid for incoming print jobs
+        :type print_action: ``str`` or ``class``
+        :param kwargs: extra options for the specified print action
+        :type kwargs: ``dict``
 
         """
-        if print_action in defaults.X2GO_PRINT_ACTIONS.keys():
+        if print_action in list(defaults.X2GO_PRINT_ACTIONS.keys()):
             print_action = defaults.X2GO_PRINT_ACTIONS[print_action]
 
-        if print_action in defaults.X2GO_PRINT_ACTIONS.values():
+        if print_action in list(defaults.X2GO_PRINT_ACTIONS.values()):
             self.print_action = eval ('printactions.%s(**kwargs)' % print_action)
 
     def run(self):
         """\
-        Start this L{X2GoPrintQueue} thread...
+        Start this :class:`x2go.printing.X2GoPrintQueue` thread...
+
 
         """
         self.logger('starting print queue thread: %s' % repr(self), loglevel=log.loglevel_DEBUG)
@@ -219,22 +227,24 @@ class X2GoPrintQueue(threading.Thread):
 def x2go_printjob_handler(job_file=None, pdf_file=None, job_title=None, print_action=None, parent_thread=None, logger=None, ):
     """\
     This function is called as a handler function for each incoming X2Go print job
-    represented by the class L{X2GoPrintJob}.
+    represented by the class :class:`x2go.printqueue.X2GoPrintJob`.
 
     The handler function will (re-)read the »printing« configuration file (if no
-    explicit C{print_action} is passed to this function...). It then will
-    execute the C{<print_action>.do_print()} command.
+    explicit ``print_action`` is passed to this function...). It then will
+    execute the ``<print_action>.do_print()`` command.
 
-    @param pdf_file: PDF file name as placed in to the X2Go spool directory
-    @type pdf_file: C{str}
-    @param job_title: human readable print job title
-    @type job_title: C{str}
-    @param print_action: an instance of either of the possible C{X2GoPrintActionXXX} classes
-    @type print_action: C{X2GoPrintActionXXX} nstance
-    @param parent_thread: the L{X2GoPrintQueue} thread that actually created this handler's L{X2GoPrintJob} instance
-    @type parent_thread: C{obj}
-    @param logger: the L{X2GoPrintQueue}'s logging instance
-    @type logger: C{obj}
+    :param job_file: file name of this print job's Job File (Default value = None)
+    :type job_file: ``str``
+    :param pdf_file: PDF file name as placed in to the X2Go spool directory (Default value = None)
+    :type pdf_file: ``str``
+    :param job_title: human readable print job title (Default value = None)
+    :type job_title: ``str``
+    :param print_action: an instance of either of the possible ``X2GoPrintActionXXX`` classes (Default value = None)
+    :type print_action: ``X2GoPrintActionXXX`` nstance
+    :param parent_thread: the :class:`x2go.printing.X2GoPrintQueue` thread that actually created this handler's :class:`x2go.printqueue.X2GoPrintJob` instance (Default value = None)
+    :type parent_thread: ``obj``
+    :param logger: the :class:`x2go.printing.X2GoPrintQueue`'s logging instance (Default value = None)
+    :type logger: ``obj``
 
     """
     if print_action is None:
@@ -279,16 +289,17 @@ class X2GoPrintJob(threading.Thread):
     For each X2Go print job we create a sub-thread that let's
     the print job be processed in the background.
 
-    As a handler for this class the function L{x2go_printjob_handler()}
+    As a handler for this class the function :func:`x2go_printjob_handler()`
     is used.
 
+
     """
     def __init__(self, **kwargs):
         """\
         Construct the X2Go print job thread...
 
         All parameters (**kwargs) are passed through to the constructor
-        of C{threading.Thread()}.
+        of ``threading.Thread()``.
 
         """
         threading.Thread.__init__(self, **kwargs)
