@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -27,7 +27,10 @@ X2GoPulseAudio class - a Pulseaudio daemon guardian thread.
 
 __NAME__ = 'x2gopulseaudio-pylib'
 
-from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
+__package__ = 'x2go'
+__name__    = 'x2go.pulseaudio'
+
+from .defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 if _X2GOCLIENT_OS == 'Windows':
     import win32process
     import win32con
@@ -41,35 +44,32 @@ import gevent
 import copy
 import socket
 
-from defaults import LOCAL_HOME as _LOCAL_HOME
+from .defaults import LOCAL_HOME as _LOCAL_HOME
 
 # Python X2Go modules
-import log
+from . import log
 
-import exceptions
-class OSNotSupportedException(exceptions.StandardError): pass
+class OSNotSupportedException(BaseException): pass
 """ Exception denoting that this operating system is not supported. """
 
 class X2GoPulseAudio(threading.Thread):
-    """
-    This class controls the Pulse Audio daemon.
-    """
+    """This class controls the Pulse Audio daemon."""
 
     def __init__(self, path=None, client_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         Initialize a Pulse Audio daemon instance.
 
-        @param path: full path to pulseaudio.exe
-        @type path: C{str}
-        @param client_instance: the calling L{X2GoClient} instance
-        @type client_instance: L{X2GoClient} instance
-        @param logger: you can pass an L{X2GoLogger} object to the L{X2GoClientXConfig} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param path: full path to pulseaudio.exe
+        :type path: ``str``
+        :param client_instance: the calling :class:`x2go.client.X2GoClient` instance
+        :type client_instance: :class:`x2go.client.X2GoClient` instance
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the :class:`x2go.xserver.X2GoClientXConfig` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
-        @raise OSNotSupportedException: on non-Windows platforms Python X2Go presumes that pulseaudio is already launched
+        :raises OSNotSupportedException: on non-Windows platforms Python X2Go presumes that pulseaudio is already launched
 
         """
         if _X2GOCLIENT_OS not in ("Windows"):
@@ -91,8 +91,9 @@ class X2GoPulseAudio(threading.Thread):
 
     def run(self):
         """\
-        This method is called once the C{X2GoPulseAudio.start()} method has been called. To tear
-        down the Pulseaudio daemon call the L{X2GoPulseAudio.stop_thread()} method.
+        This method is called once the ``X2GoPulseAudio.start()`` method has been called. To tear
+        down the Pulseaudio daemon call the :func:`X2GoPulseAudio.stop_thread() <x2go.pulseaudio.X2GoPulseAudio.stop_thread()` method.
+
 
         """
         self._keepalive = True
@@ -101,7 +102,6 @@ class X2GoPulseAudio(threading.Thread):
             '-n',
             '--exit-idle-time=-1',
             '-L "module-native-protocol-tcp port=4713 auth-cookie=\\\\.pulse-cookie"',
-            '-L "module-esound-protocol-tcp port=16001"',
             '-L module-waveout',
         ]
 
@@ -147,7 +147,7 @@ class X2GoPulseAudio(threading.Thread):
                                               )
         _is_alive = ( rc != win32event.WAIT_OBJECT_0 )
         if self.client_instance and not _is_alive:
-            if os.environ.has_key('CLIENTNAME'):
+            if 'CLIENTNAME' in os.environ:
                 self.client_instance.HOOK_pulseaudio_not_supported_in_RDPsession()
             else:
                 self.client_instance.HOOK_pulseaudio_server_startup_failed()
@@ -175,6 +175,7 @@ class X2GoPulseAudio(threading.Thread):
         """\
         Tear down a running Pulseaudio daemon.
 
+
         """
         self.logger('stop_thread() method has been called', loglevel=log.loglevel_DEBUG)
         self._keepalive = False
