@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,6 +23,9 @@ server sessions.
 """
 __NAME__ = 'x2gorevtunnel-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.rforward'
+
 # modules
 import copy
 import threading
@@ -33,33 +36,42 @@ import paramiko
 from gevent import select, socket, Timeout
 
 # Python X2Go modules
-import log
+from . import log
 
 
-def x2go_transport_tcp_handler(chan, (origin_addr, origin_port), (server_addr, server_port)):
+def x2go_transport_tcp_handler(chan, origin, server):
     """\
-    An X2Go customized TCP handler for the Paramiko/SSH C{Transport()} class.
+    An X2Go customized TCP handler for the Paramiko/SSH ``Transport()`` class.
 
     Incoming channels will be put into Paramiko's default accept queue. This corresponds to
-    the default behaviour of Paramiko's C{Transport} class.
+    the default behaviour of Paramiko's ``Transport`` class.
 
     However, additionally this handler function checks the server port of the incoming channel
     and detects if there are Paramiko/SSH reverse forwarding tunnels waiting for the incoming
-    channels. The Paramiko/SSH reverse forwarding tunnels are initiated by an L{X2GoSession} instance
+    channels. The Paramiko/SSH reverse forwarding tunnels are initiated by an :class:`x2go.session.X2GoSession` instance
     (currently supported: reverse tunneling auf audio data, reverse tunneling of SSH requests).
 
-    If the server port of an incoming Paramiko/SSH channel matches the configured port of an L{X2GoRevFwTunnel}
-    instance, this instance gets notified of the incoming channel and a new L{X2GoRevFwChannelThread} is
-    started. This L{X2GoRevFwChannelThread} then takes care of the new channel's incoming data stream.
+    If the server port of an incoming Paramiko/SSH channel matches the configured port of an :class:`x2go.rforward.X2GoRevFwTunnel`
+    instance, this instance gets notified of the incoming channel and a new :class:`x2go.rforward.X2GoRevFwChannelThread` is
+    started. This :class:`x2go.rforward.X2GoRevFwChannelThread` then takes care of the new channel's incoming data stream.
+
+    :param chan: a Paramiko channel object
+    :type chan: ``paramiko.Channel`` object
+    :param origin: host/port tuple where a connection originates from
+    :type origin: ``tuple``
+    :param server: host/port tuple where to connect to
+    :type server: ``tuple``
 
     """
+    (origin_addr, origin_port) = origin
+    (server_addr, server_port) = server
     transport = chan.get_transport()
     transport._queue_incoming_channel(chan)
     rev_tuns = transport.reverse_tunnels
 
-    for session_name in rev_tuns.keys():
+    for session_name in list(rev_tuns.keys()):
 
-        if int(server_port) in [ int(tunnel[0]) for tunnel in rev_tuns[session_name].values() ]:
+        if int(server_port) in [ int(tunnel[0]) for tunnel in list(rev_tuns[session_name].values()) ]:
 
             if rev_tuns[session_name]['snd'] is not None and int(server_port) == int(rev_tuns[session_name]['snd'][0]):
                 rev_tuns[session_name]['snd'][1].notify()
@@ -70,37 +82,38 @@ def x2go_transport_tcp_handler(chan, (origin_addr, origin_port), (server_addr, s
 
 class X2GoRevFwTunnel(threading.Thread):
     """\
-    L{X2GoRevFwTunnel} class objects are used to reversely tunnel
+    :class:`x2go.rforward.X2GoRevFwTunnel` class objects are used to reversely tunnel
     X2Go audio, X2Go printing and X2Go folder sharing / device mounting
     through Paramiko/SSH.
 
+
     """
     def __init__(self, server_port, remote_host, remote_port, ssh_transport, session_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         Setup a reverse tunnel through Paramiko/SSH.
 
-        After the reverse tunnel has been setup up with L{X2GoRevFwTunnel.start()} it waits
-        for notification from L{X2GoRevFwTunnel.notify()} to accept incoming channels. This
-        notification (L{X2GoRevFwTunnel.notify()} gets called from within the transport's
-        TCP handler function L{x2go_transport_tcp_handler} of the L{X2GoSession} instance.
+        After the reverse tunnel has been setup up with :func:`X2GoRevFwTunnel.start() <x2go.rforward.X2GoRevFwTunnel.start()>` it waits
+        for notification from :func:`X2GoRevFwTunnel.notify() <x2go.rforward.X2GoRevFwTunnel.notify()>` to accept incoming channels. This
+        notification (:func:`X2GoRevFwTunnel.notify() <x2go.rforward.X2GoRevFwTunnel.notify()>` gets called from within the transport's
+        TCP handler function :func:`x2go_transport_tcp_handler()` of the :class:`x2go.session.X2GoSession` instance.
 
-        @param server_port: the TCP/IP port on the X2Go server (starting point of the tunnel),
+        :param server_port: the TCP/IP port on the X2Go server (starting point of the tunnel),
             normally some number above 30000
-        @type server_port: int
-        @param remote_host: the target address for reversely tunneled traffic. With X2Go this should
+        :type server_port: int
+        :param remote_host: the target address for reversely tunneled traffic. With X2Go this should
             always be set to the localhost (IPv4) address.
-        @type remote_host: str
-        @param remote_port: the TCP/IP port on the X2Go client (end point of the tunnel),
+        :type remote_host: str
+        :param remote_port: the TCP/IP port on the X2Go client (end point of the tunnel),
             normally an application's standard port (22 for SSH, 4713 for pulse audio, etc.)
-        @type remote_port: int
-        @param ssh_transport: the L{X2GoSession}'s Paramiko/SSH transport instance
-        @type ssh_transport: C{paramiko.Transport} instance
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoRevFwTunnel} constructor
-        @type logger: L{X2GoLogger} instance
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :type remote_port: int
+        :param ssh_transport: the :class:`x2go.session.X2GoSession`'s Paramiko/SSH transport instance
+        :type ssh_transport: ``paramiko.Transport`` instance
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.rforward.X2GoRevFwTunnel` constructor
+        :type logger: :class:`x2go.log.X2GoLogger` instance
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: int
+        :type loglevel: int
 
         """
         if logger is None:
@@ -135,10 +148,10 @@ class X2GoRevFwTunnel(threading.Thread):
         Cancel a port forwarding request. This cancellation request is sent to the server and
         on the server the port forwarding should be unregistered.
 
-        @param address: remote server address
-        @type address: C{str}
-        @param port: remote port
-        @type port: C{int}
+        :param address: remote server address
+        :type address: ``str``
+        :param port: remote port
+        :type port: ``int``
 
         """
         timeout = Timeout(10)
@@ -153,9 +166,10 @@ class X2GoRevFwTunnel(threading.Thread):
     def pause(self):
         """\
         Prevent acceptance of new incoming connections through the Paramiko/SSH
-        reverse forwarding tunnel. Also, any active connection on this L{X2GoRevFwTunnel}
+        reverse forwarding tunnel. Also, any active connection on this :class:`x2go.rforward.X2GoRevFwTunnel`
         instance will be closed immediately, if this method is called.
 
+
         """
         if self._accept_channels == True:
             self.cancel_port_forward('', self.server_port)
@@ -167,6 +181,7 @@ class X2GoRevFwTunnel(threading.Thread):
         Resume operation of the Paramiko/SSH reverse forwarding tunnel
         and continue accepting new incoming connections.
 
+
         """
         if self._accept_channels == False:
             self._accept_channels = True
@@ -175,14 +190,15 @@ class X2GoRevFwTunnel(threading.Thread):
 
     def notify(self):
         """\
-        Notify an L{X2GoRevFwTunnel} instance of an incoming Paramiko/SSH channel.
+        Notify an :class:`x2go.rforward.X2GoRevFwTunnel` instance of an incoming Paramiko/SSH channel.
 
         If an incoming reverse tunnel channel appropriate for this instance has
-        been detected, this method gets called from the L{X2GoSession}'s transport
+        been detected, this method gets called from the :class:`x2go.session.X2GoSession`'s transport
         TCP handler.
 
-        The sent notification will trigger a C{thread.Condition()} waiting for notification
-        in L{X2GoRevFwTunnel.run()}.
+        The sent notification will trigger a ``thread.Condition()`` waiting for notification
+        in :func:`X2GoRevFwTunnel.run() <x2go.rforward.X2GoRevFwTunnel.run()>`.
+
 
         """
         self.incoming_channel.acquire()
@@ -192,7 +208,8 @@ class X2GoRevFwTunnel(threading.Thread):
 
     def stop_thread(self):
         """\
-        Stops this L{X2GoRevFwTunnel} thread completely.
+        Stops this :class:`x2go.rforward.X2GoRevFwTunnel` thread completely.
+
 
         """
         self.pause()
@@ -210,7 +227,7 @@ class X2GoRevFwTunnel(threading.Thread):
             gevent.sleep(1)
             try:
                 self._requested_port = self.ssh_transport.request_port_forward('127.0.0.1', self.server_port, handler=x2go_transport_tcp_handler)
-            except paramiko.SSHException, e:
+            except paramiko.SSHException as e:
                 if self.session_instance:
                     self.session_instance.HOOK_rforward_request_denied(server_port=self.server_port)
                 else:
@@ -218,23 +235,24 @@ class X2GoRevFwTunnel(threading.Thread):
 
     def run(self):
         """\
-        This method gets run once an L{X2GoRevFwTunnel} has been started with its
-        L{start()} method. Use L{X2GoRevFwTunnel}.stop_thread() to stop the
+        This method gets run once an :class:`x2go.rforward.X2GoRevFwTunnel` has been started with its
+        :func:`start()` method. Use :class:`x2go.rforward.X2GoRevFwTunnel`.stop_thread() to stop the
         reverse forwarding tunnel again. You can also temporarily lock the tunnel
-        down with L{X2GoRevFwTunnel.pause()} and L{X2GoRevFwTunnel.resume()}).
+        down with :func:`X2GoRevFwTunnel.pause() <x2go.rforward.X2GoRevFwTunnel.pause()>` and :func:`X2GoRevFwTunnel.resume() <x2go.rforward.X2GoRevFwTunnel.resume()>`).
 
-        L{X2GoRevFwTunnel.run()} waits for notifications of an appropriate incoming
-        Paramiko/SSH channel (issued by L{X2GoRevFwTunnel.notify()}). Appropriate in
+        :func:`X2GoRevFwTunnel.run() <x2go.rforward.X2GoRevFwTunnel.run()>` waits for notifications of an appropriate incoming
+        Paramiko/SSH channel (issued by :func:`X2GoRevFwTunnel.notify() <x2go.rforward.X2GoRevFwTunnel.notify()>`). Appropriate in
         this context means, that its start point on the X2Go server matches the class's
-        property C{server_port}.
+        property ``server_port``.
 
-        Once a new incoming channel gets announced by the L{notify()} method, a new
-        L{X2GoRevFwChannelThread} instance will be initialized. As a data stream handler,
-        the function L{x2go_rev_forward_channel_handler()} will be used.
+        Once a new incoming channel gets announced by the :func:`notify()` method, a new
+        :class:`x2go.rforward.X2GoRevFwChannelThread` instance will be initialized. As a data stream handler,
+        the function :func:`x2go_rev_forward_channel_handler()` will be used.
 
         The channel will last till the connection gets dropped on the X2Go server side or
-        until the tunnel gets paused by an L{X2GoRevFwTunnel.pause()} call or stopped via the
-        L{X2GoRevFwTunnel.stop_thread()} method.
+        until the tunnel gets paused by an :func:`X2GoRevFwTunnel.pause() <x2go.rforward.X2GoRevFwTunnel.pause()>` call or stopped via the
+        :func:`X2GoRevFwTunnel.stop_thread() <x2go.rforward.X2GoRevFwTunnel.stop_thread()>` method.
+
 
         """
         self._request_port_forwarding()
@@ -271,29 +289,29 @@ class X2GoRevFwTunnel(threading.Thread):
 
 def x2go_rev_forward_channel_handler(chan=None, addr='', port=0, parent_thread=None, logger=None, ):
     """\
-    Handle the data stream of a requested channel that got set up by a L{X2GoRevFwTunnel} (Paramiko/SSH
+    Handle the data stream of a requested channel that got set up by a :class:`x2go.rforward.X2GoRevFwTunnel` (Paramiko/SSH
     reverse forwarding tunnel).
 
     The channel (and the corresponding connections) close either ...
 
         - ... if the connecting application closes the connection and thus, drops
-        the channel, or
-        - ... if the L{X2GoRevFwTunnel} parent thread gets paused. The call
-        of L{X2GoRevFwTunnel.pause()} on the instance can be used to shut down all incoming
-        tunneled SSH connections associated to this L{X2GoRevFwTunnel} instance
-        from within a Python X2Go application.
+          the channel, or
+        - ... if the :class:`x2go.rforward.X2GoRevFwTunnel` parent thread gets paused. The call
+          of :func:`X2GoRevFwTunnel.pause() <x2go.rforward.X2GoRevFwTunnel.pause()>` on the instance can be used to shut down all incoming
+          tunneled SSH connections associated to this :class:`x2go.rforward.X2GoRevFwTunnel` instance
+          from within a Python X2Go application.
 
-    @param chan: channel
-    @type chan: C{class}
-    @param addr: bind address
-    @type addr: C{str}
-    @param port: bind port
-    @type port: C{int}
-    @param parent_thread: the calling L{X2GoRevFwTunnel} instance
-    @type parent_thread: L{X2GoRevFwTunnel} instance
-    @param logger: you can pass an L{X2GoLogger} object to the
-        L{X2GoRevFwTunnel} constructor
-    @type logger: L{X2GoLogger} instance
+    :param chan: channel (Default value = None)
+    :type chan: ``class``
+    :param addr: bind address (Default value = '')
+    :type addr: ``str``
+    :param port: bind port (Default value = 0)
+    :type port: ``int``
+    :param parent_thread: the calling :class:`x2go.rforward.X2GoRevFwTunnel` instance (Default value = None)
+    :type parent_thread: :class:`x2go.rforward.X2GoRevFwTunnel` instance
+    :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+        :class:`x2go.rforward.X2GoRevFwTunnel` constructor (Default value = None)
+    :type logger: :class:`x2go.log.X2GoLogger` instance
 
     """
     fw_socket = socket.socket()
@@ -305,7 +323,7 @@ def x2go_rev_forward_channel_handler(chan=None, addr='', port=0, parent_thread=N
 
     try:
         fw_socket.connect((addr, port))
-    except Exception, e:
+    except Exception as e:
         logger('Reverse forwarding request to %s:%d failed: %r' % (addr, port, e), loglevel=log.loglevel_INFO)
         return
 
@@ -325,7 +343,7 @@ def x2go_rev_forward_channel_handler(chan=None, addr='', port=0, parent_thread=N
                 if len(data) == 0:
                     break
                 fw_socket.send(data)
-        except socket.error, e:
+        except socket.error as e:
             logger('Reverse tunnel %s encoutered socket error: %s' % (chan, str(e)), loglevel=log.loglevel_WARN)
 
     chan.close()
@@ -338,16 +356,17 @@ class X2GoRevFwChannelThread(threading.Thread):
     Starts a thread for each incoming Paramiko/SSH data channel trough the reverse
     forwarding tunnel.
 
+
     """
     def __init__(self, channel, remote=None, **kwargs):
         """\
         Initializes a reverse forwarding channel thread.
 
-        @param channel: incoming Paramiko/SSH channel from the L{X2GoSession}'s transport
+        :param channel: incoming Paramiko/SSH channel from the :class:`x2go.session.X2GoSession`'s transport
             accept queue
-        @type channel: class
-        @param remote: tuple (addr, port) that specifies the data endpoint of the channel
-        @type remote: C{tuple(str, int)}
+        :type channel: class
+        :param remote: tuple (addr, port) that specifies the data endpoint of the channel
+        :type remote: ``tuple(str, int)``
 
         """
         self.channel = channel
