@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 
 # The Python X2Go sFTPServer code was originally written by Richard Murri,
 # for further information see his website: http://www.richardmurri.com
@@ -31,6 +31,9 @@ for further information see his website: http://www.richardmurri.com
 """
 __NAME__ = "x2gosftpserver-pylib"
 
+__package__ = 'x2go'
+__name__    = 'x2go.sftpserver'
+
 import os
 import shutil
 import copy
@@ -39,29 +42,30 @@ import paramiko
 import gevent
 
 # Python X2Go modules
-import rforward
-import defaults
-import log
+from . import rforward
+from . import defaults
+from . import log
 
 class _SSHServer(paramiko.ServerInterface):
     """\
     Implementation of a basic SSH server that is supposed
     to run with its sFTP server implementation.
 
+
     """
     def __init__(self, auth_key=None, session_instance=None, logger=None, loglevel=log.loglevel_DEFAULT, *args, **kwargs):
         """\
         Initialize a new sFTP server interface.
 
-        @param auth_key: Server key that the client has to authenticate against
-        @type auth_key: C{paramiko.RSAKey} instance
-        @param session_instance: the calling L{X2GoSession} instance
-        @type session_instance: L{X2GoSession} instance
-        @param logger: you can pass an L{X2GoLogger} object to the L{X2GoClientXConfig} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param auth_key: Server key that the client has to authenticate against
+        :type auth_key: ``paramiko.RSAKey`` instance
+        :param session_instance: the calling :class:`x2go.session.X2GoSession` instance
+        :type session_instance: :class:`x2go.session.X2GoSession` instance
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the :class:`x2go.xserver.X2GoClientXConfig` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if logger is None:
@@ -80,13 +84,12 @@ class _SSHServer(paramiko.ServerInterface):
         """\
         Only allow session requests.
 
-        @param kind: request type
-        @type kind: C{str}
-        @param chanid: channel id (unused)
-        @type chanid: C{any}
-
-        @return: returns a Paramiko/SSH return code
-        @rtype: C{int}
+        :param kind: request type
+        :type kind: ``str``
+        :param chanid: channel id (unused)
+        :type chanid: ``any``
+        :returns: returns a Paramiko/SSH return code
+        :rtype: ``int``
 
         """
         self.logger('detected a channel request for sFTP', loglevel=log.loglevel_DEBUG_SFTPXFER)
@@ -98,13 +101,12 @@ class _SSHServer(paramiko.ServerInterface):
         """\
         Ensure proper authentication.
 
-        @param username: username of incoming authentication request
-        @type username: C{str}
-        @param key: incoming SSH key to be used for authentication
-        @type key: C{paramiko.RSAKey} instance
-
-        @return: returns a Paramiko/SSH return code
-        @rtype: C{int}
+        :param username: username of incoming authentication request
+        :type username: ``str``
+        :param key: incoming SSH key to be used for authentication
+        :type key: ``paramiko.RSAKey`` instance
+        :returns: returns a Paramiko/SSH return code
+        :rtype: ``int``
 
         """
         self.logger('sFTP server %s: username is %s' % (self, self.current_local_user), loglevel=log.loglevel_DEBUG)
@@ -119,11 +121,10 @@ class _SSHServer(paramiko.ServerInterface):
         """\
         Only allow public key authentication.
 
-        @param username: username of incoming authentication request
-        @type username: C{str}
-
-        @return: statically returns C{publickey} as auth mechanism
-        @rtype: C{str}
+        :param username: username of incoming authentication request
+        :type username: ``str``
+        :returns: statically returns ``publickey`` as auth mechanism
+        :rtype: ``str``
 
         """
         self.logger('sFTP client asked for support auth methods, answering: publickey', loglevel=log.loglevel_DEBUG_SFTPXFER)
@@ -134,18 +135,21 @@ class _SFTPHandle(paramiko.SFTPHandle):
     """\
     Represents a handle to an open file.
 
+
     """
     def stat(self):
         """\
         Create an SFTPAttributes object from an existing stat object (an object returned by os.stat).
 
-        return: new C{SFTPAttributes} object with the same attribute fields.
-        rtype: C{obj}
+
+        :returns: new ``SFTPAttributes`` object with the same attribute fields.
+
+        :rtype: ``obj``
 
         """
         try:
             return paramiko.SFTPAttributes.from_stat(os.fstat(self.readfile.fileno()))
-        except OSError, e:
+        except OSError as e:
             return paramiko.SFTPServer.convert_errno(e.errno)
 
 
@@ -153,22 +157,23 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
     """\
     sFTP server implementation.
 
+
     """
     def __init__(self, server, chroot=None, logger=None, loglevel=log.loglevel_DEFAULT, server_event=None, *args, **kwargs):
         """\
         Make user information accessible as well as set chroot jail directory.
 
-        @param server: a C{paramiko.ServerInterface} instance to use with this SFTP server interface
-        @type server: C{paramiko.ServerInterface} instance
-        @param chroot: chroot environment for this SFTP interface
-        @type chroot: C{str}
-        @param logger: you can pass an L{X2GoLogger} object to the L{X2GoClientXConfig} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param server: a ``paramiko.ServerInterface`` instance to use with this SFTP server interface
+        :type server: ``paramiko.ServerInterface`` instance
+        :param chroot: chroot environment for this SFTP interface
+        :type chroot: ``str``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the :class:`x2go.xserver.X2GoClientXConfig` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
-        @param server_event: a C{threading.Event} instance that can signal SFTP session termination
-        @type server_event: C{threading.Event} instance
+        :type loglevel: ``int``
+        :param server_event: a ``threading.Event`` instance that can signal SFTP session termination
+        :type server_event: ``threading.Event`` instance
 
         """
         if logger is None:
@@ -186,11 +191,10 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         Enforce the chroot jail. On Windows systems the drive letter is incorporated in the
         chroot path name (/windrive/<drive_letter>/path/to/file/or/folder).
 
-        @param path: path name within chroot
-        @type path: C{str}
-
-        @return: real path name (including drive letter on Windows systems)
-        @rtype: C{str}
+        :param path: path name within chroot
+        :type path: ``str``
+        :returns: real path name (including drive letter on Windows systems)
+        :rtype: ``str``
 
         """
         if defaults.X2GOCLIENT_OS == 'Windows' and path.startswith('/windrive'):
@@ -208,11 +212,10 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         List the contents of a folder.
 
-        @param path: path to folder
-        @type path: C{str}
-
-        @return: returns the folder contents, on failure returns a Paramiko/SSH return code
-        @rtype: C{dict} or C{int}
+        :param path: path to folder
+        :type path: ``str``
+        :returns: returns the folder contents, on failure returns a Paramiko/SSH return code
+        :rtype: ``dict`` or ``int``
 
         """
         path = self._realpath(path)
@@ -228,12 +231,12 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
                     attr.filename = fname
                     self.logger('sFTP server %s: file attributes ok: %s' % (self, fname), loglevel=log.loglevel_DEBUG_SFTPXFER)
                     out.append(attr)
-                except OSError, e:
+                except OSError as e:
                     self.logger('sFTP server %s: encountered error processing attributes of file %s: %s' % (self, fname, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
 
             self.logger('sFTP server: folder list is : %s' % str([ a.filename for a in out ]), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return out
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
 
@@ -241,18 +244,17 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Stat on a file.
 
-        @param path: path to file/folder
-        @type path: C{str}
-
-        @return: returns the file's stat output, on failure: returns a Paramiko/SSH return code
-        @rtype: C{class} or C{int}
+        :param path: path to file/folder
+        :type path: ``str``
+        :returns: returns the file's stat output, on failure: returns a Paramiko/SSH return code
+        :rtype: ``class`` or ``int``
 
         """
         path = self._realpath(path)
         self.logger('sFTP server %s: calling stat on path: %s' % (self, path), loglevel=log.loglevel_DEBUG_SFTPXFER)
         try:
             return paramiko.SFTPAttributes.from_stat(os.stat(path))
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
 
@@ -260,18 +262,17 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         LStat on a file.
 
-        @param path: path to folder
-        @type path: C{str}
-
-        @return: returns the file's lstat output, on failure: returns a Paramiko/SSH return code
-        @rtype: C{class} or C{int}
+        :param path: path to folder
+        :type path: ``str``
+        :returns: returns the file's lstat output, on failure: returns a Paramiko/SSH return code
+        :rtype: ``class`` or ``int``
 
         """
         path = self._realpath(path)
         self.logger('sFTP server: calling lstat on path: %s' % path, loglevel=log.loglevel_DEBUG_SFTPXFER)
         try:
             return paramiko.SFTPAttributes.from_stat(os.lstat(path))
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
 
@@ -279,15 +280,14 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Open a file for reading, writing, appending etc.
 
-        @param path: path to file
-        @type path: C{str}
-        @param flags: file flags
-        @type flags: C{str}
-        @param attr: file attributes
-        @type attr: C{class}
-
-        @return: file handle/object for remote file, on failure: returns a Paramiko/SSH return code
-        @rtype: L{_SFTPHandle} instance or C{int}
+        :param path: path to file
+        :type path: ``str``
+        :param flags: file flags
+        :type flags: ``str``
+        :param attr: file attributes
+        :type attr: ``class``
+        :returns: file handle/object for remote file, on failure: returns a Paramiko/SSH return code
+        :rtype: :class:`x2go.sftpserver._SFTPHandle` instance or ``int``
 
         """
         path = self._realpath(path)
@@ -301,8 +301,8 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
             else:
                 # os.open() defaults to 0777 which is
                 # an odd default mode for files
-                fd = os.open(path, flags, 0666)
-        except OSError, e:
+                fd = os.open(path, flags, 0o666)
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
         if (flags & os.O_CREAT) and (attr is not None):
@@ -323,7 +323,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
             fstr = 'rb'
         try:
             f = os.fdopen(fd, fstr)
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
         fobj = _SFTPHandle(flags)
@@ -336,11 +336,10 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Remove a file.
 
-        @param path: path to file
-        @type path: C{str}
-
-        @return: returns Paramiko/SSH return code
-        @rtype: C{int}
+        :param path: path to file
+        :type path: ``str``
+        :returns: returns Paramiko/SSH return code
+        :rtype: ``int``
 
         """
         path = self._realpath(path)
@@ -352,13 +351,12 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Rename/move a file.
 
-        @param oldpath: old path/location/file name
-        @type oldpath: C{str}
-        @param newpath: new path/location/file name
-        @type newpath: C{str}
-
-        @return: returns Paramiko/SSH return code
-        @rtype: C{int}
+        :param oldpath: old path/location/file name
+        :type oldpath: ``str``
+        :param newpath: new path/location/file name
+        :type newpath: ``str``
+        :returns: returns Paramiko/SSH return code
+        :rtype: ``int``
 
         """
         self.logger('sFTP server %s: renaming path from %s to %s' % (self, oldpath, newpath), loglevel=log.loglevel_DEBUG_SFTPXFER)
@@ -366,7 +364,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         newpath = self._realpath(newpath)
         try:
             shutil.move(oldpath, newpath)
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
         return paramiko.SFTP_OK
@@ -375,20 +373,19 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Make a directory.
 
-        @param path: path to new folder
-        @type path: C{str}
-        @param attr: file attributes
-        @type attr: C{class}
-
-        @return: returns Paramiko/SSH return code
-        @rtype: C{int}
+        :param path: path to new folder
+        :type path: ``str``
+        :param attr: file attributes
+        :type attr: ``class``
+        :returns: returns Paramiko/SSH return code
+        :rtype: ``int``
 
         """
         self.logger('sFTP server: creating new dir (perms: %s): %s' % (attr.st_mode, path), loglevel=log.loglevel_DEBUG_SFTPXFER)
         path = self._realpath(path)
         try:
             os.mkdir(path, attr.st_mode)
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
         return paramiko.SFTP_OK
@@ -397,18 +394,17 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Remove a directory (if needed recursively).
 
-        @param path: folder to be removed
-        @type path: C{str}
-
-        @return: returns Paramiko/SSH return code
-        @rtype: C{int}
+        :param path: folder to be removed
+        :type path: ``str``
+        :returns: returns Paramiko/SSH return code
+        :rtype: ``int``
 
         """
         self.logger('sFTP server %s: removing dir: %s' % (self, path), loglevel=log.loglevel_DEBUG_SFTPXFER)
         path = self._realpath(path)
         try:
             shutil.rmtree(path)
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
         return paramiko.SFTP_OK
@@ -417,13 +413,12 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Change file attributes.
 
-        @param path: path of file/folder
-        @type path: C{str}
-        @param attr: new file attributes
-        @type attr: C{class}
-
-        @return: returns Paramiko/SSH return code
-        @rtype: C{int}
+        :param path: path of file/folder
+        :type path: ``str``
+        :param attr: new file attributes
+        :type attr: ``class``
+        :returns: returns Paramiko/SSH return code
+        :rtype: ``int``
 
         """
         self.logger('sFTP server %s: modifying attributes of path: %s' % (self, path), loglevel=log.loglevel_DEBUG_SFTPXFER)
@@ -433,7 +428,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
                 os.chmod(path, attr.st_mode)
             if attr.st_uid is not None:
                 os.chown(path, attr.st_uid, attr.st_gid)
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
         return paramiko.SFTP_OK
@@ -442,13 +437,12 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Create a symbolic link.
 
-        @param target_path: link shall point to this path
-        @type target_path: C{str}
-        @param path: link location
-        @type path: C{str}
-
-        @return: returns Paramiko/SSH return code
-        @rtype: C{int}
+        :param target_path: link shall point to this path
+        :type target_path: ``str``
+        :param path: link location
+        :type path: ``str``
+        :returns: returns Paramiko/SSH return code
+        :rtype: ``int``
 
         """
         self.logger('sFTP server %s: creating symlink from: %s to target: %s' % (self, path, target_path), loglevel=log.loglevel_DEBUG_SFTPXFER)
@@ -457,7 +451,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
             target_path = self._realpath(target_path)
         try:
             os.symlink(target_path, path)
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
         return paramiko.SFTP_OK
@@ -466,17 +460,16 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Read the target of a symbolic link.
 
-        @param path: path of symbolic link
-        @type path: C{str}
-
-        @return: target location of the symbolic link, on failure: returns a Paramiko/SSH return code
-        @rtype: C{str} or C{int}
+        :param path: path of symbolic link
+        :type path: ``str``
+        :returns: target location of the symbolic link, on failure: returns a Paramiko/SSH return code
+        :rtype: ``str`` or ``int``
 
         """
         path = self._realpath(path)
         try:
             return os.readlink(path)
-        except OSError, e:
+        except OSError as e:
             self.logger('sFTP server %s: encountered error: %s' % (self, str(e)), loglevel=log.loglevel_DEBUG_SFTPXFER)
             return paramiko.SFTPServer.convert_errno(e.errno)
 
@@ -484,6 +477,7 @@ class _SFTPServerInterface(paramiko.SFTPServerInterface):
         """\
         Tidy up when the sFTP session has ended.
 
+
         """
         if self.server_event is not None:
             self.logger('sFTP server %s: session has ended' % self, loglevel=log.loglevel_DEBUG_SFTPXFER)
@@ -496,26 +490,27 @@ class X2GoRevFwTunnelToSFTP(rforward.X2GoRevFwTunnel):
     reverse forwarding tunnel is used to provide access to local X2Go client folders
     from within the the remote X2Go server session.
 
+
     """
     def __init__(self, server_port, ssh_transport, auth_key=None, session_instance=None, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         Start a Paramiko/SSH reverse forwarding tunnel, that has an sFTP server listening at
         the endpoint of the tunnel.
 
-        @param server_port: the TCP/IP port on the X2Go server (starting point of the tunnel),
+        :param server_port: the TCP/IP port on the X2Go server (starting point of the tunnel),
             normally some number above 30000
-        @type server_port: C{int}
-        @param ssh_transport: the L{X2GoSession}'s Paramiko/SSH transport instance
-        @type ssh_transport: C{paramiko.Transport} instance
-        @param auth_key: Paramiko/SSH RSAkey object that has to be authenticated against by
+        :type server_port: ``int``
+        :param ssh_transport: the :class:`x2go.session.X2GoSession`'s Paramiko/SSH transport instance
+        :type ssh_transport: ``paramiko.Transport`` instance
+        :param auth_key: Paramiko/SSH RSAkey object that has to be authenticated against by
             the remote sFTP client
-        @type auth_key: C{paramiko.RSAKey} instance
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoRevFwTunnelToSFTP} constructor
-        @type logger: L{X2GoLogger} instance
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :type auth_key: ``paramiko.RSAKey`` instance
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.sftpserver.X2GoRevFwTunnelToSFTP` constructor
+        :type logger: :class:`x2go.log.X2GoLogger` instance
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         self.ready = False
@@ -541,22 +536,23 @@ class X2GoRevFwTunnelToSFTP(rforward.X2GoRevFwTunnel):
 
     def run(self):
         """\
-        This method gets run once an L{X2GoRevFwTunnelToSFTP} has been started with its
-        L{start()} method. Use L{X2GoRevFwTunnelToSFTP}.stop_thread() to stop the
+        This method gets run once an :class:`x2go.sftpserver.X2GoRevFwTunnelToSFTP` has been started with its
+        :func:`start()` method. Use :func:`X2GoRevFwTunnelToSFTP.stop_thread() <x2go.sftpserver.X2GoRevFwTunnelToSFTP.stop_thread()>` to stop the
         reverse forwarding tunnel again (refer also to its pause() and resume() method).
 
-        L{X2GoRevFwTunnelToSFTP.run()} waits for notifications of an appropriate incoming
-        Paramiko/SSH channel (issued by L{X2GoRevFwTunnelToSFTP.notify()}). Appropriate in
+        :func:`X2GoRevFwTunnelToSFTP.run() <x2go.rforward.X2GoRevFwTunnelToSFTP.run()>` waits for notifications of an appropriate incoming
+        Paramiko/SSH channel (issued by :func:`X2GoRevFwTunnelToSFTP.notify() <x2go.rforward.X2GoRevFwTunnelToSFTP.notify()>`). Appropriate in
         this context means, that its starting point on the X2Go server matches the class's
-        property C{server_port}.
+        property ``server_port``.
 
-        Once a new incoming channel gets announced by the L{notify()} method, a new
-        L{X2GoRevFwSFTPChannelThread} instance will be initialized. As a data stream handler,
-        the function L{x2go_rev_forward_sftpchannel_handler()} will be used.
+        Once a new incoming channel gets announced by the :func:`notify()` method, a new
+        :class:`x2go.sftpserver.X2GoRevFwSFTPChannelThread` instance will be initialized. As a data stream handler,
+        the function :func:`x2go_rev_forward_sftpchannel_handler()` will be used.
 
         The channel will last till the connection gets dropped on the X2Go server side or
-        until the tunnel gets paused by an L{X2GoRevFwTunnelToSFTP.pause()} call or
-        stopped via the C{X2GoRevFwTunnelToSFTP.stop_thread()} method.
+        until the tunnel gets paused by an :func:`X2GoRevFwTunnelToSFTP.pause() <x2go.rforward.X2GoRevFwTunnelToSFTP.pause()>` call or
+        stopped via the ``X2GoRevFwTunnelToSFTP.stop_thread()`` method.
+
 
         """
         self._request_port_forwarding()
@@ -592,24 +588,24 @@ class X2GoRevFwTunnelToSFTP(rforward.X2GoRevFwTunnel):
 
 def x2go_rev_forward_sftpchannel_handler(chan=None, auth_key=None, logger=None):
     """\
-    Handle incoming sFTP channels that got setup by an L{X2GoRevFwTunnelToSFTP} instance.
+    Handle incoming sFTP channels that got setup by an :class:`x2go.sftpserver.X2GoRevFwTunnelToSFTP` instance.
 
     The channel (and the corresponding connections) close either ...
 
         - ... if the connecting application closes the connection and thus, drops
-        the sFTP channel, or
-        - ... if the L{X2GoRevFwTunnelToSFTP} parent thread gets paused. The call
-        of L{X2GoRevFwTunnelToSFTP.pause()} on the instance can be used to shut down all incoming
-        tunneled SSH connections associated to this L{X2GoRevFwTunnelToSFTP} instance
-        from within a Python X2Go application.
+          the sFTP channel, or
+        - ... if the :class:`x2go.sftpserver.X2GoRevFwTunnelToSFTP` parent thread gets paused. The call
+          of :func:`X2GoRevFwTunnelToSFTP.pause() <x2go.rforward.X2GoRevFwTunnelToSFTP.pause()>` on the instance can be used to shut down all incoming
+          tunneled SSH connections associated to this :class:`x2go.sftpserver.X2GoRevFwTunnelToSFTP` instance
+          from within a Python X2Go application.
 
-    @param chan: an incoming sFTP channel
-    @type chan: paramiko.Channel instance
-    @param auth_key: Paramiko/SSH RSAkey object that has to be authenticated against by
-        the remote sFTP client
-    @type auth_key: C{paramiko.RSAKey} instance
-    @param logger: you must pass an L{X2GoLogger} object to this handler method
-    @type logger: C{X2GoLogger} instance
+    :param chan: an incoming sFTP channel (Default value = None)
+    :type chan: paramiko.Channel instance
+    :param auth_key: Paramiko/SSH RSAkey object that has to be authenticated against by
+        the remote sFTP client (Default value = None)
+    :type auth_key: ``paramiko.RSAKey`` instance
+    :param logger: you must pass an :class:`x2go.log.X2GoLogger` object to this handler method (Default value = None)
+    :type logger: ``X2GoLogger`` instance
 
     """
     if logger is None:
@@ -644,4 +640,4 @@ def x2go_rev_forward_sftpchannel_handler(chan=None, auth_key=None, logger=None):
 
 
 class X2GoRevFwSFTPChannelThread(rforward.X2GoRevFwChannelThread): pass
-"""A clone of L{rforward.X2GoRevFwChannelThread}."""
+"""A clone of :class:`x2go.rforward.X2GoRevFwChannelThread`."""
