@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -18,11 +18,15 @@
 # 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """\
-L{X2GoSSHProxy} class - providing a forwarding tunnel for connecting to servers behind firewalls.
+:class:`x2go.sshproxy.X2GoSSHProxy` class - providing a forwarding tunnel for connecting to servers behind firewalls.
 
 """
+from __future__ import print_function
 __NAME__ = 'x2gosshproxy-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.sshproxy'
+
 # modules
 import gevent
 import os
@@ -35,11 +39,11 @@ import string
 import random
 
 # Python X2Go modules
-import forward
-import checkhosts
-import log
-import utils
-import x2go_exceptions
+from . import forward
+from . import checkhosts
+from . import log
+from . import utils
+from . import x2go_exceptions
 
 from x2go.defaults import CURRENT_LOCAL_USER as _CURRENT_LOCAL_USER
 from x2go.defaults import LOCAL_HOME as _LOCAL_HOME
@@ -52,6 +56,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
     """\
     X2GoSSHProxy can be used to proxy X2Go connections through a firewall via SSH.
 
+
     """
     fw_tunnel = None
 
@@ -70,79 +75,79 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         a proxying SSH server (i.e. to subLANs that are separated by firewalls or to private IP subLANs that
         are NATted behind routers).
 
-        @param username: login user name to be used on the SSH proxy host
-        @type username: C{str}
-        @param password: user's password on the SSH proxy host, with private key authentication it will be
+        :param username: login user name to be used on the SSH proxy host
+        :type username: ``str``
+        :param password: user's password on the SSH proxy host, with private key authentication it will be
             used to unlock the key (if needed)
-        @type password: C{str}
-        @param passphrase: a passphrase to use for unlocking
+        :type password: ``str``
+        :param passphrase: a passphrase to use for unlocking
             a private key in case the password is already needed for two-factor
             authentication
-        @type passphrase: {str}
-        @param key_filename: name of a SSH private key file
-        @type key_filename: C{str}
-        @param pkey: a private DSA/RSA key object (as provided by Paramiko/SSH)
-        @type pkey: C{RSA/DSA key instance}
-        @param force_password_auth: enforce password authentication even if a key(file) is present
-        @type force_password_auth: C{bool}
-        @param look_for_keys: look for key files with standard names and try those if any can be found
-        @type look_for_keys: C{bool}
-        @param allow_agent: try authentication via a locally available SSH agent
-        @type allow_agent: C{bool}
-        @param local_host: bind SSH tunnel to the C{local_host} IP socket address (default: localhost)
-        @type local_host: C{str}
-        @param local_port: IP socket port to bind the SSH tunnel to (default; 22022)
-        @type local_port: C{int}
-        @param remote_host: remote endpoint of the SSH proxying/forwarding tunnel (default: localhost)
-        @type remote_host: C{str}
-        @param remote_port: remote endpoint's IP socket port for listening SSH daemon (default: 22)
-        @type remote_port: C{int}
-        @param known_hosts: full path to a custom C{known_hosts} file
-        @type known_hosts: C{str}
-        @param add_to_known_hosts: automatically add host keys of unknown SSH hosts to the C{known_hosts} file
-        @type add_to_known_hosts: C{bool}
-        @param hostname: alias for C{local_host}
-        @type hostname: C{str}
-        @param port: alias for C{local_port}
-        @type port: C{int}
-        @param sshproxy_host: alias for C{hostname}
-        @type sshproxy_host: C{str}
-        @param sshproxy_port: alias for C{post}
-        @type sshproxy_port: C{int}
-        @param sshproxy_user: alias for C{username}
-        @type sshproxy_user: C{str}
-        @param sshproxy_password: alias for C{password}
-        @type sshproxy_password: C{str}
-        @param sshproxy_passphrase: alias for C{passphrase}
-        @type sshproxy_passphrase: C{str}
-        @param sshproxy_key_filename: alias for C{key_filename}
-        @type sshproxy_key_filename: C{str}
-        @param sshproxy_pkey: alias for C{pkey}
-        @type sshproxy_pkey: C{RSA/DSA key instance} (Paramiko)
-        @param sshproxy_force_password_auth: alias for C{force_password_auth}
-        @type sshproxy_force_password_auth: C{bool}
-        @param sshproxy_look_for_keys: alias for C{look_for_keys}
-        @type sshproxy_look_for_keys: C{bool}
-        @param sshproxy_allow_agent: alias for C{allow_agent}
-        @type sshproxy_allow_agent: C{bool}
+        :type passphrase: {str}
+        :param key_filename: name of a SSH private key file
+        :type key_filename: ``str``
+        :param pkey: a private DSA/RSA key object (as provided by Paramiko/SSH)
+        :type pkey: ``RSA/DSA key instance``
+        :param force_password_auth: enforce password authentication even if a key(file) is present
+        :type force_password_auth: ``bool``
+        :param look_for_keys: look for key files with standard names and try those if any can be found
+        :type look_for_keys: ``bool``
+        :param allow_agent: try authentication via a locally available SSH agent
+        :type allow_agent: ``bool``
+        :param local_host: bind SSH tunnel to the ``local_host`` IP socket address (default: localhost)
+        :type local_host: ``str``
+        :param local_port: IP socket port to bind the SSH tunnel to (default; 22022)
+        :type local_port: ``int``
+        :param remote_host: remote endpoint of the SSH proxying/forwarding tunnel (default: localhost)
+        :type remote_host: ``str``
+        :param remote_port: remote endpoint's IP socket port for listening SSH daemon (default: 22)
+        :type remote_port: ``int``
+        :param known_hosts: full path to a custom ``known_hosts`` file
+        :type known_hosts: ``str``
+        :param add_to_known_hosts: automatically add host keys of unknown SSH hosts to the ``known_hosts`` file
+        :type add_to_known_hosts: ``bool``
+        :param hostname: alias for ``local_host``
+        :type hostname: ``str``
+        :param port: alias for ``local_port``
+        :type port: ``int``
+        :param sshproxy_host: alias for ``hostname``
+        :type sshproxy_host: ``str``
+        :param sshproxy_port: alias for ``post``
+        :type sshproxy_port: ``int``
+        :param sshproxy_user: alias for ``username``
+        :type sshproxy_user: ``str``
+        :param sshproxy_password: alias for ``password``
+        :type sshproxy_password: ``str``
+        :param sshproxy_passphrase: alias for ``passphrase``
+        :type sshproxy_passphrase: ``str``
+        :param sshproxy_key_filename: alias for ``key_filename``
+        :type sshproxy_key_filename: ``str``
+        :param sshproxy_pkey: alias for ``pkey``
+        :type sshproxy_pkey: ``RSA/DSA key instance`` (Paramiko)
+        :param sshproxy_force_password_auth: alias for ``force_password_auth``
+        :type sshproxy_force_password_auth: ``bool``
+        :param sshproxy_look_for_keys: alias for ``look_for_keys``
+        :type sshproxy_look_for_keys: ``bool``
+        :param sshproxy_allow_agent: alias for ``allow_agent``
+        :type sshproxy_allow_agent: ``bool``
 
-        @param sshproxy_tunnel: a string of the format <local_host>:<local_port>:<remote_host>:<remote_port>
-            which will override---if used---the options: C{local_host}, C{local_port}, C{remote_host} and C{remote_port}
-        @type sshproxy_tunnel: C{str}
+        :param sshproxy_tunnel: a string of the format <local_host>:<local_port>:<remote_host>:<remote_port>
+            which will override---if used---the options: ``local_host``, ``local_port``, ``remote_host`` and ``remote_port``
+        :type sshproxy_tunnel: ``str``
 
-        @param ssh_rootdir: local user's SSH base directory (default: ~/.ssh)
-        @type ssh_rootdir: C{str}
-        @param session_instance: the L{X2GoSession} instance that builds up this SSH proxying tunnel
-        @type session_instance: L{X2GoSession} instance
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoSSHProxy} constructor
-        @type logger: L{X2GoLogger} instance
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param ssh_rootdir: local user's SSH base directory (default: ~/.ssh)
+        :type ssh_rootdir: ``str``
+        :param session_instance: the :class:`x2go.session.X2GoSession` instance that builds up this SSH proxying tunnel
+        :type session_instance: :class:`x2go.session.X2GoSession` instance
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.sshproxy.X2GoSSHProxy` constructor
+        :type logger: :class:`x2go.log.X2GoLogger` instance
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: int
+        :type loglevel: int
 
-        @raise X2GoSSHProxyAuthenticationException: if the SSH proxy caused a C{paramiko.AuthenticationException}
-        @raise X2GoSSHProxyException: if the SSH proxy caused a C{paramiko.SSHException}
+        :raises X2GoSSHProxyAuthenticationException: if the SSH proxy caused a ``paramiko.AuthenticationException``
+        :raises X2GoSSHProxyException: if the SSH proxy caused a ``paramiko.SSHException``
         """
         if logger is None:
             self.logger = log.X2GoLogger(loglevel=loglevel)
@@ -150,9 +155,9 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
             self.logger = copy.deepcopy(logger)
         self.logger.tag = __NAME__
 
-        if hostname and hostname in (types.UnicodeType, types.StringType):
+        if hostname and hostname in (str, bytes):
             hostname = [hostname]
-        if hostname and hostname in (types.ListType, types.TupleType):
+        if hostname and hostname in (list, tuple):
             hostname = random.choice(list(hostname))
         self.hostname, self.port, self.username = hostname, port, username
 
@@ -161,9 +166,9 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         # translate between X2GoSession options and paramiko.SSHCLient.connect() options
         # if <hostname>:<port> is used for sshproxy_host, then this <port> is used
         if sshproxy_host:
-            if sshproxy_host and type(sshproxy_host) in (types.UnicodeType, types.StringType):
+            if sshproxy_host and type(sshproxy_host) in (str, bytes):
                 sshproxy_host = [sshproxy_host]
-            if type(sshproxy_host) in (types.ListType, types.TupleType):
+            if type(sshproxy_host) in (list, tuple):
                 sshproxy_host = random.choice(list(sshproxy_host))
             if sshproxy_host.find(':'):
                 self.hostname = sshproxy_host.split(':')[0]
@@ -185,15 +190,15 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
             self.local_port = int(self.local_port)
             self.remote_port = int(self.remote_port)
         else:
-            if local_host and type(local_host) in (types.UnicodeType, types.StringType):
+            if local_host and type(local_host) in (str, bytes):
                 local_host = [local_host]
-            if local_host and type(local_host) in (types.ListType, types.TupleType):
+            if local_host and type(local_host) in (list, tuple):
                 local_host = random.choice(list(local_host))
-            if remote_host and type(remote_host) in (types.UnicodeType, types.StringType):
+            if remote_host and type(remote_host) in (str, bytes):
                 remote_host = [remote_host]
-            if remote_host and type(remote_host) in (types.ListType, types.TupleType):
+            if remote_host and type(remote_host) in (list, tuple):
                 remote_host = random.choice(remote_host)
-            print "LOCAL_HOST: ", local_host
+            print(("LOCAL_HOST: ", local_host))
             self.local_host = local_host
             self.local_port = int(local_port)
             self.remote_host =  remote_host
@@ -226,7 +231,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         if username is None:
             username = _CURRENT_LOCAL_USER
 
-        if type(password) not in (types.StringType, types.UnicodeType):
+        if type(password) not in (bytes, str):
             password = ''
 
         self._keepalive = True
@@ -279,7 +284,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
                                      allow_agent=allow_agent,
                                     )
 
-                except (paramiko.PasswordRequiredException, paramiko.SSHException), e:
+                except (paramiko.PasswordRequiredException, paramiko.SSHException) as e:
                     self.close()
                     if type(e) == paramiko.SSHException and str(e).startswith('Two-factor authentication requires a password'):
                         self.logger('SSH proxy host requests two-factor authentication', loglevel=log.loglevel_NOTICE)
@@ -328,7 +333,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
                                                  look_for_keys=look_for_keys,
                                                  allow_agent=allow_agent,
                                                 )
-                        except x2go_exceptions.AuthenticationException, auth_e:
+                        except x2go_exceptions.AuthenticationException as auth_e:
                             raise x2go_exceptions.X2GoSSHProxyAuthenticationException(str(auth_e))
 
                     else:
@@ -370,7 +375,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
                     self.close()
                     raise
 
-        except (x2go_exceptions.SSHException, IOError), e:
+        except (x2go_exceptions.SSHException, IOError) as e:
             self.close()
             raise x2go_exceptions.X2GoSSHProxyException(str(e))
         except:
@@ -386,6 +391,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         """\
         Wraps around a Paramiko/SSH host key check.
 
+
         """
         _hostname = self.hostname
 
@@ -403,7 +409,8 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         """\
         Start the SSH proxying tunnel...
 
-        @raise X2GoSSHProxyException: if the SSH proxy could not retrieve an SSH transport for proxying a X2Go server-client connection
+
+        :raises X2GoSSHProxyException: if the SSH proxy could not retrieve an SSH transport for proxying a X2Go server-client connection
 
         """
         if self.get_transport() is not None and self.get_transport().is_authenticated():
@@ -427,8 +434,10 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         """\
         Retrieve the local IP socket address this SSH proxying tunnel is (about to) bind/bound to.
 
-        @return: local IP socket address
-        @rtype: C{str}
+
+        :returns: local IP socket address
+
+        :rtype: ``str``
 
         """
         return self.local_host
@@ -437,8 +446,10 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         """\
         Retrieve the local IP socket port this SSH proxying tunnel is (about to) bind/bound to.
 
-        @return: local IP socket port
-        @rtype: C{int}
+
+        :returns: local IP socket port
+
+        :rtype: ``int``
 
         """
         return self.local_port
@@ -447,8 +458,10 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         """\
         Retrieve the remote IP socket address at the remote end of the SSH proxying tunnel.
 
-        @return: remote IP socket address
-        @rtype: C{str}
+
+        :returns: remote IP socket address
+
+        :rtype: ``str``
 
         """
         return self.remote_host
@@ -457,8 +470,10 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         """\
         Retrieve the remote IP socket port of the target system's SSH daemon.
 
-        @return: remote SSH port
-        @rtype: C{int}
+
+        :returns: remote SSH port
+
+        :rtype: ``int``
 
         """
         return self.remote_port
@@ -467,6 +482,7 @@ class X2GoSSHProxy(paramiko.SSHClient, threading.Thread):
         """\
         Tear down the SSH proxying tunnel.
 
+
         """
         if self.fw_tunnel is not None and self.fw_tunnel.is_active:
             self.logger('taking down SSH proxy tunnel via [%s]:%s' % (self.hostname, self.port), loglevel=log.loglevel_NOTICE)
