@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,6 +23,9 @@ X2GoTelekinesisClient class - Connect to Telekinesis Server on X2Go Server.
 """
 __NAME__ = 'x2gotelekinesisclient-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.telekinesis'
+
 # modules
 import gevent
 import os
@@ -54,6 +57,7 @@ class X2GoTelekinesisClient(threading.Thread):
     This class implements the startup of the telekinesis client used by
     Python X2Go.
 
+
     """
     TEKICLIENT_CMD = 'telekinesis-client'
     """Telekinesis client command. Might be OS specific."""
@@ -68,21 +72,21 @@ class X2GoTelekinesisClient(threading.Thread):
                  session_instance=None,
                  logger=None, loglevel=log.loglevel_DEFAULT, ):
         """\
-        @param session_info: session information provided as an C{X2GoServerSessionInfo*} backend
+        :param session_info: session information provided as an ``X2GoServerSessionInfo*`` backend
             instance
-        @type session_info: C{X2GoServerSessionInfo*} instance
-        @param ssh_transport: SSH transport object from C{paramiko.SSHClient}
-        @type ssh_transport: C{paramiko.Transport} instance
-        @param sessions_rootdir: base dir where X2Go session files are stored (by default: ~/.x2go)
-        @type sessions_rootdir: C{str}
-        @param logger: you can pass an L{X2GoLogger} object to the
-            L{X2GoTelekinesisClient} constructor
-        @param session_instance: the L{X2GoSession} instance this C{X2GoProxy*} instance belongs to
-        @type session_instance: L{X2GoSession} instance
-        @type logger: L{X2GoLogger} instance
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :type session_info: ``X2GoServerSessionInfo*`` instance
+        :param ssh_transport: SSH transport object from ``paramiko.SSHClient``
+        :type ssh_transport: ``paramiko.Transport`` instance
+        :param sessions_rootdir: base dir where X2Go session files are stored (by default: ~/.x2go)
+        :type sessions_rootdir: ``str``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the
+            :class:`x2go.telekinesis.X2GoTelekinesisClient` constructor
+        :param session_instance: the :class:`x2go.session.X2GoSession` instance this ``X2GoProxy*`` instance belongs to
+        :type session_instance: :class:`x2go.session.X2GoSession` instance
+        :type logger: :class:`x2go.log.X2GoLogger` instance
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: int
+        :type loglevel: int
 
         """
         self.tekiclient_log_stdout = None
@@ -137,8 +141,10 @@ class X2GoTelekinesisClient(threading.Thread):
         """\
         Test if the Telekinesis client command is installed on this machine.
 
-        @return: C{True} if the Telekinesis client command is available
-        @rtype: C{bool}
+
+        :returns: ``True`` if the Telekinesis client command is available
+
+        :rtype: ``bool``
 
         """
         ###
@@ -154,12 +160,13 @@ class X2GoTelekinesisClient(threading.Thread):
         Close any left open port forwarding tunnel, also close Telekinesis client's log file,
         if left open.
 
+
         """
         if self.tekiclient:
             self.logger('Shutting down Telekinesis client subprocess', loglevel=log.loglevel_DEBUG)
             try:
                 self.tekiclient.kill()
-            except OSError, e:
+            except OSError as e:
                 self.logger('Telekinesis client shutdown gave a message that we may ignore: %s' % str(e), loglevel=log.loglevel_WARN)
             self.tekiclient = None
 
@@ -196,6 +203,7 @@ class X2GoTelekinesisClient(threading.Thread):
         """\
         End the thread runner and tidy up.
 
+
         """
         self._keepalive = False
         # wait for thread loop to finish...
@@ -212,13 +220,14 @@ class X2GoTelekinesisClient(threading.Thread):
         Paramiko/SSH based forwarding tunnel (openssh -L option). This tunnel
         gets started here and is forked into background (Greenlet/gevent).
 
+
         """
         self._keepalive = True
         self.tekiclient = None
 
         try:
             os.makedirs(self.session_info.local_container)
-        except OSError, e:
+        except OSError as e:
             if e.errno == 17:
                 # file exists
                 pass
@@ -262,7 +271,7 @@ class X2GoTelekinesisClient(threading.Thread):
         try:
             p.terminate()
             self.logger('terminating Telekinesis client: %s' % p, loglevel=log.loglevel_DEBUG)
-        except OSError, e:
+        except OSError as e:
             if e.errno == 3:
                 # No such process
                 pass
@@ -279,6 +288,7 @@ class X2GoTelekinesisClient(threading.Thread):
         """\
         Generate the NX proxy command line for execution.
 
+
         """
         cmd_line = [ self.TEKICLIENT_CMD, ]
         _tekiclient_args = " ".join(self.TEKICLIENT_ARGS).format(sid=self.session_name).split(' ')
@@ -289,8 +299,10 @@ class X2GoTelekinesisClient(threading.Thread):
         """\
         Start the thread runner and wait for the Telekinesis client to come up.
 
-        @return: a subprocess instance that knows about the externally started Telekinesis client command.
-        @rtype: C{obj}
+
+        :returns: a subprocess instance that knows about the externally started Telekinesis client command.
+
+        :rtype: ``obj``
 
         """
         self.logger('starting local Telekinesis client...', loglevel=log.loglevel_INFO)
@@ -320,7 +332,7 @@ class X2GoTelekinesisClient(threading.Thread):
         self.tekiclient_datalog_stderr = open('%s/%s' % (self.session_info.local_container, self.tekiclient_datalog, ), 'a')
         try:
             os.makedirs(os.path.normpath('/tmp/.x2go-{local_user}/telekinesis/S-{sid}/'.format(local_user=_CURRENT_LOCAL_USER, sid=self.session_name)))
-        except OSError, e:
+        except OSError as e:
             if e.errno == 17:
                 # file exists
                 pass
@@ -352,7 +364,7 @@ class X2GoTelekinesisClient(threading.Thread):
                                                       stdout=self.tekiclient_datalog_stdout,
                                                       stderr=self.tekiclient_datalog_stderr,
                                                       shell=False)
-        except OSError, e:
+        except OSError as e:
             if e.errno == 2:
                 self.logger("The 'sshfs' command is not available on your client machine, please install it to get Telekinesis up and running!!!", loglevel=log.loglevel_WARN)
             else:
@@ -400,8 +412,10 @@ class X2GoTelekinesisClient(threading.Thread):
         """\
         Check if a proxy instance is up and running.
 
-        @return: Proxy state, C{True} for proxy being up-and-running, C{False} otherwise
-        @rtype C{bool}
+
+        :returns: Proxy state, ``True`` for proxy being up-and-running, ``False`` otherwise
+
+        :rtype: ``bool``
 
         """
         return bool(self.tekiclient and self.tekiclient.poll() is None) and self.fw_ctrl_tunnel.is_active and self.fw_data_tunnel.is_active
