@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,6 +23,9 @@ Python X2Go helper functions, constants etc.
 """
 __NAME__ = 'x2goutils-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.utils'
+
 import sys
 import os
 import locale
@@ -37,18 +40,18 @@ import distutils.version
 import paramiko
 
 # Python X2Go modules
-from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
-from defaults import X2GO_SESSIONPROFILE_DEFAULTS as _X2GO_SESSIONPROFILE_DEFAULTS
-from defaults import X2GO_MIMEBOX_ACTIONS as _X2GO_MIMEBOX_ACTIONS
-from defaults import pack_methods_nx3
+from .defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
+from .defaults import X2GO_SESSIONPROFILE_DEFAULTS as _X2GO_SESSIONPROFILE_DEFAULTS
+from .defaults import X2GO_MIMEBOX_ACTIONS as _X2GO_MIMEBOX_ACTIONS
+from .defaults import pack_methods_nx3
 
-from defaults import BACKENDS as _BACKENDS
+from .defaults import BACKENDS as _BACKENDS
 
-import x2go_exceptions
+from . import x2go_exceptions
 
 if _X2GOCLIENT_OS != 'Windows':
     import Xlib
-    from defaults import X_DISPLAY as _X_DISPLAY
+    from .defaults import X_DISPLAY as _X_DISPLAY
 
 if _X2GOCLIENT_OS == 'Windows':
     import win32gui
@@ -60,8 +63,10 @@ def is_in_nx3packmethods(method):
     """\
     Test if a given compression method is valid for NX3 Proxy.
 
-    @return: C{True} if C{method} is in the hard-coded list of NX3 compression methods.
-    @rtype: C{bool}
+    :param method: name of an NXv3 pack method
+    :type method: ``str``
+    :returns: ``True`` if ``method`` is in the hard-coded list of NX3 compression methods.
+    :rtype: ``bool``
 
     """
     return method in pack_methods_nx3
@@ -70,15 +75,14 @@ def is_in_nx3packmethods(method):
 def find_session_line_in_x2golistsessions(session_name, stdout):
     """\
     Return the X2Go session meta information as returned by the
-    C{x2golistsessions} server command for session C{session_name}.
+    ``x2golistsessions`` server command for session ``session_name``.
 
-    @param session_name: name of a session
-    @type session_name: C{str}
-    @param stdout: raw output from the ,,x2golistsessions'' command, as list of strings
-    @type stdout: C{list}
-
-    @return: the output line that contains C{<session_name>}
-    @rtype: C{str} or C{None}
+    :param session_name: name of a session
+    :type session_name: ``str``
+    :param stdout: raw output from the ,,x2golistsessions'' command, as list of strings
+    :type stdout: ``list``
+    :returns: the output line that contains ``<session_name>``
+    :rtype: ``str`` or ``None``
 
     """
     sessions = stdout.read().split("\n")
@@ -96,15 +100,14 @@ def slugify(value):
     Normalizes string, converts to lowercase, removes non-alpha characters,
     converts spaces to hyphens and replaces round brackets by pointed brackets.
 
-    @param value: a string that shall be sluggified
-    @type value: C{str}
-
-    @return: the sluggified string
-    @rtype: C{str}
+    :param value: a string that shall be sluggified
+    :type value: ``str``
+    :returns: the sluggified string
+    :rtype: ``str``
 
     """
     import unicodedata
-    value = unicodedata.normalize('NFKD', unicode(value)).encode('ascii', 'ignore')
+    value = unicodedata.normalize('NFKD', str(value)).encode('ascii', 'ignore')
     value = re.sub('[^\w\s-]', '', value).strip().lower()
     value = re.sub('[(]', '<', value).strip().lower()
     value = re.sub('[)]', '>', value).strip().lower()
@@ -114,8 +117,10 @@ def _genSessionProfileId():
     """\
     Generate a session profile ID as used in x2goclient's sessions config file.
 
-    @return: profile ID
-    @rtype: C{str}
+
+    :returns: profile ID
+
+    :rtype: ``str``
 
     """
     import datetime
@@ -126,19 +131,18 @@ def _checkIniFileDefaults(data_structure):
     """\
     Check an ini file data structure passed on by a user app or class.
 
-    @param data_structure: an ini file date structure
-    @type data_structure: C{dict} of C{dict}s
-
-    @return: C{True} if C{data_structure} matches that of an ini file data structure
-    @rtype: C{bool}
+    :param data_structure: an ini file date structure
+    :type data_structure: ``dict`` of ``dict``s
+    :returns: ``True`` if ``data_structure`` matches that of an ini file data structure
+    :rtype: ``bool``
 
     """
     if data_structure is None:
         return False
-    if type(data_structure) is not types.DictType:
+    if type(data_structure) is not dict:
         return False
-    for sub_dict in data_structure.values():
-        if type(sub_dict) is not types.DictType:
+    for sub_dict in list(data_structure.values()):
+        if type(sub_dict) is not dict:
             return False
     return True
 
@@ -147,16 +151,15 @@ def _checkSessionProfileDefaults(data_structure):
     """\
     Check the data structure of a default session profile passed by a user app.
 
-    @param data_structure: an ini file date structure
-    @type data_structure: C{dict} of C{dict}s
-
-    @return: C{True} if C{data_structure} matches that of an ini file data structure
-    @rtype: C{bool}
+    :param data_structure: an ini file date structure
+    :type data_structure: ``dict`` of ``dict``s
+    :returns: ``True`` if ``data_structure`` matches that of an ini file data structure
+    :rtype: ``bool``
 
     """
     if data_structure is None:
         return False
-    if type(data_structure) is not types.DictType:
+    if type(data_structure) is not dict:
         return False
     return True
 
@@ -166,18 +169,17 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
     Convert session profile options as used in x2goclient's sessions file to
     Python X2Go session parameters.
 
-    @param options: a dictionary of options, parameter names as in the X2Go ,,sessions'' file
-    @type options: C{dict}
-
-    @return: session options as used in C{X2GoSession} instances
-    @rtype: C{dict}
+    :param options: a dictionary of options, parameter names as in the X2Go ,,sessions'' file
+    :type options: ``dict``
+    :returns: session options as used in ``X2GoSession`` instances
+    :rtype: ``dict``
 
     """
     _params = copy.deepcopy(options)
 
     # get rid of unknown session profile options
-    _known_options = _X2GO_SESSIONPROFILE_DEFAULTS.keys()
-    for p in _params.keys():
+    _known_options = list(_X2GO_SESSIONPROFILE_DEFAULTS.keys())
+    for p in list(_params.keys()):
         if p not in _known_options:
             del _params[p]
 
@@ -229,10 +231,10 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
             '4': 'lan',
     }
 
-    for opt, val in options.iteritems():
+    for opt, val in list(options.items()):
 
         # rename options if necessary
-        if opt in _rename_dict.keys():
+        if opt in list(_rename_dict.keys()):
             del _params[opt]
             opt = _rename_dict[opt]
             if opt in _known_options:
@@ -244,14 +246,14 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
         # translate integer values for connection speed to readable strings
         if opt == 'link':
             val = str(val).lower()
-            if val in _speed_dict.keys():
+            if val in list(_speed_dict.keys()):
                 val = _speed_dict[val]
             val = val.lower()
             _params['link'] = val
 
         # share_local_folders is a list
         if opt in ('share_local_folders', 'mimebox_extensions'):
-            if type(val) is types.StringType:
+            if type(val) is bytes:
                 if val:
                     _params[opt] = val.split(',')
                 else:
@@ -271,14 +273,14 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
 
     del _params['fstunnel']
 
-    if _params.has_key('export'):
+    if 'export' in _params:
 
         _export = _params['export']
         del _params['export']
-        if type(_export) is types.DictType:
+        if type(_export) is dict:
 
             # since Python X2Go 0.5.0.0
-            _params['share_local_folders'] = _export.keys()
+            _params['share_local_folders'] = list(_export.keys())
 
         else:
 
@@ -318,7 +320,7 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
         _params['session_type'] = 'application'
     del _params['rootless']
 
-    if _params['mimebox_action'] not in _X2GO_MIMEBOX_ACTIONS.keys():
+    if _params['mimebox_action'] not in list(_X2GO_MIMEBOX_ACTIONS.keys()):
         _params['mimebox_action'] = 'OPEN'
 
     if not options['usekbd']:
@@ -346,7 +348,7 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
     if _params['use_sshproxy']:
 
         # compat code for Python X2Go 0.2.1.0 -> 0.2.2.0
-        if options.has_key('sshproxytunnel'):
+        if 'sshproxytunnel' in options:
             if not options['sshproxytunnel'].startswith('DEPRECATED'):
                 _params['server'] = options['sshproxytunnel'].split(":")[-2]
                 _params['port'] = options['sshproxytunnel'].split(":")[-1]
@@ -360,7 +362,6 @@ def _convert_SessionProfileOptions_2_SessionParams(options):
             'soundtunnel',
             'defsndport',
             'icon',
-            'xinerama',
             'multidisp',
             'display',
             'krblogin',
@@ -380,14 +381,13 @@ def session_names_by_timestamp(session_infos):
     """\
     Sorts session profile names by their timestamp (as used in the file format's section name).
 
-    @param session_infos: a dictionary of session infos as reported by L{X2GoClient.list_sessions()}
-    @type session_infos: C{dict}
-
-    @return: a timestamp-sorted list of session names found in C{session_infos}
-    @rtype: C{list}
+    :param session_infos: a dictionary of session infos as reported by :func:`X2GoClient.list_sessions() <x2go.client.X2GoClient.list_sessions()>`
+    :type session_infos: ``dict``
+    :returns: a timestamp-sorted list of session names found in ``session_infos``
+    :rtype: ``list``
 
     """
-    session_names = session_infos.keys()
+    session_names = list(session_infos.keys())
     sortable_session_names = [ '%s|%s' % (session_name.split('-')[-1].split('_')[0], session_name) for session_name in session_names ]
     sortable_session_names.sort()
     return [ session_name.split('|')[1] for session_name in sortable_session_names ]
@@ -397,14 +397,14 @@ def touch_file(filename, mode='a'):
     """\
     Imitates the behaviour of the GNU/touch command.
 
-    @param filename: name of the file to touch
-    @type filename: C{str}
-    @param mode: the file mode (as used for Python file objects)
-    @type mode: C{str}
+    :param filename: name of the file to touch
+    :type filename: ``str``
+    :param mode: the file mode (as used for Python file objects) (Default value = 'a')
+    :type mode: ``str``
 
     """
     if not os.path.isdir(os.path.dirname(filename)):
-        os.makedirs(os.path.dirname(filename), mode=00700)
+        os.makedirs(os.path.dirname(filename), mode=0o0700)
     f = open(filename, mode=mode)
     f.close()
 
@@ -413,11 +413,10 @@ def unique(seq):
     """\
     Imitates the behaviour of the GNU/uniq command.
 
-    @param seq: a list/sequence containing consecutive duplicates.
-    @type seq: C{list}
-
-    @return: list that has been clean up from the consecutive duplicates
-    @rtype: C{list}
+    :param seq: a list/sequence containing consecutive duplicates.
+    :type seq: ``list``
+    :returns: list that has been clean up from the consecutive duplicates
+    :rtype: ``list``
 
     """
     # order preserving
@@ -431,11 +430,12 @@ def known_encodings():
     Render a list of all-known-to-Python character encodings (including
     all known aliases)
 
+
     """
     from encodings.aliases import aliases
     _raw_encname_list = []
-    _raw_encname_list.extend(aliases.keys())
-    _raw_encname_list.extend(aliases.values())
+    _raw_encname_list.extend(list(aliases.keys()))
+    _raw_encname_list.extend(list(aliases.values()))
     _raw_encname_list.sort()
     _encname_list = []
     for _raw_encname in _raw_encname_list:
@@ -451,10 +451,10 @@ def patiently_remove_file(dirname, filename):
     """\
     Try to remove a file, wait for unlocking, remove it once removing is possible...
 
-    @param dirname: directory name the file is in
-    @type dirname: C{str}
-    @param filename: name of the file to be removed
-    @type filename: C{str}
+    :param dirname: directory name the file is in
+    :type dirname: ``str``
+    :param filename: name of the file to be removed
+    :type filename: ``str``
 
     """
     _not_removed = True
@@ -471,13 +471,12 @@ def detect_unused_port(bind_address='127.0.0.1', preferred_port=None):
     """\
     Detect an unused IP socket.
 
-    @param bind_address: IP address to bind to
-    @type bind_address: C{str}
-    @param preferred_port: IP socket port that shall be tried first for availability
-    @type preferred_port: C{str}
-
-    @return: free local IP socket port that can be used for binding
-    @rtype: C{str}
+    :param bind_address: IP address to bind to (Default value = '127.0.0.1')
+    :type bind_address: ``str``
+    :param preferred_port: IP socket port that shall be tried first for availability (Default value = None)
+    :type preferred_port: ``str``
+    :returns: free local IP socket port that can be used for binding
+    :rtype: ``str``
 
     """
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
@@ -497,8 +496,10 @@ def get_encoding():
     """\
     Detect systems default character encoding.
 
-    @return: The system's local character encoding.
-    @rtype: C{str}
+
+    :returns: The system's local character encoding.
+
+    :rtype: ``str``
 
     """
     try:
@@ -517,11 +518,10 @@ def is_abs_path(path):
     """\
     Test if a given path is an absolute path name.
 
-    @param path: test this path for absolutism...
-    @type path: C{str}
-
-    @return: Returns C{True} if path is an absolute path name
-    @rtype: C{bool}
+    :param path: test this path for absolutism...
+    :type path: ``str``
+    :returns: Returns ``True`` if path is an absolute path name
+    :rtype: ``bool``
 
     """
     return bool((path.startswith('/') or re.match('^[%s]\:\\\\' % string.ascii_letters, path)))
@@ -531,8 +531,10 @@ def xkb_rules_names():
     """\
     Wrapper for: xprop -root _XKB_RULES_NAMES
 
-    @return: A Python dictionary that contains the current X11 keyboard rules.
-    @rtype: C{dict}
+
+    :returns: A Python dictionary that contains the current X11 keyboard rules.
+
+    :rtype: ``dict``
 
     """
     p = subprocess.Popen(['xprop', '-root', '_XKB_RULES_NAMES',], stdout=subprocess.PIPE, )
@@ -550,14 +552,16 @@ def local_color_depth():
     """\
     Detect the current local screen's color depth.
 
-    @return: the local color depth in bits
-    @rtype: C{int}
+
+    :returns: the local color depth in bits
+
+    :rtype: ``int``
 
     """
     if _X2GOCLIENT_OS != 'Windows':
         try:
             p = subprocess.Popen(['xwininfo', '-root',], stdout=subprocess.PIPE, )
-            _depth_line = [ _info.strip() for _info in p.stdout.read().split('\n') if 'Depth:' in _info ][0]
+            _depth_line = [ _info.strip() for _info in p.stdout.read().decode().split('\n') if 'Depth:' in _info ][0]
             _depth = _depth_line.split(' ')[1]
             return int(_depth)
         except IndexError:
@@ -579,13 +583,12 @@ def is_color_depth_ok(depth_session, depth_local):
     Test if color depth of this session is compatible with the
     local screen's color depth.
 
-    @param depth_session: color depth of the session
-    @type depth_session: C{int}
-    @param depth_local: color depth of local screen
-    @type depth_local: C{int}
-
-    @return: Does the session color depth work with the local display?
-    @rtype: C{bool}
+    :param depth_session: color depth of the session
+    :type depth_session: ``int``
+    :param depth_local: color depth of local screen
+    :type depth_local: ``int``
+    :returns: Does the session color depth work with the local display?
+    :rtype: ``bool``
 
     """
     if depth_session == 0:
@@ -603,11 +606,10 @@ def find_session_window(session_name):
     """\
     Find a session window by its X2GO session ID.
 
-    @param session_name: session name/ID of an X2Go session window
-    @type session_name: C{str}
-
-    @return: the window object (or ID) of the searched for session window
-    @rtype: C{obj} on Unix, C{int} on Windows
+    :param session_name: session name/ID of an X2Go session window
+    :type session_name: ``str``
+    :returns: the window object (or ID) of the searched for session window
+    :rtype: ``obj`` on Unix, ``int`` on Windows
 
     """
     if _X2GOCLIENT_OS != 'Windows':
@@ -658,8 +660,10 @@ def get_desktop_geometry():
     """\
     Get the geometry of the current screen's desktop.
 
-    @return: a (<width>, <height>) tuple will be returned
-    @rtype: C{tuple}
+
+    :returns: a (<width>, <height>) tuple will be returned
+
+    :rtype: ``tuple``
 
     """
     if _X2GOCLIENT_OS != 'Windows':
@@ -677,13 +681,18 @@ def get_workarea_geometry():
 
         xprop -root '_NET_WORKAREA'
 
-    @return: a (<width>, <height>) tuple will be returned
-    @rtype: C{tuple}
+
+    :returns: a (<width>, <height>) tuple will be returned
+
+    :rtype: ``tuple``
 
     """
     if _X2GOCLIENT_OS != 'Windows':
         p = subprocess.Popen(['xprop', '-root', '_NET_WORKAREA',], stdout=subprocess.PIPE, )
-        _list = p.stdout.read().rstrip('\n').split(',')
+        if sys.version_info[0] >= 3:
+            _list = p.stdout.read().decode().rstrip('\n').split(',')
+        else:
+            _list = p.stdout.read().rstrip('\n').split(',')
         if len(_list) == 4:
             return (_list[2].strip(), _list[3].strip())
         else:
@@ -697,10 +706,10 @@ def set_session_window_title(session_window, session_title):
     """\
     Set title of session window.
 
-    @param session_window: session window instance
-    @type session_window: C{obj}
-    @param session_title: session title to be set for C{session_window}
-    @type session_title: C{str}
+    :param session_window: session window instance
+    :type session_window: ``obj``
+    :param session_title: session title to be set for ``session_window``
+    :type session_title: ``str``
 
     """
     if _X2GOCLIENT_OS != 'Windows':
@@ -719,8 +728,8 @@ def raise_session_window(session_window):
     """\
     Raise session window. Not functional for Unix-like operating systems.
 
-    @param session_window: session window instance
-    @type session_window: C{obj}
+    :param session_window: session window instance
+    :type session_window: ``obj``
 
     """
     if _X2GOCLIENT_OS != 'Windows':
@@ -734,13 +743,12 @@ def merge_ordered_lists(l1, l2):
     """\
     Merge sort two sorted lists
 
-    @param l1: first sorted list
-    @type l1: C{list}
-    @param l2: second sorted list
-    @type l2: C{list}
-
-    @return: the merge result of both sorted lists
-    @rtype: C{list}
+    :param l1: first sorted list
+    :type l1: ``list``
+    :param l2: second sorted list
+    :type l2: ``list``
+    :returns: the merge result of both sorted lists
+    :rtype: ``list``
 
     """
     ordered_list = []
@@ -769,15 +777,15 @@ def merge_ordered_lists(l1, l2):
 def compare_versions(version_a, op, version_b):
     """\
     Compare <version_a> with <version_b> using operator <op>.
-    In the background C{distutils.version.LooseVersion} is
+    In the background ``distutils.version.LooseVersion`` is
     used for the comparison operation.
 
-    @param version_a: a version string
-    @type version_a: C{str}
-    @param op: an operator provide as string (e.g. '<', '>', '==', '>=' etc.)
-    @type op: C{str}
-    @param version_b: another version string that is to be compared with <version_a>
-    @type version_b: C{str}
+    :param version_a: a version string
+    :type version_a: ``str``
+    :param op: an operator provide as string (e.g. '<', '>', '==', '>=' etc.)
+    :type op: ``str``
+    :param version_b: another version string that is to be compared with <version_a>
+    :type version_b: ``str``
 
     """
 
@@ -792,13 +800,14 @@ class ProgressStatus(object):
     """\
     A simple progress status iterator class.
 
+
     """
-    def __init__(self, progress_event, progress_func=range(0, 100, 10)):
+    def __init__(self, progress_event, progress_func=list(range(0, 100, 10))):
         """\
-        @param progress_event: a threading.Event() object that gets notified on progress
-        @type progress_event: C{obj}
-        @param progress_func: a function that delivers a value between 0 and 100 (progress percentage value)
-        @type progress_func: C{func}
+        :param progress_event: a threading.Event() object that gets notified on progress
+        :type progress_event: ``obj``
+        :param progress_func: a function that delivers a value between 0 and 100 (progress percentage value)
+        :type progress_func: ``func``
 
         """
         self.ev = progress_event
@@ -806,13 +815,13 @@ class ProgressStatus(object):
 
     def __iter__(self):
         """\
-        Intialize the L{ProgressStatus} iterator object.
+        Intialize the :class:`x2go.utils.ProgressStatus` iterator object.
 
         """
         self.status = self.progress_func()
         return self
 
-    def next(self):
+    def __next__(self):
         """\
         On each iteration wait for the progress event to get triggered from an outside
         part of the application.
@@ -832,13 +841,15 @@ class ProgressStatus(object):
 def _get_backend_class(backend, class_name):
     # silence pyflakes, the _this_class var will be assigned in an exec() statement below...
     _this_class = None
-    if type(backend) not in (types.StringType, types.UnicodeType): return backend
+    if type(backend) not in (bytes, str): return backend
     backend = backend.upper()
-    available_backends = [ k for k in _BACKENDS[class_name].keys() if k != 'default' ]
+    available_backends = [ k for k in list(_BACKENDS[class_name].keys()) if k != 'default' ]
     # if for backend is given 'default' use the default backend module
     if backend == 'default': backend = _BACKENDS[class_name]['default']
     if backend in available_backends:
-        exec("from {backend} import {class_name} as _this_class".format(backend=_BACKENDS[class_name][backend], class_name=class_name))
+        namespace = {}
+        exec("from {backend} import {class_name} as _this_class".format(backend=_BACKENDS[class_name][backend], class_name=class_name), namespace)
+        _this_class = namespace['_this_class']
     else:
         raise x2go_exceptions.X2GoBackendException('unknown backend name %s for class %s' % (backend, class_name))
     return _this_class
@@ -847,21 +858,21 @@ def genkeypair(local_username, client_address, key_type='RSA'):
     """\
     Generate an SSH pub/priv key pair without writing the private key to file.
 
-    @param local_username: the key is for this user
-    @type local_username: C{unicode}
-    @param client_address: the key is only valid for this client
-    @type client_address: C{unicode}
-    @param key_type: either of: RSA, DSA
-    @type key_type: C{unicode}
+    :param local_username: the key is for this user
+    :type local_username: ``unicode``
+    :param client_address: the key is only valid for this client
+    :type client_address: ``unicode``
+    :param key_type: either of: RSA, DSA (Default value = 'RSA')
+    :type key_type: ``unicode``
 
     """
     key = None
     pubkey = None
 
     # generate key pair
-    if unicode(key_type) == u'RSA':
+    if str(key_type) == 'RSA':
         key = paramiko.RSAKey.generate(2048)
-    elif unicode(key_type) == u'DSA':
+    elif str(key_type) == 'DSA':
         key = paramiko.DSSKey.generate(1024)
 
     if key:
@@ -883,11 +894,10 @@ def which(basename):
     """\
     Python equivalent to the shell command "which".
 
-    @param basename: the basename of an application to be search for in $PATH
-    @type basename: C{str}
-
-    @return: full path to the application
-    @rtype: C{str}
+    :param basename: the basename of an application to be search for in $PATH
+    :type basename: ``str``
+    :returns: full path to the application
+    :rtype: ``str``
 
     """
     if _X2GOCLIENT_OS == 'Windows':
