@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -23,11 +23,13 @@ Python X2Go exceptions.
 """
 __NAME__ = 'x2goexceptions-pylib'
 
+__package__ = 'x2go'
+__name__    = 'x2go.x2go_exceptions'
+
 # modules
 import paramiko
-import exceptions
 
-from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
+from .defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 
 # Python X2Go Exceptions
 AuthenticationException = paramiko.AuthenticationException
@@ -39,7 +41,7 @@ BadHostKeyException = paramiko.BadHostKeyException
 SSHException = paramiko.SSHException
 """inherited from Python Paramiko library"""
 
-class _X2GoException(exceptions.BaseException): pass
+class _X2GoException(BaseException): pass
 class X2GoClientException(_X2GoException): pass
 class X2GoClientPrintingException(_X2GoException): pass
 class X2GoClientSettingsException(_X2GoException): pass
