@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (C) 2010-2016 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
+# Copyright (C) 2010-2018 by Mike Gabriel <mike.gabriel@das-netzwerkteam.de>
 #
 # Python X2Go is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -25,7 +25,10 @@
 
 __NAME__ = 'x2goxserver-pylib'
 
-from defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
+__package__ = 'x2go'
+__name__    = 'x2go.xserver'
+
+from .defaults import X2GOCLIENT_OS as _X2GOCLIENT_OS
 if _X2GOCLIENT_OS == 'Windows':
     import wmi
     import win32process
@@ -37,11 +40,11 @@ import gevent
 import copy
 
 # Python X2Go modules
-import log
-from defaults import X2GO_XCONFIG_CONFIGFILES as _X2GO_XCONFIG_CONFIGFILES
-from defaults import X2GO_CLIENTXCONFIG_DEFAULTS as _X2GO_CLIENTXCONFIG_DEFAULTS
-import inifiles
-import utils
+from .import log
+from .defaults import X2GO_XCONFIG_CONFIGFILES as _X2GO_XCONFIG_CONFIGFILES
+from .defaults import X2GO_CLIENTXCONFIG_DEFAULTS as _X2GO_CLIENTXCONFIG_DEFAULTS
+from . import inifiles
+from . import utils
 
 class X2GoClientXConfig(inifiles.X2GoIniFile):
     """\
@@ -49,37 +52,38 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
 
     This class is needed for Windows systems and (maybe soon) for Unix desktops using Wayland.
 
+
     """
 
     def __init__(self, config_files=_X2GO_XCONFIG_CONFIGFILES, defaults=_X2GO_CLIENTXCONFIG_DEFAULTS, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
-        Constructs an L{X2GoClientXConfig} instance. This is normally done by an L{X2GoClient} instance.
-        You can retrieve this L{X2GoClientXConfig} instance with the C{X2GoClient.get_client_xconfig()}
+        Constructs an :class:`x2go.xserver.X2GoClientXConfig` instance. This is normally done by an :class:`x2go.client.X2GoClient` instance.
+        You can retrieve this :class:`x2go.xserver.X2GoClientXConfig` instance with the :func:`X2GoClient.get_client_xconfig() <x2go.client.X2GoClient.get_client_xconfig()>`
         method.
 
-        On construction the L{X2GoClientXConfig} instance is filled with values from the configuration files::
+        On construction the :class:`x2go.xserver.X2GoClientXConfig` instance is filled with values from the configuration files::
 
             /etc/x2goclient/xconfig
             ~/.x2goclient/xconfig
 
         The files are read in the specified order and config options of both files are merged. Options
-        set in the user configuration file (C{~/.x2goclient/xconfig}) override global options set in
-        C{/etc/x2goclient/xconfig}.
+        set in the user configuration file (``~/.x2goclient/xconfig``) override global options set in
+        ``/etc/x2goclient/xconfig``.
 
-        @param config_files: a list of configuration file names
-        @type config_files: C{list}
-        @param defaults: a Python dictionary with configuration file defaults (use on your own risk)
-        @type defaults: C{dict}
-        @param logger: you can pass an L{X2GoLogger} object to the L{X2GoClientXConfig} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param config_files: a list of configuration file names
+        :type config_files: ``list``
+        :param defaults: a Python dictionary with configuration file defaults (use on your own risk)
+        :type defaults: ``dict``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the :class:`x2go.xserver.X2GoClientXConfig` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if _X2GOCLIENT_OS not in ("Windows"):
             import exceptions
-            class OSNotSupportedException(exceptions.StandardError): pass
+            class OSNotSupportedException(exceptions.Exception): pass
             raise OSNotSupportedException('classes of x2go.xserver module are for Windows only')
 
         inifiles.X2GoIniFile.__init__(self, config_files, defaults=defaults, logger=logger, loglevel=loglevel)
@@ -95,11 +99,13 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
         """\
         Store the Xserver configuration to the storage backend (i.e. on disk).
 
-        For writing the first of the C{config_files} specified on instance construction
+        For writing the first of the ``config_files`` specified on instance construction
         that is writable will be used.
 
-        @return: C{True} if the user config file has been successfully written, C{False} otherwise.
-        @rtype: C{bool}
+
+        :returns: ``True`` if the user config file has been successfully written, ``False`` otherwise.
+
+        :rtype: ``bool``
 
         """
         self._write_user_config = self.write_user_config
@@ -109,11 +115,10 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
         """\
         Retrieve the XServer configuration (from the xconfig file) for the given XServer application.
 
-        @param xserver_name: name of the XServer application
-        @type xserver_name: C{str}
-
-        @return: A Python dictionary containing the XServer's configuration settings
-        @rtype: C{list}
+        :param xserver_name: name of the XServer application
+        :type xserver_name: ``str``
+        :returns: A Python dictionary containing the XServer's configuration settings
+        :rtype: ``list``
 
         """
         _xserver_config = {}
@@ -129,6 +134,7 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
         """\
         Renders a list of XServers that are known to Python X2Go.
 
+
         """
         return self.get_value('XServers', 'known_xservers')
 
@@ -138,6 +144,7 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
         Among the known XServers renders a list of XServers that are actually
         installed on the system.
 
+
         """
         _installed = []
         for xserver_name in self.known_xservers:
@@ -150,6 +157,7 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
         """\
         Tries to render a list of running XServer processes from the system's process list.
 
+
         """
         _running = []
         _wmi = wmi.WMI()
@@ -168,7 +176,8 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
     def xserver_launch_possible(self):
         """\
         Detect if there is an XServer (that is known to Python X2Go) installed on the system.
-        Equals C{True} if we have found an installed XServer that we can launch.
+        Equals ``True`` if we have found an installed XServer that we can launch.
+
 
         """
         return bool(self.installed_xservers)
@@ -177,9 +186,10 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
     def xserver_launch_needed(self):
         """\
         Detect if an XServer launch is really needed (or if we use an already running XServer instance).
-        Equals C{True} if we have to launch an XServer before we can start/resume
+        Equals ``True`` if we have to launch an XServer before we can start/resume
         X2Go sessions.
 
+
         """
         return not bool(self.running_xservers)
 
@@ -188,8 +198,10 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
         """\
         Returns a tuple of (<xserver_name>, <xserver_config>).
 
-        return: (<xserver_name>, <xserver_config>)
-        rtype: C{tuple}
+
+        :returns: xserver_name>, <xserver_config>)
+
+        :rtype: ``tuple``
 
         """
         if self.xserver_launch_possible:
@@ -202,6 +214,7 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
         """\
         Returns the list of preferred XServer names (most preferred first).
 
+
         """
         return self.installed_xservers
 
@@ -210,8 +223,8 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
         Get an unused TCP/IP port for the to-be-launched X server and write it
         to the user's X configuration file.
 
-        @param xserver_name: name of the XServer application
-        @type xserver_name: C{str}
+        :param xserver_name: name of the XServer application
+        :type xserver_name: ``str``
 
         """
         _default_display = self.get_xserver_config(xserver_name)['display']
@@ -258,31 +271,32 @@ class X2GoClientXConfig(inifiles.X2GoIniFile):
 
 
 class X2GoXServer(threading.Thread):
-    """
+    """\
     This class is responsible for starting/stopping an external XServer application.
 
     X2Go applications require a running XServer on the client system. This class will
     manage/handle the XServer while your X2Go application is running.
 
+
     """
     def __init__(self, xserver_name, xserver_config, logger=None, loglevel=log.loglevel_DEFAULT):
         """\
         Initialize an XServer thread.
 
-        @param xserver_name: name of the XServer to start (refer to the xconfig file for available names)
-        @type xserver_name: C{str}
-        @param xserver_config: XServer configuration node (as derived from L{X2GoClientXConfig.get_xserver_config()}
-        @type xserver_config: C{dict}
-        @param logger: you can pass an L{X2GoLogger} object to the L{X2GoClientXConfig} constructor
-        @type logger: C{obj}
-        @param loglevel: if no L{X2GoLogger} object has been supplied a new one will be
+        :param xserver_name: name of the XServer to start (refer to the xconfig file for available names)
+        :type xserver_name: ``str``
+        :param xserver_config: XServer configuration node (as derived from :func:`X2GoClientXConfig.get_xserver_config() <x2go.xserver.X2GoClientXConfig.get_xserver_config()>`
+        :type xserver_config: ``dict``
+        :param logger: you can pass an :class:`x2go.log.X2GoLogger` object to the :class:`x2go.xserver.X2GoClientXConfig` constructor
+        :type logger: ``obj``
+        :param loglevel: if no :class:`x2go.log.X2GoLogger` object has been supplied a new one will be
             constructed with the given loglevel
-        @type loglevel: C{int}
+        :type loglevel: ``int``
 
         """
         if _X2GOCLIENT_OS not in ("Windows"):
             import exceptions
-            class OSNotSupportedException(exceptions.StandardError): pass
+            class OSNotSupportedException(exceptions.Exception): pass
             raise OSNotSupportedException('classes of x2go.xserver module are for Windows only')
 
         if logger is None:
@@ -297,7 +311,7 @@ class X2GoXServer(threading.Thread):
         self.xserver_config = xserver_config
         self.hProcess = None
 
-        if self.xserver_config.has_key('last_display'):
+        if 'last_display' in self.xserver_config:
 
             self.logger('setting DISPLAY environment variable to %s' % self.xserver_config['last_display'], loglevel=log.loglevel_NOTICE)
             os.environ.update({'DISPLAY': str(self.xserver_config['last_display'])})
@@ -314,7 +328,8 @@ class X2GoXServer(threading.Thread):
 
     def run(self):
         """\
-        Start this L{X2GoXServer} thread. This will launch the configured XServer application.
+        Start this :class:`x2go.xserver.X2GoXServer` thread. This will launch the configured XServer application.
+
 
         """
         self._keepalive = True
@@ -345,6 +360,7 @@ class X2GoXServer(threading.Thread):
         """\
         Terminate the runnint XServer process.
 
+
         """
         self.logger('terminating running XServer ,,%s\'\'' % self.xserver_name, loglevel=log.loglevel_DEBUG)
 
@@ -358,7 +374,7 @@ class X2GoXServer(threading.Thread):
         """\
         A call to this method will stop the XServer application and do a cleanup afterwards.
 
+
         """
         self._keepalive = False
         self.logger('stop_thread() method has been called', loglevel=log.loglevel_DEBUG)
-
